function getUserLocation(optionArr) {

    var _self = this;

    this.properties = {
        // Process settings
        gl_alert: 'Geolocation is not supported by this browser.',
        gl_alert_storage: 'Sorry! No Web Storage support..',
        gl_output: 'info_block',
        gl_test: false,
        gl_name: 'geolocation',
        expires: 0,
        path: '/',
        domain: '',
        secure: false,
        errorObj: new Errors()
    };

    var glTest = function (latitude, longitude) {
        var img = new Image();
        img.src = "http://maps.googleapis.com/maps/api/staticmap?center="
                + latitude + "," + longitude + "&zoom=13&size=300x300&sensor=false";
        $('#' + _self.properties.gl_output).append(img);
        $('#' + _self.properties.gl_output).show();
    };

    var storageIsAvailable = function () {
        return typeof (Storage) !== "undefined";
    };

    this.geolocationIsAvailable = function () {
        return "geolocation" in navigator;
    };

    this.resetUserData = function () {
        localStorage.removeItem('userLocation');
        localStorage.removeItem('isNewLocation');
        document.cookie = _self.properties.gl_name + '=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
        return true;
    };

    this.savePoint = function (point) {
        if (localStorage.getItem('userLocation') === point) {
            return true;
        }
        localStorage.setItem("userLocation", point);
        localStorage.setItem("isNewLocation", 1);
        _self.callSuccess(point, function () {
            localStorage.setItem("isNewLocation", 0);
        });
    };

    this.getCurrentPosition = function () {
        navigator.geolocation.getCurrentPosition(function (position) {
            var latitude = position.coords.latitude;
            var longitude = position.coords.longitude;
            if (_self.properties.gl_test) {
                glTest(latitude, longitude);
            }
            if (!storageIsAvailable()) {
                _self.properties.errorObj.show_error_block(_self.properties.gl_alert_storage, 'error');
                return false;
            } else {
                _self.savePoint(latitude + ";" + longitude);
            }
        });
    };

    this.Init = function (options) {
        _self.properties = $.extend(_self.properties, options);

        if (!_self.geolocationIsAvailable()) {
            _self.properties.errorObj.show_error_block(_self.properties.gl_alert, 'error');
            return false;
        }

        if (!getCookie()) {
            setCookie();
            if (typeof (geo_success) !== 'undefined' && geo_success.length > 0) {
                _self.getCurrentPosition();
            }
        } else if (storageIsAvailable() && localStorage.getItem('isNewLocation') == 1) {
            _self.callSuccess(localStorage.getItem('userLocation'), function () {
                localStorage.setItem('isNewLocation', 0);
            });
        }
    };

    this.callSuccess = function (point, callback) {
        if (geo_success.length > 0) {
            geo_success.pop()(point, function () {
                _self.callSuccess(point, callback);
            });
        } else {
            callback();
        }
    };

    var getCookie = function () {
        var matches = document.cookie.match(new RegExp("(?:^|; )" + _self.properties.gl_name.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + "=([^;]*)"));
        return matches ? decodeURIComponent(matches[1]) : undefined;
    };

    var setCookie = function () {
        var cookie = _self.properties.gl_name + "=1";
        if (_self.properties.expires) {
            cookie += ';expires=' + _self.properties.expires;
        }
        if (_self.properties.secure) {
            cookie += ';path=' + _self.properties.path;
        }
        if (_self.properties.secure) {
            cookie += ';domain=' + _self.properties.domain;
        }
        if (_self.properties.secure) {
            cookie += ';secure=true';
        }
        document.cookie = cookie;
    };

    this.clear = function ()
    {
        localStorage.removeItem('userLocation');
        localStorage.removeItem('isNewLocation');
        document.cookie = 'geolocation=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
        document.cookie = 'lang_id=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
        document.cookie = 'user_no_change_location=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
        document.cookie = 'id_city=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
        document.cookie = 'id_region=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
        document.cookie = 'id_country=;expires=Thu, 01 Jan 1970 00:00:01 GMT;';
    };

    _self.Init(optionArr);
}

geo = new getUserLocation();
