<?php

/**
 * Countries module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Countries management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Countries
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('country_select')) {
    /**
     * Return country selector block
     *
     * @param string  $select_type       selector type
     * @param string  $id_country        country code
     * @param integer $id_region         region identifier
     * @param integer $id_city           city identifier
     * @param string  $var_country_name  country variable
     * @param string  $var_region_name   region variable
     * @param string  $var_city_name     city variable
     * @param string  $var_js_name       javascript variable
     * @param string  $id_district       district identifier
     * @param string  $var_district_name district variable
     *
     * @return string
     */
    function country_select($select_type = 'city', $id_country = '', $id_region = '', $id_city = '', $var_country_name = 'id_country', $var_region_name = 'id_region', $var_city_name = 'id_city', $var_js_name = '', $id_district = '', $var_district_name = '', $is_edit = false)
    {
        $ci = &get_instance();
        $ci->load->model("Countries_model");

        if (!empty($id_country)) {
            $data["country"] = $ci->Countries_model->get_country($id_country);
        }
        if (!empty($id_region)) {
            $data["region"] = $ci->Countries_model->get_region($id_region);
        }
        if (!empty($id_city)) {
            $data["city"] = $ci->Countries_model->get_city($id_city);
        }
        if (!empty($id_district)) {
            $data["district"] = $ci->Countries_model->get_district($id_district);
        }

        $data["var_country_name"] = $var_country_name ? $var_country_name : "id_country";
        $data["var_region_name"] = $var_region_name ? $var_region_name : "id_region";
        $data["var_city_name"] = $var_city_name ? $var_city_name : "id_city";
        $data["var_district_name"] = $var_district_name ? $var_district_name : "id_district";
        $data["select_type"] = $select_type ? $select_type : "city";
        $data["var_js_name"] = $var_js_name;

        if ($data['select_type'] == 'city') {
            $use_district = $ci->pg_module->get_module_config('countries', 'use_district');
            if ($use_district) {
                $data['select_type'] = 'district';
            }
        }

        $data["rand"] = rand(100000, 999999);

        $ci->view->assign('country_helper_data', $data);
        $ci->view->assign('is_edit', $is_edit);

        return $ci->view->fetch('helper_country_select', null, 'countries');
    }
}

if (!function_exists('country_input')) {
    /**
     * Return country input block
     *
     * @param string  $select_type       selector type
     * @param string  $id_country        country code
     * @param integer $id_region         region identifier
     * @param integer $id_city           city identifier
     * @param string  $var_country_name  country variable
     * @param string  $var_region_name   region variable
     * @param string  $var_city_name     city variable
     * @param string  $var_js_name       javascript variable
     * @param string  $placeholder       input placeholder
     * @param string  $id_district       district identifier
     * @param string  $var_district_name district variable
     *
     * @return string
     */
    function country_input($select_type = 'city', $id_country = '', $id_region = '', $id_city = '', $var_country_name = 'id_country', $var_region_name = 'id_region', $var_city_name = 'id_city', $var_js_name = '', $placeholder = '', $id_district = '', $var_district_name = '')
    {
        $ci = &get_instance();
        $ci->load->model("Countries_model");

        if (!empty($id_country)) {
            $data["country"] = $ci->Countries_model->get_country($id_country);
        }
        if (!empty($id_region)) {
            $data["region"] = $ci->Countries_model->get_region($id_region);
        }
        if (!empty($id_city)) {
            $data["city"] = $ci->Countries_model->get_city($id_city);
        }
        if (!empty($id_district)) {
            $data["district"] = $ci->Countries_model->get_district($id_district);
        }

        $data["var_country_name"] = $var_country_name ? $var_country_name : "id_country";
        $data["var_region_name"] = $var_region_name ? $var_region_name : "id_region";
        $data["var_city_name"] = $var_city_name ? $var_city_name : "id_city";
        $data["var_district_name"] = $var_district_name ? $var_district_name : "id_district";
        $data["select_type"] = $select_type ? $select_type : "city";
        $data["var_js_name"] = $var_js_name;
        $data["placeholder"] = $placeholder;

        if (!empty($data["district"]["name"])) {
            $data["location_text"] = $data["district"]["name"];
        } elseif (!empty($data["city"]["name"])) {
            $data["location_text"] = $data["city"]["name"];
        } elseif (!empty($data["region"]["name"])) {
            $data["location_text"] = $data["region"]["name"];
        } elseif (!empty($data["country"]["name"])) {
            $data["location_text"] = $data["country"]["name"];
        }

        if ($data['select_type'] == 'city') {
            $use_district = $ci->pg_module->get_module_config('countries', 'use_district');
            if ($use_district) {
                $data['select_type'] = 'district';
            }
        }

        $data["rand"] = rand(100000, 999999);

        $ci->view->assign('country_helper_data', $data);

        return $ci->view->fetch('helper_country_input', null, 'countries');
    }
}
if (!function_exists('with_map_country_input')) {
    /**
     * Return country input block
     *
     * @param string  $select_type       selector type
     * @param string  $id_country        country code
     * @param integer $id_region         region identifier
     * @param integer $id_city           city identifier
     * @param string  $var_country_name  country variable
     * @param string  $var_region_name   region variable
     * @param string  $var_city_name     city variable
     * @param string  $var_js_name       javascript variable
     * @param string  $placeholder       input placeholder
     * @param string  $id_district       district identifier
     * @param string  $var_district_name district variable
     *
     * @return string
     */
function with_map_country_input($select_type = 'city', $id_country = '', $id_region = '', $id_city = '', $var_country_name = 'id_country', $var_region_name = 'id_region', $var_city_name = 'id_city', $var_js_name = '', $placeholder = '', $id_district = '', $var_district_name = '')
    {
        $ci = &get_instance();
        $ci->load->model("Countries_model");

        if (!empty($id_country)) {
            $data["country"] = $ci->Countries_model->get_country($id_country);
        }
        if (!empty($id_region)) {
            $data["region"] = $ci->Countries_model->get_region($id_region);
        }
        if (!empty($id_city)) {
            $data["city"] = $ci->Countries_model->get_city($id_city);
        }
        if (!empty($id_district)) {
            $data["district"] = $ci->Countries_model->get_district($id_district);
        }

        $data["var_country_name"] = $var_country_name ? $var_country_name : "id_country";
        $data["var_region_name"] = $var_region_name ? $var_region_name : "id_region";
        $data["var_city_name"] = $var_city_name ? $var_city_name : "id_city";
        $data["var_district_name"] = $var_district_name ? $var_district_name : "id_district";
        $data["select_type"] = $select_type ? $select_type : "city";
        $data["var_js_name"] = $var_js_name;
        $data["placeholder"] = $placeholder;

        if (!empty($data["district"]["name"])) {
            $data["location_text"] = $data["district"]["name"];
        } elseif (!empty($data["city"]["name"])) {
            $data["location_text"] = $data["city"]["name"];
        } elseif (!empty($data["region"]["name"])) {
            $data["location_text"] = $data["region"]["name"];
        } elseif (!empty($data["country"]["name"])) {
            $data["location_text"] = $data["country"]["name"];
        }

        if ($data['select_type'] == 'city') {
            $use_district = $ci->pg_module->get_module_config('countries', 'use_district');
            if ($use_district) {
                $data['select_type'] = 'district';
            }
        }

        $data["rand"] = rand(100000, 999999);

        $ci->view->assign('country_helper_data', $data);

        return $ci->view->fetch('helper_with_map_country_input', null, 'countries');
    }
}

if (!function_exists('country')) {
    /**
     * Return location output name
     *
     * @param string  $id_country  country code
     * @param integer $id_region   region identifier
     * @param integer $id_city     city idnetifier
     * @param integer $id_district district identifier
     *
     * @return string
     */
    function country($id_country = '', $id_region = '', $id_city = '', $id_district = '')
    {
        $ci = &get_instance();
        $ci->load->model("Countries_model");
        if (!empty($id_country)) {
            $data["country"] = $ci->Countries_model->get_country($id_country);
            $return_array[] = $data["country"]["name"];
        }
        if (!empty($id_region)) {
            $data["region"] = $ci->Countries_model->get_region($id_region);
            $return_array[] = $data["region"]["name"];
        }
        if (!empty($id_city)) {
            $data["city"] = $ci->Countries_model->get_city($id_city);
            $return_array[] = $data["city"]["name"];
        }
        if (!empty($id_district)) {
            $data["district"] = $ci->Countries_model->get_district($id_district);
            $return_array[] = $data["district"]["name"];
        }
        $return = (is_array($return_array)) ? implode(', ', $return_array) : '';

        return $return;
    }
}

/*
 * Data is array( id => array(id_country, id_region, id_city), id => array(id_country, id_region, id_city), .....)
 * return array(id => str, id => str, id => str...);
 */
if (!function_exists('countries_output_format')) {
    /**
     * Return locations output names in country format
     *
     * @param array $data locations data
     *
     * @return string
     */
    function countries_output_format($data)
    {
        if (empty($data)) {
            return array();
        }
        $ci = &get_instance();
        $location_data = get_location_data($data, 'country');
        $country_template = $ci->pg_module->get_module_config('countries', 'output_country_format');

        $return = array();
        foreach ($data as $id => $location) {
            if (isset($location['country']) && isset($location_data["country"][$location['country']])) {
                $str = str_replace("[country]", $location_data["country"][$location['country']]["name"], $country_template);
                $str = str_replace('[country_code]', $location['country'], $str);
            } else {
                $str = "";
            }
            $return[$id] = $str;
        }

        return $return;
    }
}

if (!function_exists('regions_output_format')) {
    /**
     * Return locations output names in region format
     *
     * @param array $data locations data
     *
     * @return string
     */
    function regions_output_format($data)
    {
        if (empty($data)) {
            return array();
        }
        $ci = &get_instance();
        $location_data = get_location_data($data, 'region');
        $country_template = $ci->pg_module->get_module_config('countries', 'output_country_format');
        $region_template = $ci->pg_module->get_module_config('countries', 'output_region_format');

        $return = array();
        foreach ($data as $id => $location) {
            $template = $country = $country_code = $region = '';
            if (isset($location['country']) && !empty($location_data["country"][$location['country']])) {
                $country = $location_data["country"][$location['country']]["name"];
                $country_code = $location['country'];
                $template = $country_template;
            }

            if (isset($location['region']) && !empty($location_data["region"][$location['region']])) {
                $region = $location_data["region"][$location['region']]["name"];
                $template = $region_template;
            }

            if ($template) {
                $template = str_replace("[country]", $country, $template);
                $template = str_replace("[country_code]", $country_code, $template);
                $template = str_replace("[region]", $region, $template);
            }
            $return[$id] = $template;
        }

        return $return;
    }
}

if (!function_exists('cities_output_format')) {
    /**
     * Return locations output names in cities format
     *
     * @param array $data locations data
     *
     * @return string
     */
    function cities_output_format($data)
    {
        if (empty($data)) {
            return array();
        }
        $ci = &get_instance();
        $location_data = get_location_data($data, 'city');
        $country_template = $ci->pg_module->get_module_config('countries', 'output_country_format');
        $region_template = $ci->pg_module->get_module_config('countries', 'output_region_format');
        $city_template = $ci->pg_module->get_module_config('countries', 'output_city_format');

        $return = array();
        foreach ($data as $id => $location) {
            $template = $country = $country_code = $region = $city = '';
            if (isset($location['country']) && !empty($location_data["country"][$location['country']])) {
                $country = $location_data["country"][$location['country']]["name"];
                $country_code = $location['country'];
                $template = $country_template;
            }

            if (isset($location['region']) && !empty($location_data["region"][$location['region']])) {
                $region = $location_data["region"][$location['region']]["name"];
                $template = $region_template;
            }

            if (isset($location['city']) && !empty($location_data["city"][$location['city']])) {
                $city = $location_data["city"][$location['city']]["name"];
                $template = $city_template;
            }

            if ($template) {
                $template = str_replace("[country]", $country, $template);
                $template = str_replace("[country_code]", $country_code, $template);
                $template = str_replace("[region]", $region, $template);
                $template = str_replace("[city]", $city, $template);
            }
            $return[$id] = $template;
        }

        return $return;
    }
}

if (!function_exists('districts_output_format')) {
    /**
     * Return locations output names in district format
     *
     * @param array $data locations data
     *
     * @return array
     */
    function districts_output_format($data)
    {
        if (empty($data)) {
            return array();
        }

        $ci = &get_instance();

        $use_district = $ci->pg_module->get_module_config('countries', 'use_district');
        if (!$use_district) {
            return cities_output_format($data);
        }

        $location_data = get_location_data($data, 'district');
        $country_template = $ci->pg_module->get_module_config('countries', 'output_country_format');
        $region_template = $ci->pg_module->get_module_config('countries', 'output_region_format');
        $city_template = $ci->pg_module->get_module_config('countries', 'output_city_format');
        $districts_template = $ci->pg_module->get_module_config('countries', 'output_district_format');

        $return = array();
        foreach ($data as $id => $location) {
            $template = $country = $country_code = $region = $city = '';
            if (isset($location['country']) && !empty($location_data["country"][$location['country']])) {
                $country = $location_data["country"][$location['country']]["name"];
                $country_code = $location['country'];
                $template = $country_template;
            }

            if (isset($location['region']) && !empty($location_data["region"][$location['region']])) {
                $region = $location_data["region"][$location['region']]["name"];
                $template = $region_template;
            }

            if (isset($location['city']) && !empty($location_data["city"][$location['city']])) {
                $city = $location_data["city"][$location['city']]["name"];
                $template = $city_template;
            }

            if (isset($location['district']) && !empty($location_data["district"][$location['district']])) {
                $district = $location_data["district"][$location['district']]["name"];
                $template = $districts_template;
            }

            if ($template) {
                $template = str_replace("[country]", $country, $template);
                $template = str_replace("[country_code]", $country_code, $template);
                $template = str_replace("[region]", $region, $template);
                $template = str_replace("[city]", $city, $template);
                $template = str_replace("[district]", $district, $template);
            }
            $return[$id] = $template;
        }

        return $return;
    }
}

if (!function_exists('address_output_format')) {
    /**
     * Return locations output names in address format
     *
     * @param array   $data      locations data
     * @param string  $format    use format
     * @param boolean $microdata add company microdata
     *
     * @return array
     */
    function address_output_format($data, $format = 'city', $microdata = false)
    {
        if (empty($data)) {
            return array();
        }
        $ci = &get_instance();
        $location_data = get_location_data($data, $format);
        $country_template = $ci->pg_module->get_module_config('countries', 'output_country_format');
        $region_template = $ci->pg_module->get_module_config('countries', 'output_region_format');
        $city_template = $ci->pg_module->get_module_config('countries', 'output_city_format');
        $district_template = $ci->pg_module->get_module_config('countries', 'output_district_format');
        $address_template = $ci->pg_module->get_module_config('countries', 'output_address_format');

        $return = array();
        foreach ($data as $id => $location) {
            $template = $country = $country_code = $region = $city = '';
            if (isset($location['country']) && !empty($location_data["country"][$location['country']])) {
                $country = $location_data["country"][$location['country']]["name"];
                $country_code = $location['country'];
                $template = $country_template;
            }

            if (isset($location['region']) && !empty($location_data["region"][$location['region']])) {
                $region = $location_data["region"][$location['region']]["name"];
                $template = $region_template;
            }

            if (isset($location['city']) && !empty($location_data["city"][$location['city']])) {
                $city = $location_data["city"][$location['city']]["name"];
                $template = $city_template;
            }

            if (isset($location['district']) && !empty($location_data["district"][$location['district']])) {
                $use_district = $ci->pg_module->get_module_config('countries', 'use_district');
                if ($use_district) {
                    $district = $location_data["district"][$location['district']]["name"];
                    $template = $district_template;
                } else {
                    $district = '';
                }
            } else {
                $district = '';
            }

            if (!isset($location['address'])) {
                $location['address'] = '';
            }
            if (!isset($location['zip_code'])) {
                $location['zip_code'] = '';
            }

            if (!empty($location['address']) || !empty($location['zip_code'])) {
                $template = $address_template;
            }

            if ($microdata) {
                $country = '<span itemprop="country-name">' . $country . '</span>';
                $region = '<span itemprop="region">' . $region . '</span>';
                $city = '<span itemprop="locality">' . $city . '</span>';
                $location['address'] = '<span itemprop="street-address">' . $location['address'] . '</span>';
                $location['zip_code'] = '<span itemprop="postal-code">' . $location['zip_code'] . '</span>';
            }

            if ($template) {
                $template = str_replace("[country]", $country, $template);
                $template = str_replace("[country_code]", $country_code, $template);
                $template = str_replace("[region]", $region, $template);
                $template = str_replace("[city]", $city, $template);
                $template = str_replace("[district]", $district, $template);
                $template = str_replace("[address]", $location['address'], $template);
                $template = str_replace("[zip_code]", $location['zip_code'], $template);
            }
            $return[$id] = preg_replace('/^(\s)*,(\s)*|(\s)*,(\s)*$/', '', preg_replace('/,(\s)*,/', ',', $template));
        }

        return $return;
    }
}

if (!function_exists('get_location_data')) {
    /**
     * Return location data from identifiers
     *
     * @param array  $data location identifiers
     * @param string $type max level data type
     *
     * @return array
     */
    function get_location_data($data, $type = 'city')
    {
        $ci = &get_instance();
        $ci->load->model("Countries_model");

        if (empty($data)) {
            return array();
        }

        $use_district = $ci->pg_module->get_module_config('countries', 'use_district');

        $return = $country_ids = $region_ids = $city_ids = $district_ids = array();
        foreach ($data as $set) {
            // country
            if (isset($set['country']) && !empty($set['country']) && !in_array($set['country'], $country_ids)) {
                $country_ids[] = $set['country'];
            }

            if ($type != 'country') {
                // region
                if (isset($set['region']) && !empty($set['region']) && !in_array($set['region'], $region_ids)) {
                    $region_ids[] = $set['region'];
                }

                if ($type != 'region') {
                    // city
                    if (isset($set['city']) && !empty($set['city']) && !in_array($set['city'], $city_ids)) {
                        $city_ids[] = $set['city'];
                    }

                    if ($type != 'city') {
                        // district
                        if (isset($set['district']) && !empty($set['district']) && !in_array($set['district'], $district_ids)) {
                            $district_ids[] = $set['district'];
                        }
                    }
                }
            }
        }

        if (!empty($country_ids)) {
            $return["country"] = $ci->Countries_model->get_countries_by_code($country_ids);
        }

        if (!empty($region_ids)) {
            $return["region"] = $ci->Countries_model->get_regions_by_id($region_ids);
        }

        if (!empty($city_ids)) {
            $return["city"] = $ci->Countries_model->get_cities_by_id($city_ids);
        }

        if (!empty($district_ids)) {
            $return["district"] = $ci->Countries_model->get_districts_by_id($district_ids);
        }

        return $return;
    }
}
