<?php

use Pg\Modules\Country_identification\Models\Country_identification_model;

/**
 * Country_identification module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Country_identification helpers
 *
 * @package     PG_RealEstate
 * @subpackage  Country_identification
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('admin_country_identification_settings')) {

    function admin_country_identification_settings($params = array())
    {
        $ci = &get_instance();
        $module_gid = Country_identification_model::MODULE_GID;
        $ci->view->assign('action', site_url() . "admin/" . $module_gid . "/ajax_settings");
        $ci->view->assign('automatic_identification', $ci->pg_module->get_module_config($module_gid, 'automatic_identification'));
        return $ci->view->fetch('helper_country_identification_settings', 'admin', $module_gid);
    }

}

if (!function_exists('user_country_identification')) {

    function user_country_identification($params = array())
    {
        $ci = &get_instance();
        $module_gid = Country_identification_model::MODULE_GID;
        $ci->view->assign('action', site_url() . $module_gid);
        return $ci->view->fetch('helper_country_identification_user', 'user', $module_gid);
    }

}

if (!function_exists('country_identification_selector')) {

    function country_identification_selector($params = array())
    {
        $ci = &get_instance();

        if (!empty($params['city'])) {
            $city = $params['city'];
        } else {
            $city = array();
        }

        if (isset($params['is_container'])) {
            $is_container = $params['is_container'];
        } else {
            $is_container = true;
        }

        if (isset($params['mobile'])) {
            $ci->view->assign('is_mobile_containter', '1');
            $is_container = false;
        }

        $user_request_change_location = false;
        $guest_request_change_location = false;

        $user_db_city = array();

        $ci->load->model('Countries_model');

        $current_country = $current_region = array();

        if ($ci->session->userdata('auth_type') === 'user') {
            $ci->load->model('Users_model');
            $user = $ci->Users_model->get_user_by_id($ci->session->userdata('user_id'));
            if (!empty($user)) {
                if ((!empty($current_country['code']) && ($user['id_country'] != $current_country['code']) || 
                    (!empty($city['id']) && $user['id_city'] != $city['id'])) && !$ci->session->userdata('user_no_change_location')) {
                    $user_db_city = $ci->Countries_model->get_city($user['id_city']);
                    $user_request_change_location = true;
                } else if (!empty($user['id_city'])) {
                    $city = $ci->Countries_model->get_city($user['id_city']);
                }
            }
        } else {
            $ci->load->helper('cookie');
            if (!get_cookie('user_no_change_location')) {
                $guest_request_change_location = true;
            } else {
                $city = $ci->Countries_model->get_city(intval(get_cookie('id_city')));
            }
        }

        if (!empty($city['country_code'])) {
            $current_country = $ci->Countries_model->get_country($city['country_code']);
        }

        $countries = $ci->Countries_model->get_countries(1);
        if (empty($current_country) && !empty($countries)) {
            $current_country = current($countries);
        }

        $country_regions = $ci->Countries_model->get_regions($current_country['code']);

        if (!empty($city['id_region'])) {
            $current_region = $ci->Countries_model->get_region($city['id_region']);
        }

        if (empty($current_region) && !empty($country_regions)) {
            $current_region = current($country_regions);
        }

        $country_cities = $ci->Countries_model->get_cities(
            null, null, array(), array(
                'where' => array(
                    'country_code' => $current_country['code'],
                    'id_region' => $current_region['id']
                )
            )
        );

        if (empty($city)) {
            if (empty($country_cities)) {
                return '';
            }
            $city = current($country_cities);
        }
        $ci->view->assign('user_request_change_location', $user_request_change_location);
        $ci->view->assign('user_db_city', $user_db_city);
        $ci->view->assign('guest_request_change_location', $guest_request_change_location);
        $ci->view->assign('current_country', $current_country);
        $ci->view->assign('current_region', $current_region);
        $ci->view->assign('current_city', $city);
        $ci->view->assign('countries', $ci->Countries_model->get_countries());
        $ci->view->assign('regions', $country_regions);
        $ci->view->assign('cities', $country_cities);
        $ci->view->assign('action', 'country_identification/change_user_default_country');
        $ci->view->assign('is_containter', $is_container);

        return $ci->view->fetch('helper_user_country_block', 'user', 'country_identification');
    }

}
