<?php

namespace Pg\Modules\Field_editor\Models\Fields;

/**
 * Field editor module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Text field model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Field editor
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Text_field_model extends Field_type_model
{

    /**
     * Field definition
     *
     * @var array
     */
    public $base_field_param = array(
        'type' => 'VARCHAR',
        'constraint' => '255',
        'null' => false,
        'default' => '',
    );

    /**
     * Default value configuration
     *
     * @var array
     */
    public $manage_field_param = array(
        'default_value' => array('type' => 'string', "default" => ''),
        'min_char' => array('type' => 'int', 'min' => 0, 'max' => 255, "default" => 0),
        'max_char' => array('type' => 'int', 'min' => 1, 'max' => 255, "default" => 255),
        'template' => array('type' => 'string', 'options' => array('string', 'intval', 'floatval', 'email', 'url', 'price'), "default" => 'string'),
        'format' => array('type' => 'string', 'options' => array('none', 'year', 'price', 'url', 'facebook', 'twitter', 'vk', 'skype', 'whatsapp'), "default" => 'none'),
    );

    /**
     * Field settings of form
     *
     * @var array
     */
    public $form_field_settings = array(
        "search_type" => array("values" => array('text', 'number'), "default" => 'text'),
        "view_type" => array("values" => array('equal', 'range'), "default" => 'equal'),
    );

    /**
     * Validate file type
     *
     * @param array $settings_data settings data
     *
     * @return array
     */
    public function validate_field_type($settings_data)
    {
        $return = parent::validate_field_type($settings_data);

        $settings = $this->manage_field_param;
        if (!in_array($return["data"]["template"], $settings["template"]["options"])) {
            $return["data"]["template"] = $settings["template"]["default"];
        }
        if (!in_array($return["data"]["format"], $settings["format"]["options"])) {
            $return["data"]["format"] = $settings["format"]["default"];
        }
        if ($return["data"]["min_char"] < $settings["min_char"]["min"]) {
            $return["data"]["min_char"] = $settings["min_char"]["min"];
        }
        if ($return["data"]["min_char"] > $settings["min_char"]["max"]) {
            $return["data"]["min_char"] = $settings["min_char"]["max"];
        }
        if ($return["data"]["max_char"] < $settings["max_char"]["min"]) {
            $return["data"]["max_char"] = $settings["max_char"]["min"];
        }
        if ($return["data"]["max_char"] > $settings["max_char"]["max"]) {
            $return["data"]["max_char"] = $settings["max_char"]["max"];
        }

        return $return;
    }

    /**
     * Format field value for used in view
     *
     * @param array  $settings settings data
     * @param array  $field    field data
     * @param string $value    field value
     *
     * @return array
     */
    public function format_view_fields($settings, $field, $value)
    {
        $field["value_original"] = $value;
        $field['format'] = $settings["settings_data_array"]["format"];
        switch ($settings["settings_data_array"]["format"]) {
            case "price":
                $this->ci->load->helper('start');
                $field["value"] = currency_format_output(array('value' => sprintf("%01.2f", $value)));
                break;
            case "year":
                $field["value"] = sprintf("%4d", $value);
                break;
            case "none":
                $field["value"] = $value;
                break;
            case 'url':
            case 'facebook':
            case 'twitter':
            case 'vk':
                if (!empty($value)) {
                    $field["value"] = '<a href="' . (!preg_match('#^https?://#i', $value) ? 'http://' : '') . $value . '">' . $value . '</a>';
                } else {
                    $field["value"] = '';
                }
                break;
            case 'skype':
                if (!empty($value)) {
                    $field["value"] = '<a href="skype:' . urlencode($value) . '?call" target="_self">' . $value . '</a>';
                } else {
                    $field["value"] = '';
                }
                break;
            case 'whatsapp':
                if (!empty($value)) {
                    $field["value"] = $value;
                } else {
                    $field["value"] = '';
                }
                break;
            case 'currency':
                if (!empty($value)) {
                    $this->ci->load->helper('start');
                    $field['value'] = currency_output(array('value' => $value));
                } else {
                    $field['value'] = '';
                }
                break;
        }
        $field["value_str"] = $field["value"];

        return $field;
    }

    /**
     * Validate field data
     *
     * @param array  $settings settings data
     * @param string $value    field value
     *
     * @return array
     */
    public function validate_field($settings, $value)
    {
        $return = array("errors" => array(), "data" => $value);
        switch ($settings["settings_data_array"]["template"]) {
            case "string": $return["data"] = trim(strip_tags($return["data"]));
                break;
            case "intval": $return["data"] = intval($return["data"]);
                break;
            case "floatval": $return["data"] = floatval($return["data"]);
                break;
            case "email":
                $return["data"] = trim(strip_tags($return["data"]));
                if (!empty($return["data"])) {
                    $this->ci->config->load('reg_exps', true);
                    $email_expr = $this->ci->config->item('email', 'reg_exps');
                    if (!preg_match($email_expr, $return["data"])) {
                        $return["errors"][] = l('error_email_incorrect', 'field_editor');
                    }
                }
                break;
            case "url":
                $return["data"] = trim(strip_tags($return["data"]));
                if (!empty($return["data"])) {
                    $this->ci->config->load('reg_exps', true);
                    $url_expr = $this->ci->config->item('url', 'reg_exps');
                    if (!preg_match($url_expr, $return["data"])) {
                        $return["errors"][] = l('error_url_incorrect', 'field_editor');
                    }
                }
                break;
            case "price":
                $return["data"] = sprintf("%01.2f", $return["data"]);
                break;
        }
        $string_length = strlen($return["data"]);
        if ($settings["settings_data_array"]["min_char"] > $string_length) {
            $return["errors"][] = $settings['name'] . ': ' . str_replace("[length]", $settings["settings_data_array"]["min_char"], l('error_field_length_less_than', 'field_editor'));
        }
        if ($settings["settings_data_array"]["max_char"] < $string_length) {
            $return["errors"][] = $settings['name'] . ': ' . str_replace("[length]", $settings["settings_data_array"]["max_char"], l('error_field_length_more_than', 'field_editor'));
        }

        return $return;
    }

    /**
     * Return criteria for used in search by field
     *
     * @param array  $field    field data
     * @param array  $settings settings data
     * @param array  $data     search data
     * @param string $prefix   field name prefix
     *
     * @return array
     */
    public function get_search_field_criteria($field, $settings, $data, $prefix)
    {
        $criteria = array();
        $gid = $field['gid'];
        if ($settings["search_type"] == "text") {
            if (!empty($data[$gid])) {
                if ($settings["view_type"] == "equal") {
                    $criteria["where"][$prefix . $gid] = trim(strip_tags($data[$gid]));
                } else {
                    $criteria["where"][$prefix . $gid . " LIKE"] = "%" . trim(strip_tags($data[$gid])) . "%";
                }
            }
        } else {
            if ($settings["view_type"] == "equal") {
                if (!empty($data[$gid])) {
                    $data[$gid] = (!is_numeric($data[$gid])) ? floatval($data[$gid]) : $data[$gid];
                    $criteria["where"][$prefix . $gid] = $data[$gid];
                }
            } else {
                if (!empty($data[$gid . "_min"])) {
                    $data[$gid . "_min"] = (!is_numeric($data[$gid . "_min"])) ? floatval($data[$gid . "_min"]) : $data[$gid . "_min"];
                    $criteria["where"]["CONVERT(" . $prefix . $gid . ", DECIMAL) >="] = $data[$gid . "_min"];
                }
                if (!empty($data[$gid . "_max"])) {
                    $data[$gid . "_max"] = (!is_numeric($data[$gid . "_max"])) ? floatval($data[$gid . "_max"]) : $data[$gid . "_max"];
                    $criteria["where"]["CONVERT(" . $prefix . $gid . ", DECIMAL) <="] = $data[$gid . "_max"];
                }
            }
        }

        return $criteria;
    }

    /**
     * Save field option (not available)
     *
     * @param array  $field      field settings
     * @param string $option_gid option guid
     * @param string $data       option data
     *
     * @return void
     */
    public function set_field_option($field, $option_gid, $data)
    {
        return;
    }

    /**
     * Remove field option (not available)
     *
     * @param array  $field      field settings
     * @param string $option_gid option guid
     *
     * @return void
     */
    public function delete_field_option($field, $option_gid)
    {
        return;
    }

    /**
     * Resort field options (not available)
     *
     * @param array $field       field settings
     * @param array $sorter_data sorting data
     *
     * @return void
     */
    public function sorter_field_option($field, $sorter_data)
    {
        return;
    }

}
