<?php

namespace Pg\Modules\Geomap\Controllers;

/**
 * Geomap module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Geomap api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Geomap
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Geomap extends \Controller
{

    /**
     * Class constructor
     *
     * @return Api_Geomap
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('geomap/models/Geomap_settings_model');
    }

    /**
     * Save geomap settings for the user
     *
     * @param string $gid       map guid
     * @param int    $zoom      map zoom
     * @param int    $view_type map type
     * @param float  $lat       map center latitude
     * @param float  $lon       map center longitude
     *
     * @return void
     */
    public function save_settings()
    {
        $user_id = $this->session->userdata('user_id');
        if (!$user_id) {
            log_message('error', 'geomap API: Empty user id');
            $this->set_api_content('errors', l('error_cant_save_settings', 'geomap'));
            return false;
        }
        $gid = $this->input->post('gid');
        if (!$gid) {
            log_message('error', 'geomap API: Empty gid');
            $this->set_api_content('errors', l('error_cant_save_settings', 'geomap'));
            return false;
        }
        $object_id = $this->input->post('object_id');

        $post_data = array(
            'zoom' => intval($this->input->post('zoom')),
            'view_type' => intval($this->input->post('view_type')),
            'lat' => $this->input->post('lat', true),
            'lon' => $this->input->post('lon', true),
        );

        $validate_data = $this->Geomap_settings_model->validate_settings($post_data);
        if (!empty($validate_data['errors'])) {
            $this->set_api_content('errors', $validate_data['errors']);
        } else {
            $this->Geomap_settings_model->save_settings($gid, $user_id, $object_id, $gid, $validate_data['data']);
            $this->set_api_content('data', $post_data);
        }
        return;
    }

}
