<?php

namespace Pg\Modules\Geomap\Controllers;

/**
 * Geomap module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Geomap user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Geomap
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Geomap extends \Controller
{

    /**
     * Class constructor
     *
     * @return Geomap
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Geomap_model');
    }

    /**
     * Map example
     *
     * @return void
     */
    public function index()
    {
        $this->example();
    }

    /**
     * Save map settings by ajax
     *
     * @param integer $user_id   user identifier
     * @param integer $object_id map object identifier
     * @param string  $gid       map gid
     *
     * @return void
     */
    public function ajax_save_settings($user_id, $object_id, $gid = "")
    {
        $errors = array();

        if (empty($user_id)) {
            $errors[] = l('error_cant_save_settings', 'geomap');
        } else {
            $post_data = array(
                "zoom" => intval($this->input->post('zoom')),
                "view_type" => intval($this->input->post('view_type')),
                "lat" => $this->input->post('lat', true),
                "lon" => $this->input->post('lon', true),
            );

            $validate_data = $this->Geomap_settings_model->validate_settings($post_data);
            if (!empty($validate_data["errors"])) {
                $errors = $validate_data["errors"];
            } else {
                $this->Geomap_settings_model->save_settings($gid, $user_id, $object_id, $gid, $validate_data["data"]);
            }
        }

        if (!empty($errors)) {
            $return["errors"] = implode("<br>", $errors);
            $return["result"] = "error";
        } else {
            $return["result"] = "ok";
        }

        $this->view->assign($return);

        $this->view->render();

        return;
    }

    /**
     * Map example
     *
     * @return void
     */
    public function example()
    {
        $markers = [
            [
                'gid' => 'england',
                'lat' => '51.50874245880332',
                'lon' => '0',
                'html' => l('map_example_1_text', 'geomap', '', 'js'),
                "dragging" => true,
            ],
            [
                'gid' => 'france',
                'lat' => '48.86471476180277',
                'lon' => '2.3291015625',
                'html' => l('map_example_2_text', 'geomap', '', 'js'),
                'dragging' => true,
            ],
            [
                'gid' => 'italy',
                'lat' => '43.731414',
                'lon' => '10.404053',
                'html' => l('map_example_3_text', 'geomap', '', 'js'),
                'dragging' => true,
            ],
            [
                'gid' => 'germany',
                'lat' => '47.562859',
                'lon' => '10.750122',
                'html' => l('map_example_4_text', 'geomap', '', 'js'),
                'dragging' => true,
            ],
        ];

        $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
        $view_settings = array(
            "gid" => "example",
            "width" => "660",
            "height" => "250",
            "class" => "",
            "zoom_listener" => "test1",
            "type_listener" => "test2",
            "drag_listener" => "test3",
            "lang" => $current_language["code"],
        );
        $this->view->assign('user_id', $this->session->userdata('user_id'));
        $this->view->assign('map_settings', $view_settings);
        $this->view->assign('markers', $markers);
        $this->view->render('example');
    }

}
