var category;
var location_change_wait = 0;
var country_old = $("input[name='id_country']").attr('value');
var region_old = $("input[name='id_region']").attr('value');
var city_old = $("input[name='id_city']").attr('value');
var address_old = $('input[name=data\\[address\\]]').val();
var postal_code_old = $('input[name=data\\[zip\\]]').val();
var id;

function getMapId() {
    return $("div[class*='map_container']").attr('id');
}

function getMarker() {
    var lat = $('#lat').val();
    var lon = $('#lon').val();
    var country = $('input[name="id_country"]').attr('data-name');
    var region = $('input[name="id_region"]').attr('data-name');
    var city = $('input[name="id_city"]').attr('data-name');
    var address = $('input[name=data\\[address\\]]').val();
    var postal_code = $('input[name=data\\[zip\\]]').val();
    var dragging = true;
    var gid = 1;
    var info = null;

    if (country && city && address) {
        info = country + ', ' + city + ', ' + address;
    } else {
        if (!address) {
            info = country + ', ' + city;
        }
    }

    if (lat && lon && country && region && city) {
        marker = {
            lat: lat,
            lon: lon,
            country: country,
            region: region,
            city: city,
            address: address,
            postal_code: postal_code,
            dragging: dragging,
            gid: gid,
            info: info
        };
    } else {
        return 0;
    }


    return marker;
}

function getSettings(setting_id) {
    var settings = $("#" + setting_id).attr('value');
    if (settings) {
        settings = JSON.parse(settings);
    }
    settings.drag_listener = get_listing_drag_data;
    return settings;
}


function updateMap(marker, id_map) {
    id = id_map.replace('map_', '');
    var map = id_map;
    var map_settings = getSettings('map_settings_data');
    var listing_settings = getSettings('listing_map_settings_data');
    var zoom = 15;
    if (listing_settings) {
        listing_zoom = parseInt(listing_settings.zoom);
        if (listing_zoom) {
            zoom = listing_zoom;
        } else {
            $('#map_zoom').attr('value', zoom);
        }  
    }
    
    var options = {};
    options.info = marker.info;
    options.draggable = true;
    options.drag_listener = map_settings.drag_listener;


    if(this[map].markers[0]) {
        this[map].moveMarker(this[map].markers[0].getGUID(), marker['lat'], marker['lon']);
        this[map].markers[0].prop.info = marker.info;

        if (this[map].markers[0].infobox) {
            this[map].markers[0].infobox.close();
            this[map].markers[0].infobox = null;
            this[map].markers[0].displayInfobox(); 
        }
        this[map].setZoom(zoom);
        this[map].addPanorama(marker['lat'], marker['lon']);
        this[map].setCenter(marker['lat'], marker['lon']);
    } else {
        this[map].setCenter(marker['lat'], marker['lon']);
        this[map].addMarker(marker['lat'], marker['lon'], options);
    }

    $('#pano_container_' + id).show();
   
    var checker = function(){
        if ($('#pano_container_' + id).children().length > 1) {
            $('#pano_container_' + id).children().first().remove();
        }
    }
    setTimeout(checker, 1000);
    
}   


function update_coordinates(country, region, city, address, postal_code) {
    if (typeof(geocoder) != 'undefined') {
        var location = geocoder.getLocationFromAddress(country, region, city, address, postal_code);

        geocoder.geocodeLocation(location, function(latitude, longitude) {
            $('#lat').val(latitude);
            $('#lon').val(longitude);
            if (typeof(street_view) != 'undefined') {
                street_view.checkStreetView(latitude, longitude, function(is_map_panorama) {
                    $('#is_map_panorama').val(is_map_panorama);
                });
            }
            var id = getMapId();
            if (id) {
                var marker = getMarker();
                if (marker) {
                    updateMap(marker, id);
                }
            }
        });
    }
}

function check_address_updated() {
    location_change_wait--;
    if (location_change_wait)
        return;
    var country = $('input[name=id_country]').val();
    var region = $('input[name=id_region]').val();
    var city = $('input[name=id_city]').val();
    var address = $('input[name=data\\[address\\]]').val();
    var postal_code = $('input[name=data\\[zip\\]]').val();
    if (country == country_old && region == region_old &&
        city == city_old && address == address_old && postal_code == postal_code_old)
        return;
    country_old = country;
    region_old = region;
    city_old = city;
    address_old = address;
    postal_code_old = postal_code;
    var country_name = $('input[name="id_country"]').attr('data-name');
    var region_name = $('input[name="id_region"]').attr('data-name');
    var city_name = $('input[name="id_city"]').attr('data-name');
    update_coordinates(country_name, region_name, city_name, address, postal_code);
}

function getContentParams() {
    var country_name = $('input[name=id_country]').attr('data-name');
    var region_name = $('input[name=id_region]').attr('data-name');
    var city_name = $('input[name=id_city]').attr('data-name');
    if (country_name && region_name && city_name) {
        update_coordinates(country_name, region_name, city_name, null, null);
    }
}

function initMap() {
    var id = getMapId();
    if ($("#markers_data").attr('value')) {
        var marker = JSON.parse($("#markers_data").attr('value'));
        updateMap(marker[0], id);
    } else {
        var marker = getMarker();
        if (marker) {
            updateMap(marker, id);
        } else {
            if (navigator.geolocation) {
                var successCallback = function(position){
                    var marker = {
                        lat: position.coords.latitude,
                        lon: position.coords.longitude,
                        info: 'Your location'
                    };
                    updateMap(marker, id);
                }
                var errorCallback = function(){
                    console.log('geolocation not use');
                }
                navigator.geolocation.getCurrentPosition(successCallback,errorCallback,{timeout:10000});
            }
        }
    }
}   

function updateLocationFields() {
    location_change_wait++;
    setTimeout(check_address_updated, 1000);
}

$(function() {
    $('input[name=id_city]').bind('change', function() {
        var city = $(this).val();
        if (city == 0)
            return;
        location_change_wait++;
        check_address_updated();
    });

    $('input[name=data\\[address\\]], input[name=data\\[zip\\]]').keyup(function() {
        updateLocationFields();
    });
    $('input[name=data\\[address\\]], input[name=data\\[zip\\]]').change(function() {
        updateLocationFields();
    });

    category = new listingsEditCategory();

    new listingsAlert({
        siteUrl: $('#site_root').attr('value'),
        listingId: $('#listing_id').attr('value'),
        closeClass: 'alert-close',
    });


    $("span[data='location']").change(function() {
        getContentParams();
    });

    initMap();

});