<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

if (!defined('LISTINGS_EVENTS_TABLE')) {
    define("LISTINGS_EVENTS_TABLE", DB_PREFIX . "listings_events");
}

/**
 * Listings events model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_events_model extends \Model
{

    /**
     * CodeIgniter object
     * 
     * @var object
     */
    protected $CI;

    /**
     * Database object
     * 
     * @var object
     */
    protected $DB;

    /**
     * Properties of update module in data source
     * 
     * @var array
     */
    protected $fields = array(
        'id',
        'module',
        'model',
        'save_callback',
        'delete_callback',
    );

    /**
     * Updates cache
     * 
     * @var array
     */
    protected $updates = array();

    /**
     * Class constructor
     *
     * @return Listings_events_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = & get_instance();
        $this->DB = &$this->CI->db;

        $this->updates = $this->_get_updates_list();
    }

    /**
     * Return updates modules as array
     * 
     * @return array
     */
    protected function _get_updates_list()
    {
        $this->DB->select(implode(', ', $this->fields));
        $this->DB->from(LISTINGS_EVENTS_TABLE);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results))
            return $results;
        return array();
    }

    /**
     * Add module
     * 
     * @param array $module_data module data
     * @return void
     */
    public function add_module($module_data)
    {
        $this->DB->insert(LISTINGS_EVENTS_TABLE, $module_data);
    }

    /**
     * Remove module
     * 
     * @param string $module_gid module guid
     * @return void
     */
    public function delete_module($module_gid)
    {
        $this->DB->where("module", $module_gid);
        $this->DB->delete(LISTINGS_EVENTS_TABLE);
    }

    /**
     * Call functions on save listing
     * 
     * @param array $listing_old_data listing old data
     * @param array $listing_new_data listing new data
     * @return void
     */
    public function onsave_listing($listing_old_data, $listing_new_data)
    {
        foreach ($this->updates as $update) {
            if (strtolower($update['module']) . '_model' != strtolower($update['model'])) {
                $model_path = $update['module'] . '/models/' . $update['model'];
            } else {
                $model_path = $update['model'];
            }
            $this->CI->load->model($model_path);
            $this->CI->{$update['model']}->{$update['save_callback']}($listing_old_data, $listing_new_data);
        }
    }

    /**
     * Call functions on remove listing
     * 
     * @param array $listing_data listing data
     * @return void
     */
    public function ondelete_listing($listing_data)
    {
        foreach ($this->updates as $update) {
            if (strtolower($update['module']) . '_model' != strtolower($update['model'])) {
                $model_path = $update['module'] . '/models/' . $update['model'];
            } else {
                $model_path = $update['model'];
            }
            $this->CI->load->model($model_path);
            $this->CI->{$update['model']}->{$update['delete_callback']}($listing_data);
        }
    }

}
