<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings Uploads Model
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_uploads_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $CI;

    /**
     * Listing gallery type (GUID)
     * 
     * @var string
     */
    public $gallery_type = 'listings-photo';

    /**
     * Virtual tour type (GUID)
     * 
     * @var string
     */
    public $virtual_tour_type = 'listings-vtour';

    /**
     * Listing upload file (GUID)
     * 
     * @var string
     */
    public $file_type = 'listings-file';
    public $file_config_id = 'listing-file';

    /**
     * Listing upload video (GUID)
     * 
     * @var string
     */
    public $video_type = 'listings-video';
    public $video_config_id = 'listing-video';

    /**
     * Constructor
     *
     * return Listings_uploads_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->CI = & get_instance();
        $this->CI->load->model('Listings_model');
    }

    /**
     * Return type object of listing gallery
     * 
     * @return array
     */
    public function get_gallery_type()
    {
        $this->CI->load->model('Upload_gallery_model');
        return $this->CI->Upload_gallery_model->get_type_by_gid($this->gallery_type);
    }

    /**
     * Return type object of listing virtual tour
     * 
     * @return array
     */
    public function get_vtour_type()
    {
        $this->CI->load->model('Upload_gallery_model');
        return $this->CI->Upload_gallery_model->get_type_by_gid($this->virtual_tour_type);
    }

    /**
     * Return type object of listing file
     * 
     * @return array
     */
    public function get_file_type()
    {
        $this->CI->load->model('File_uploads_model');
        return $this->CI->File_uploads_model->get_config($this->file_config_id);
    }

    /**
     * Return type object of listing video 
     * 
     * @return array
     */
    public function get_video_type()
    {
        $this->CI->load->model('Video_uploads_model');
        return $this->CI->Video_uploads_model->get_config($this->video_config_id);
    }

    /**
     * Save listing photo to data source from upload file
     * 
     * @param integer $listing_id listing identifier
     * @param integer $photo_id photo identifier
     * @param string $file_name file name
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_photo($listing_id, $photo_id, $file_name, $moderation = true)
    {
        $this->CI->load->model('Upload_gallery_model');
        $photo_type = $this->CI->Upload_gallery_model->get_type_by_gid($this->gallery_type);
        $data = array('type_id' => $photo_type['id'], 'object_id' => $listing_id);
        $return = $this->CI->Upload_gallery_model->save_file($photo_id, $data, $file_name, $moderation);
        if (!empty($return['errors']))
            return $return;

        //$moderation = $moderation && $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        if (!$moderation || !$photo_type['use_moderation']) {
            $this->update_logo_image($listing_id);
        } else {
            $file = $this->CI->Upload_gallery_model->get_file_by_id($return['id']);
            if ($file['status'])
                $this->update_logo_image($listing_id);
        }
        return $return;
    }

    /**
     * Save listing photo to data source from local file
     * 
     * @param integer $listing_id listing identifier
     * @param integer $photo_id photo identifier
     * @param string $file_name file name
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_local_photo($listing_id, $photo_id, $file_name, $moderation = true)
    {
        $this->CI->load->model('Upload_gallery_model');
        $photo_type = $this->CI->Upload_gallery_model->get_type_by_gid($this->gallery_type);
        $data = array('type_id' => $photo_type['id'], 'object_id' => $listing_id);
        $return = $this->CI->Upload_gallery_model->save_local_file($photo_id, $data, $file_name, $moderation);
        if (!empty($return['errors']))
            return $return;

        //$moderation = $moderation && $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        if (!$moderation || !$photo_type['use_moderation']) {
            $this->update_logo_image($listing_id);
        } else {
            $file = $this->CI->Upload_gallery_model->get_file_by_id($return['id']);
            if ($file['status'] == 1)
                $this->update_logo_image($listing_id);
        }
        return $return;
    }

    /**
     * Remove listing photo object from data source by identifier
     * 
     * @param integer $listing_id listing identifier
     * @param integer $photo_id photo identifier
     * @return boolean
     */
    public function delete_photo($listing_id, $photo_id)
    {
        $this->CI->load->model('Upload_gallery_model');
        $photo_delete = $this->CI->Upload_gallery_model->get_file_by_id($photo_id);
        if ($listing_id != $photo_delete['object_id'])
            return false;
        $this->CI->Upload_gallery_model->delete_file($photo_id);
        $this->update_logo_image($listing_id);
        return true;
    }

    /**
     * Remove all listing photos object from data source
     * 
     * @param integer $listing_id listing identifier
     * @return boolean
     */
    public function delete_photos($listing_id)
    {
        $gallery_type = $this->get_gallery_type();
        $params = array('where' => array('type_id' => $gallery_type['id'], 'object_id' => $listing_id));
        $this->CI->load->model('Upload_gallery_model');
        $this->CI->Upload_gallery_model->delete_files_by_param($params);
        $this->update_logo_image($listing_id);
        return true;
    }

    /**
     * Resort photos in listing gallery
     * 
     * @param integer $listing_id listing identifier
     * @param array $data sorting data
     * @return void
     */
    public function set_photo_sorter($listing_id, $data)
    {
        $this->CI->load->model('Upload_gallery_model');
        foreach ($data as $photo_id => $sort_index) {
            $this->CI->Upload_gallery_model->save_sorter($photo_id, $sort_index);
        }
        $this->update_logo_image($listing_id);
    }

    /**
     * Save listing panorama to data source from upload file
     * 
     * @param integer $listing_id listing identifier
     * @param integer $panorama_id panorama identifier
     * @param string $file_name file name
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_panorama($listing_id, $panorama_id, $file_name, $moderation = true)
    {
        $this->CI->load->model('Upload_gallery_model');
        $virtual_tour_type = $this->CI->Upload_gallery_model->get_type_by_gid($this->virtual_tour_type);
        $data = array('type_id' => $virtual_tour_type['id'], 'object_id' => $listing_id);
        $return = $this->CI->Upload_gallery_model->save_file($panorama_id, $data, $file_name, $moderation);
        if (!empty($return['errors']))
            return $return;

        //$moderation = $moderation && $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        if (!$moderation || !$virtual_tour_type['use_moderation']) {
            $this->update_vtour($listing_id);
        } else {
            $file = $this->CI->Upload_gallery_model->get_file_by_id($return['id']);
            if ($file['status'] == 1)
                $this->update_vtour($listing_id);
        }
        return $return;
    }

    /**
     * Save listing panorama to data source from local file
     * 
     * @param integer $listing_id listing identifier
     * @param integer $panorama_id panorama identifier
     * @param string $file_name file name
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_local_panorama($listing_id, $panorama_id, $file_name, $moderation = true)
    {
        $this->CI->load->model('Upload_gallery_model');
        $vtour_type = $this->CI->Upload_gallery_model->get_vtour_type();
        $data = array('type_id' => $vtour_type['id'], 'object_id' => $listing_id);
        $return = $this->CI->Upload_gallery_model->save_local_file($panorama_id, $data, $file_name, $moderation);
        if (!empty($return['errors']))
            return $return;

        $moderation = $moderation && $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        if (!$moderation || !$vtour_type['use_moderation']) {
            $this->update_vtour($listing_id);
        } else {
            $file = $this->CI->Upload_gallery_model->get_file_by_id($return['id']);
            if ($file['status'] == 1)
                $this->Listings_model->update_vtour($listing_id);
        }
        return $return;
    }

    /**
     * Remove listing panorama object from data source
     * 
     * @param integer $listing_id listing identifier
     * @param integer $panorama_id panorama identifier
     * @return boolean
     */
    public function delete_panorama($listing_id, $panorama_id)
    {
        $this->CI->load->model('Upload_gallery_model');
        $panorama_delete = $this->CI->Upload_gallery_model->get_file_by_id($panorama_id);
        if ($listing_id != $panorama_delete['object_id'])
            return false;
        $this->CI->Upload_gallery_model->delete_file($panorama_id);
        $this->update_vtour($listing_id, false);
        return true;
    }

    /**
     * Remove all listing photos objects from data source
     * 
     * @param integer $listing_id listing identifier
     * @return boolean
     */
    public function delete_panoramas($listing_id)
    {
        $vtour_type = $this->get_vtour_type();
        $params = array('where' => array('type_id' => $vtour_type['id'], 'object_id' => $listing_id));
        $this->CI->load->model('Upload_gallery_model');
        $this->CI->Upload_gallery_model->delete_files_by_param($params);
        $this->update_vtour($listing_id, false);
        return true;
    }

    /**
     * Resort panoramas in listing virtual tours
     * 
     * @param integer $listing_id listing identifier
     * @param array $data sorting data
     * @return void
     */
    public function set_panorama_sorter($listing_id, $data)
    {
        $this->CI->load->model('Upload_gallery_model');
        foreach ($data as $photo_id => $sort_index) {
            $this->CI->Upload_gallery_model->save_sorter($photo_id, $sort_index);
        }
    }

    /**
     * Save listing file to data source from upload file
     * 
     * @param integer $listing_id listing identifier
     * @param string $file_name file name
     * @param array $data file description
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_file($listing_id, $file_name, $data, $moderation = false)
    {
        if (!$this->CI->pg_module->is_module_active('file_uploads'))
            return $return;

        $return = array('errors' => array(), 'data' => array());

        $this->CI->load->model('File_uploads_model');

        $validate_upload = $this->CI->File_uploads_model->validate_upload($this->file_config_id, $file_name);

        $this->CI->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);
        $this->CI->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));

        $is_moderation_installed = $this->CI->pg_module->is_module_active('moderation');
        if ($moderation && $is_moderation_installed) {
            $moderation = $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        } else {
            $moderation = false;
        }

        if ($moderation) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            if ($this->CI->Listings_moderation_model->is_listing_moderated($listing_id)) {
                $fields_for_select = $this->CI->Listings_moderation_model->get_fields_for_moderate();
                $this->CI->Listings_moderation_model->set_dop_fields($fields_for_select);
                $moderated_data = $this->CI->Listings_moderation_model->get_listing_by_id($listing_id);
                $listing = array_merge($listing, $moderated_data);
            }
        }

        if (!isset($validate_upload['errors']))
            $validate_upload['errors'] = array();

        if (isset($validate_upload['error']) && $validate_upload['error'] != '') {
            if (is_array($validate_upload['error'])) {
                $validate_upload['errors'] = $validate_upload['error'];
            } else {
                $validate_upload['errors'][] = $validate_upload['error'];
            }
        }

        if (!empty($validate_upload['errors'])) {
            
        } else {
            if (!empty($validate_upload['data']) && !empty($listing['listing_file'])) {
                $validate_upload['errors'][] = l('error_max_files_reached', 'listings');
            }

            if (empty($validate_upload['data']) && empty($listing['listing_file'])) {
                $validate_upload['errors'][] = l('error_empty_file', 'listings');
            }
        }

        $validate_data = $this->CI->Listings_model->validate_listing($listing_id, $data);
        $validate_data['errors'] = array_merge($validate_data['errors'], $validate_upload['errors']);
        if (!empty($validate_data['errors'])) {
            $return['errors'] = $validate_data['errors'];
        } else {
            if (!empty($validate_upload['data'])) {
                $file_return = $this->CI->File_uploads_model->upload($this->file_config_id, $listing['prefix'], $file_name);
                if (!isset($validate_data['data']['listing_file_name'])) {
                   $validate_data['data']['listing_file_name'] = $validate_upload['data']['file_name'];
                }
                $validate_data['data']['listing_file'] = $file_return['file'];
                $validate_data['data']['listing_file_date'] = date('Y-m-d H:i:s');
                $return['data']['file'] = $file_return['file'];
            }
            $this->CI->Listings_model->save_listing($listing_id, $validate_data['data'], $moderation);
            $return['file'] = $validate_data['data'];
        }
        return $return;
    }

    /**
     * Save listing file to data source from local file
     * 
     * @param integer $listing_id listing identifier
     * @param string $file_name file name
     * @param array $data file description
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_local_file($listing_id, $file_name, $data, $moderation = false)
    {
        if (!$this->CI->pg_module->is_module_active('file_uploads'))
            return $return;

        $return = array('errors' => array(), 'data' => array());

        $this->CI->load->model('File_uploads_model');

        $this->CI->Listings_model->set_format_settings('use_format', false);
        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);
        $this->CI->Listings_model->set_format_settings('use_format', true);

        $is_moderation_installed = $this->CI->pg_module->is_module_active('moderation');
        if ($moderation && $is_moderation_installed) {
            $moderation = $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        } else {
            $moderation = false;
        }

        if ($moderation) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            if ($this->CI->Listings_moderation_model->is_listing_moderated($listing_id)) {
                $fields_for_select = $this->CI->Listings_moderation_model->get_fields_for_moderate();
                $this->CI->Listings_moderation_model->set_dop_fields($fields_for_select);
                $moderated_data = $this->CI->Listings_moderation_model->get_listing_by_id($listing_id);
                $listing = array_merge($listing, $moderated_data);
            }
        }

        $validate_upload['errors'] = array();

        if (!empty($listing['listing_file'])) {
            $validate_upload['errors'][] = l('error_max_files_reached', 'listings');
        }

        $validate_data = $this->CI->Listings_model->validate_listing($listing_id, $data);
        $validate_data['errors'] = array_merge($validate_data['errors'], $validate_upload['errors']);
        if (!empty($validate_data['errors'])) {
            $return['errors'] = $validate_data['errors'];
        } else {
            if (!empty($validate_upload['data'])) {
                $file_return = $this->CI->File_uploads_model->upload_exist($this->file_config_id, $listing['prefix'], $file_name);
                $validate_data['data']['listing_file'] = $file_return['file'];
                $validate_data['data']['listing_file_date'] = date('Y-m-d H:i:s');
            }
            $this->CI->Listings_model->save_listing($listing_id, $validate_data['data'], $moderation);
            $return['data']['file'] = $file_return['file'];
        }
        return $return;
    }

    /**
     * Remove listing file object from data source
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_file($listing_id)
    {
        $is_listings_installed = $this->CI->pg_module->is_module_active('moderation');
        if ($is_listings_installed) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            $already_moderated = $this->CI->Listings_moderation_model->is_listing_moderated($listing_id);
            if ($already_moderated) {
                $this->CI->Listings_moderation_model->delete_file($listing_id);
                return;
            }
        }

        $this->CI->Listings_model->set_format_settings('get_file', true);
        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);
        $this->CI->Listings_model->set_format_settings('get_file', false);

        $this->CI->load->model('File_uploads_model');
        $this->CI->File_uploads_model->delete_upload($this->file_config_id, $listing['prefix'], $listing['listing_file']);

        $listing_data = array(
            'listing_file' => '',
            'listing_file_name' => '',
            'listing_file_comment' => '',
        );
        $this->CI->Listings_model->save_listing($listing_id, $listing_data);
    }

    /**
     * Remove all listing files objects from data source 
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_files($listing_id)
    {
        $this->delete_file($listing_id);
    }

    /**
     * Save listing video object to data source from upload file
     * 
     * @param integer $listing_id listing identifier
     * @param string $video_name video upload name
     * @param string $embed embed data
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_video($listing_id, $video_name, $embed, $moderation = false)
    {
        if (!$this->CI->pg_module->is_module_active('video_uploads')) {
            return $return;
        }

        $return = array('errors' => array(), 'data' => array(), 'success' => false);

        if (empty($listing_id)) {
            $return['errors'][] = l('error_empty_video', 'listings');
            return $return;
        }

        $format_settings = array('use_format', 'get_location', 'get_video');
        $this->CI->Listings_model->set_format_settings(array_fill_keys($format_settings, true));
        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);
        $this->CI->Listings_model->set_format_settings(array_fill_keys($format_settings, false));


        if (empty($listing)) {
            $return['errors'][] = l('error_empty_video', 'listings');
            return $return;
        }

        $is_moderation_installed = $this->CI->pg_module->is_module_active("moderation");
        if ($moderation && $is_moderation_installed) {
            $moderation = $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        } else {
            $moderation = false;
        }

        if ($moderation) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            if ($this->CI->Listings_moderation_model->is_listing_moderated($listing_id)) {
                $fields_for_select = $this->CI->Listings_moderation_model->get_fields_for_moderate();
                $this->CI->Listings_moderation_model->set_dop_fields($fields_for_select);
                $moderated_data = $this->CI->Listings_moderation_model->get_listing_by_id($listing_id);
                $listing = array_merge($listing, $moderated_data);
            }
        }

        $embed_data = array();
        if (!empty($embed)) {
            $this->CI->load->library('VideoEmbed');
            $embed_data = $this->CI->videoembed->get_video_data($embed);
            if ($embed_data !== false) {
                $embed_data["string_to_save"] = $this->CI->videoembed->get_string_from_video_data($embed_data);
                $embed_data['upload_type'] = 'embed';
                $embed_data['name'] = $listing['location'];
                $embed_data['description'] = $listing['category_str'] . '_' . $listing['property_type_str'];
            } else {
                $return["errors"][] = l('error_embed_wrong', 'listings');
            }
        }

        if (isset($_FILES[$video_name]) && is_array($_FILES[$video_name]) && $_FILES[$video_name]['error'] != 4) {
            if (!is_uploaded_file($_FILES[$video_name]['tmp_name'])) {
                $return['errors'][] = l('error_upload_video', 'listings');
                return $return;
            }
        } else {
            $video_name = '';
        }

        if (!empty($listing['listing_video'])) {
            if ($listing['listing_video_data']['data']['upload_type'] == 'embed') {
                if ($video_name) {
                    $return['errors'][] = l('error_max_video_reached', 'listings');
                }
            } elseif ($video_name || $embed_data) {
                $return['errors'][] = l('error_max_video_reached', 'listings');
            }
        }

        if (empty($return['errors'])) {
            if ($video_name) {
                $is_moderation_installed = $this->CI->pg_module->is_module_active('moderation');
                if ($is_moderation_installed) {
                    $this->CI->load->model('listings/models/Listings_moderation_model');
                    $already_moderated = $this->CI->Listings_moderation_model->is_listing_moderated($listing_id);
                }
                if ($moderation && !$already_moderated) {
                    $this->CI->Listings_moderation_model->save_listing($listing_id, array('id_listing' => $listing_id));
                }

                $video_data = array(
                    'name' => $listing['location'],
                    'description' => $listing['category_str'] . '_' . $listing['property_type_str'],
                );

                $this->CI->load->model('Video_uploads_model');
                $video_return = $this->CI->Video_uploads_model->upload($this->video_config_id, $listing['prefix'], $video_name, $listing_id, $video_data);
                if (!empty($video_return['errors'])) {
                    $return['errors'] = $video_return['errors'];
                    if ($moderation && !$already_moderated) {
                        $this->CI->Listings_moderation_model->delete_listing($listing_id);
                    }
                } else {
                    $return['data']['file'] = $video_return['file'];
                }
                $return['success'] = true;
            } elseif ($embed_data) {
                $this->CI->load->model('Uploads_model');
                $save_data["listing_video_image"] = $this->CI->Uploads_model->generate_filename('.jpg');
                $save_data["listing_video_data"] = serialize(array('data' => $embed_data));
                $save_data["listing_video"] = 'embed';
                $this->CI->Listings_model->save_listing($listing_id, $save_data, $moderation);
                $return['success'] = true;
            }
        }

        return $return;
    }

    /**
     * Save listing video object to data source from local file
     * 
     * @param integer $listing_id listing identifier
     * @param string $video_name video upload name
     * @param boolean $moderation use moderation
     * @return array
     */
    public function save_local_video($listing_id, $video_name, $moderation = false)
    {
        if (!$this->CI->pg_module->is_module_active('video_uploads'))
            return $return;

        $return = array('errors' => array(), 'data' => array());

        if (empty($listing_id) || empty($video_name)) {
            $return['errors'][] = l('error_empty_video', 'listings');
            return $return;
        }

        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);

        $is_moderation_installed = $this->CI->pg_module->is_module_active('moderation');
        if ($moderation && $is_moderation_installed) {
            $moderation = $this->CI->pg_module->get_module_config('listings', 'use_moderation');
        } else {
            $moderation = false;
        }

        if ($moderation) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            if ($this->CI->Listings_moderation_model->is_listing_moderated($listing_id)) {
                $fields_for_select = $this->CI->Listings_moderation_model->get_fields_for_moderate();
                $this->CI->Listings_moderation_model->set_dop_fields($fields_for_select);
                $moderated_data = $this->CI->Listings_moderation_model->get_listing_by_id($listing_id);
                $listing = array_merge($listing, $moderated_data);
            }
        }

        if (!empty($listing['listing_video'])) {
            $return['errors'][] = l('error_max_video_reached', 'listings');
        } else {
            $video_data = array(
                'name' => $listing['location'],
                'description' => $listing['category_str'] . '_' . $listing['property_type_str'],
            );
            $this->CI->load->model('Video_uploads_model');
            $video_return = $this->CI->Video_uploads_model->upload_exists($this->video_config_id, $listing['prefix'], $video_name, $listing_id, $video_data);
            if (!empty($video_return['errors'])) {
                $return['errors'] = $video_return['errors'];
            } else {
                $return['data']['file'] = $video_return['file'];
            }
        }

        return $return;
    }

    /**
     * Remove listing video object from data source
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_video($listing_id)
    {
        $is_moderation_installed = $this->CI->pg_module->is_module_active('moderation');
        if ($is_moderation_installed) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            $already_moderated = $this->CI->Listings_moderation_model->is_listing_moderated($listing_id);
            if ($already_moderated) {
                $this->CI->Listings_moderation_model->delete_video($listing_id);
                return;
            }
        }

        $this->CI->Listings_model->set_format_settings('get_video', true);
        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);
        $this->CI->Listings_model->set_format_settings('get_video', false);

        if ($listing['listing_video_data'] && $listing['listing_video_data']['data']['upload_type'] != 'embed') {
            $this->CI->load->model('Video_uploads_model');
            $this->CI->Video_uploads_model->delete_upload($this->video_config_id, $listing['prefix'], $listing['listing_video'], $listing['listing_video_image'], $listing['listing_video_data']['data']['upload_type']);
        }

        $listing_data = array(
            'listing_video' => '',
            'listing_video_image' => '',
            'listing_video_data' => '',
        );
        $this->CI->Listings_model->save_listing($listing_id, $listing_data);
    }

    /**
     * Remove all listing videos objects from data source
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_videos($listing_id)
    {
        $this->delete_video($listing_id);
    }

    /**
     * Update listing logo and slide-show
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function update_logo_image($listing_id)
    {
        $this->CI->load->model('Uploads_model');
        $upload_config = $this->CI->Uploads_model->get_config($this->CI->Listings_model->upload_config_id);

        $format_settings = array('get_user', 'get_location');
        $this->CI->Listings_model->set_format_settings(array_fill_keys($format_settings, false));
        $listing = $this->CI->Listings_model->get_listing_by_id($listing_id);
        $this->CI->Listings_model->set_format_settings(array_fill_keys($format_settings, true));

        if (empty($listing))
            return;

        $this->CI->load->model('Upload_gallery_model');

        $gallery_type = $this->CI->Upload_gallery_model->get_type_by_gid($this->gallery_type);
        $gallery_config = $this->CI->Uploads_model->get_config($gallery_type['gid_upload_config']);

        $params = array();
        $params['where']['type_id'] = $gallery_type['id'];
        $params['where']['object_id'] = $listing['id'];
        $params['where']['status'] = 1;
        $photos = $this->CI->Upload_gallery_model->get_files_by_param($params);

        $logo_width = 0;
        $logo_height = 0;
        foreach ($upload_config['thumbs'] as $thumb_config) {
            $logo_width = max($logo_width, $thumb_config['width']);
            $logo_height = max($logo_height, $thumb_config['height']);
        }

        $logo_thumb_prefix = '';
        $logo_thumb_width = 0;
        $logo_thumb_height = 0;
        foreach ($gallery_config['thumbs'] as $thumb_config) {
            if ($thumb_config['width'] >= $logo_width && $thumb_config['height'] >= $logo_height &&
                $thumb_config['width'] / $logo_width == $thumb_config['height'] / $logo_height &&
                (!$logo_thumb_width || $thumb_config['width'] < $logo_thumb_width) &&
                (!$logo_thumb_height || $thumb_config['width'] < $logo_thumb_height)) {
                $logo_thumb_prefix = $thumb_config['prefix'];
            }
        }

        $post_data = array('photo_count' => count($photos));
        $validate_data = array();

        // check used as logo
        foreach ($photos as $i => $photo) {
            if ($photo['status'] != 1)
                continue;
            $file_path = $logo_thumb_prefix ? preg_replace('/^' . preg_quote(SITE_VIRTUAL_PATH, '/') . '/', SITE_PHYSICAL_PATH, $photo['media']['thumbs'][$logo_thumb_prefix]) : $photo['media']['file_path'];

            $img_return = $this->CI->Uploads_model->upload_exist($this->CI->Listings_model->upload_config_id, $listing_id . '/', $file_path);
            if (!empty($img_return['errors'])) {
                $validate_data['errors'] = array_merge($validate_data['errors'], $img_return['errors']);
            } else {
                $post_data['logo_image'] = $img_return['file'];
                $photos[$i]['settings']['use_for_search'] = 1;
                $validate_data_photo = $this->CI->Upload_gallery_model->validate_file_data($photo['id'], $photo['type_id'], $photos[$i]);
                $this->CI->Upload_gallery_model->save_local_file($photo['id'], $validate_data_photo['data'], '', false);

                $slider_min_width = $this->CI->pg_module->get_module_config('listings', 'slider_min_width');
                $slider_min_height = $this->CI->pg_module->get_module_config('listings', 'slider_min_height');

                $image_size = @getimagesize($photo['media']['file_path']);
                if ($image_size[0] >= $slider_min_width && $image_size[1] >= $slider_min_height) {
                    $img_return = $this->CI->Uploads_model->upload_exist($this->CI->Listings_model->slider_config_id, $listing_id . '/', $photo['media']['file_path']);
                } else {
                    $img_return['errors'] = true;
                }
                if (!empty($img_return['errors'])) {
                    $post_data['slider_image'] = '';
                } else {
                    $post_data['slider_image'] = $img_return['file'];
                }
            }
            break;
        }

        if (!isset($post_data['logo_image'])) {
            $post_data['logo_image'] = '';
            $post_data['slider_image'] = '';
        }
        if ($listing['logo_image'] && $listing['logo_image'] != $post_data['logo_image']) {
            $this->CI->Uploads_model->delete_upload($this->CI->Listings_model->upload_config_id, $listing_id . '/', $listing['logo_image']);
            if ($listing['slider_image'] && $listing['slider_image'] != $post_data['slider_image']) {
                $this->CI->Uploads_model->delete_upload($this->CI->Listings_model->slider_config_id, $listing_id . '/', $listing['slider_image']);
            }
        }

        // check used for slide show
        $sources = array();
        if (isset($post_data['logo_image']))
            $listing['logo_image'] = $post_data['logo_image'];
        foreach ($photos as $photo) {
            if ($photo['status'] != 1)
                continue;
            $file_path = $logo_thumb_prefix ? $photo['media']['thumbs'][$logo_thumb_prefix] : $photo['media']['file_path'];
            $sources[] = $file_path;
        }
        if (!empty($sources)) {
            $slide_show_images = $this->CI->pg_module->get_module_config('listings', 'slide_show_images');
            $sources = array_slice($sources, 0, $slide_show_images);
            $img_return = $this->CI->Uploads_model->upload_anim($this->CI->Listings_model->upload_config_id, $listing_id . '/', $listing['logo_image'], $sources);
            if (!isset($validate_data))
                $validate_data = array();
            if (!empty($img_return['errors'])) {
                $validate_data['errors'] = array_merge($validate_data['errors'], $img_return['errors']);
            }
        }

        if (!empty($post_data)) {
            if (!isset($validate_data)){
                $validate_data = array();
            }
            
            $format_settings['get_location'] = false;
            $this->CI->Listings_model->set_format_settings($format_settings);
            
            $data = $this->CI->Listings_model->validate_listing($listing_id, $post_data);
            $validate_data = array_merge_recursive($validate_data, $data);
            if (empty($validate_data['errors'])) {
                $this->CI->Listings_model->save_listing($listing_id, $validate_data['data']);
            }
        }

        // update wish lists
        if (!empty($listing['id_wish_lists'])) {
            $this->CI->load->model('listings/models/Wish_list_model');
            foreach ($listing['id_wish_lists'] as $wish_list_id) {
                if ($listing['logo_image']) {
                    $this->CI->Wish_list_model->change_logo($wish_list_id);
                } else {
                    $this->CI->Wish_list_model->delete_from_wish_list($wish_list_id, $listing_id);
                }
            }
        }
    }

    /**
     * Update listing virtual tour is available
     * 
     * @param integer $listing_id listing identifier
     * @param boolean $is_vtour virtual tour availability
     * @return void
     */
    public function update_vtour($listing_id, $is_vtour = true)
    {
        $post_data = array('is_vtour' => $is_vtour);
        $validate_data = $this->CI->Listings_model->validate_listing($listing_id, $post_data);
        if (!empty($validate_data['errors']))
            return;
        $this->CI->Listings_model->save_listing($listing_id, $validate_data['data']);
    }

    /**
     * Proccessing events from upload photo
     * 
     * @param string $action action name
     * @param integer $object_id object identifier
     * @param array $params photo parameters
     * @return void
     */
    public function photo_callback($action, $object_id, $params = false)
    {
        switch ($action) {
            case 'moder_set_status':
                if (!$params)
                    break;
                $this->CI->load->model('Upload_gallery_model');
                $file = $this->CI->Upload_gallery_model->get_file_by_id($object_id);
                $this->update_logo_image($file['object_id']);
                break;
            case 'moder_delete':
                if (!$params)
                    break;
                $this->update_logo_image($object_id);
                break;
            case 'recrop':
                if (!empty($params['settings']['use_for_search'])) {
                    $this->CI->load->model('Upload_gallery_model');
                    $file = $this->CI->Upload_gallery_model->get_file_by_id($object_id);
                    $this->update_logo_image($file['object_id']);
                }
                break;
            case 'rotate':
                if (!empty($params['settings']['use_for_search'])) {
                    $this->CI->load->model('Upload_gallery_model');
                    $file = $this->CI->Upload_gallery_model->get_file_by_id($object_id);
                    $this->update_logo_image($file['object_id']);
                }
                break;
            case 'check_access':
                $user_id = $this->session->userdata('user_id');
                $this->CI->Listings_model->set_format_settings('use_format', false);
                $listing = $this->CI->Listings_model->get_listing_by_id($object_id);
                $this->CI->Listings_model->set_format_settings('use_format', true);
                return $user_id == $listing['id_user'];
                break;
            case 'sorting_data':
            case 'upload_file':
            case 'delete_file':
                $this->update_logo_image($object_id);
                break;
        }
    }

    /**
     * Processing events of upload panorama
     * 
     * @param string $action action name
     * @param integer $object_id object identifier
     * @param integer $status moderation status
     * @return mixed
     */
    public function vtour_callback($action, $object_id, $status = false)
    {

        switch ($action) {
            case 'moder_set_status':
                if (!$status)
                    break;
                $this->CI->load->model('Upload_gallery_model');
                $file = $this->CI->Upload_gallery_model->get_file_by_id($object_id);
                $this->update_vtour($file['object_id']);
                break;
            case 'moder_delete':
                if (!$status)
                    break;
                $params = array();
                $params['where']['object_id'] = $object_id;
                $params['where']['status'] = 1;
                $this->CI->load->model('Upload_gallery_model');
                $vtour_type = $this->CI->Upload_gallery_model->get_type_by_gid($this->virtual_tour_type);
                $params['where']['type_id'] = $vtour_type['id'];
                $count = $this->CI->Upload_gallery_model->get_files_by_param_count($params);
                $this->update_vtour($object_id, $count ? true : false);
                break;
            case 'check_access':
                $user_id = $this->session->userdata('user_id');
                $this->CI->Listings_model->set_format_settings('use_format', false);
                $listing = $this->CI->Listings_model->get_listing_by_id($object_id);
                $this->CI->Listings_model->set_format_settings('use_format', true);
                return $user_id == $listing['id_user'];
                break;
            case 'sorting_data':
                break;
            case 'upload_file':
            case 'delete_file':
                $params = array();
                $params['where']['object_id'] = $object_id;
                $params['where']['status'] = 1;
                $this->CI->load->model('Upload_gallery_model');
                $this->CI->load->model('listings/models/Listings_uploads_model');
                $vtour_type = $this->CI->Upload_gallery_model->get_type_by_gid($this->virtual_tour_type);
                $params['where']['type_id'] = $vtour_type['id'];
                $count = $this->CI->Upload_gallery_model->get_files_by_param_count($params);
                $this->update_vtour($object_id, $count ? true : false);
                break;
        }
    }

    /**
     * Processing events from upload video
     * 
     * @param integer $object_id object identifier
     * @param string $status upload status
     * @param array $data upload data
     * @param array $errors upload errors
     * @return void
     */
    public function video_callback($object_id, $status, $data, $errors)
    {
        $this->CI->Listings_model->set_format_settings('get_video', true);
        $listing_data = $this->CI->Listings_model->get_listing_by_id($object_id);
        $this->CI->Listings_model->set_format_settings('get_video', false);

        $is_moderation_installed = $this->CI->pg_module->is_module_active('moderation');
        if ($is_moderation_installed) {
            $this->CI->load->model('listings/models/Listings_moderation_model');
            $already_moderated = $this->CI->Listings_moderation_model->is_listing_moderated($object_id);
            if ($already_moderated) {
                $fields_for_select = $this->CI->Listings_moderation_model->get_fields_for_moderate();
                $this->CI->Listings_moderation_model->set_dop_fields($fields_for_select);
                $moderated_data = $this->CI->Listings_moderation_model->get_listing_by_id($object_id);
                $listing_data = array_merge($listing_data, $moderated_data);
            }
        }

        if (isset($data['video']))
            $update['listing_video'] = $data['video'];
        if (isset($data['image']))
            $update['listing_video_image'] = $data['image'];
        $update['listing_video_data'] = $listing_data['listing_video_data'];
        if ($status == 'start' || !isset($update['listing_video_data']['data']))
            $update['listing_video_data']['data'] = array();
        if (!empty($data))
            foreach ($data as $k => $v)
                $update['listing_video_data']['data'][$k] = $v;

        $update['listing_video_data']['status'] = $status;
        $update['listing_video_data']['errors'] = $errors;

        $validate_data = $this->CI->Listings_model->validate_listing($object_id, $update);
        $this->CI->Listings_model->save_listing($object_id, $validate_data['data'], $already_moderated);
        return;
    }

    public function getSize($max_size, $module) 
    {
        $i = 1;
        while ($max_size > 1000) {
                    $max_size /= 1024;
                    $i++;
                }
        $max_size = round($max_size, 1);
        if ($max_size) {
            $max_size = $max_size . ' ' . l('text_filesize_' . $i, $module);
            return  $max_size;
        } else {
            return 0;
        }
    }

}
