<?php

namespace Pg\Modules\Memberships\Controllers;

use Pg\Libraries\View;
use Pg\Modules\Memberships\Models\Memberships_model;

/**
 * Memberships module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Memberships admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Memberships
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Memberships extends \Controller
{

    /**
     * Class constructor
     *
     * @return Admin_Memberships
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model(array('Memberships_model', 'Menu_model'));
        $this->Menu_model->set_menu_active_item('admin_menu', 'payments_menu_item');
    }

    /**
     * Memberships management
     *
     * @param string  $order           sorting order
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function index($order = null, $order_direction = null, $page = null)
    {
        $memberships_settings = isset($_SESSION['memberships']) ? $_SESSION['memberships'] : array();
        if (!isset($memberships_settings['order'])) {
            $memberships_settings['order'] = 'date_created';
        }
        if (!isset($memberships_settings['order_direction'])) {
            $memberships_settings['order_direction'] = 'DESC';
        }
        if (!isset($memberships_settings['page'])) {
            $memberships_settings['page'] = 1;
        }

        $order = strval($order);
        $order_direction = strval($order_direction);
        $page = intval($page);

        $this->load->helper('sort_order');

        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');

        $memberships_count = $this->Memberships_model->getMembershipsCount();
        $page = get_exists_page_number($page, $memberships_count, $items_on_page);
        $memberships_settings['page'] = $page;

        if (!$order) {
            $order = $memberships_settings['order'];
        }
        $this->view->assign('order', $order);
        $memberships_settings['order'] = $order;

        if (!$order_direction) {
            $order_direction = $memberships_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);
        $memberships_settings['order_direction'] = $order_direction;

        $filters = array();

        $_SESSION['memberships'] = $memberships_settings;

        $sort_links = array(
            'name' => site_url() . 'admin/memberships/name/' . (($order != 'name' xor $order_direction == 'DESC') ? 'ASC' : 'DESC'),
            'date_created' => site_url() . 'admin/memberships/date_created/' . (($order != 'date_created' xor $order_direction == 'DESC') ? 'ASC' : 'DESC'),
        );
        $this->view->assign('sort_links', $sort_links);

        switch ($order) {
            case 'date_created':
                $order_array = array('id' => $order_direction);
                break;
            default:
                $order_array = array($order => $order_direction);
                break;
        }

        if ($memberships_count > 0) {
            $memberships = $this->Memberships_model->getMembershipsList($filters, $page, $items_on_page, $order_array);
            $memberships = $this->Memberships_model->formatMemberships($memberships);
            $this->view->assign('memberships', $memberships);
        }

        $this->load->helper('navigation');
        $page_data = get_admin_pages_data(site_url() . 'admin/memberships/index/' . $order . '/' . $order_direction . '/',
                                          $memberships_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->load->library('pg_date');
        $date_format = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('date_format', $date_format);

        $this->view->setHeader(l('admin_header_list', Memberships_model::MODULE_GID));

        $this->view->render('index');
    }

    /**
     * Edit membership
     *
     * @param integer $membership_id membership identifier
     *
     * @return void
     */
    public function edit($membership_id, $price_index = -1)
    {
        $membership_id = intval($membership_id);

        if ($membership_id) {
            $langs_ids = array_keys($this->pg_language->languages);
            $membership_data = $this->Memberships_model->getMembershipById($membership_id, $langs_ids);
            if (empty($membership_data)) {
                $this->view->setRedirect(site_url() . 'admin/memberships/create');
            }
            $this->Memberships_model->setFormatSettings('get_services', true);
            $membership_data = $this->Memberships_model->formatMembership($membership_data);
            $this->Memberships_model->setFormatSettings('get_services', false);
            $header = l('admin_header_edit', Memberships_model::MODULE_GID);
        } else {
            $membership_data = array();
            $header = l('admin_header_create', Memberships_model::MODULE_GID);
        }

        if ($this->input->post('save')) {
            $post_data = $this->input->post('data', true);
            if (empty($post_data['user_type_disabled'])) {
                $post_data['user_type_disabled'] = array();
            }

            if ($membership_id) {
                $post_data['services'] = $membership_data['services_array'];

                $services = (array) $this->input->post('services', true);
                foreach ($services as $service_id => $service_status) {
                    $post_data['services'][$service_id]['is_active'] = $service_status ? 1 : 0;
                }

                $params = (array) $this->input->post('params', true);
                foreach ($params as $service_id => $params_data) {
                    if (!isset($post_data['services'][$service_id])) {
                        $post_data['services'][$service_id] = array('is_active' => 0, 'params' => array());
                    }
                    $post_data['services'][$service_id]['params'] = $params_data;
                }
            }

            $validate_data = $this->Memberships_model->validateMembership($membership_id, $post_data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
            } else {
                $services_data = array();

                if (!empty($post_data['services'])) {
                    foreach ($post_data['services'] as $service_id => $service_data) {
                        $services_data[$service_id]['data_admin'] = $service_data['params'];
                        $this->Services_model->updateServiceStatus($service_id, $service_data['is_active']);
                    }
                }

                $membership_id = $this->Memberships_model->save($membership_id, $validate_data['data'], $services_data);

                if (!empty($membership_data)) {
                    $message = l('success_item_updated', Memberships_model::MODULE_GID);
                } else {
                    $message = l('success_item_created', Memberships_model::MODULE_GID);
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
                $this->view->setRedirect(site_url() . 'admin/memberships/edit/' . $membership_id);
            }

            $membership_data = array_merge($membership_data, $post_data);
        }

        if ($this->input->post('save_params')) {
            $post_data = array(
                'services' => $membership_data['services_array'],
            );

            $params = (array) $this->input->post('params', true);
            foreach ($params as $service_id => $params_data) {
                if (!isset($post_data['services'][$service_id])) {
                    $post_data['services'][$service_id] = array('is_active' => 0, 'params' => array());
                }
                $post_data['services'][$service_id]['params'] = $params_data;
            }

            $validate_data = $this->Memberships_model->validateMembership($membership_id, $post_data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage('errors', $validate_data['errors']);
            } else {
                $services_data = array();
                foreach ($post_data['services'] as $service_id => $service_data) {
                    $services_data[$service_id]['data_admin'] = $service_data['params'];
                }

                $this->Memberships_model->save($membership_id, $validate_data['data'], $services_data);
                if ($price_index != -1) {
                    $message = l('success_price_updated', Memberships_model::MODULE_GID);
                } else {
                    $message = l('success_price_created', Memberships_model::MODULE_GID);
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
                $this->view->setRedirect(site_url() . 'admin/memberships/edit/' . $membership_id);
            }

            $membership_data = array_merge($membership_data, $post_data);
        }

        if (empty($membership_data['gid'])) {
            $membership_data['gid'] = $this->Memberships_model->generateGUID();
        }

        $this->view->assign('membership', $membership_data);

        $langs = $this->pg_language->languages;
        $this->view->assign('langs', $langs);

        $current_lang_id = $this->pg_language->current_lang_id;
        $this->view->assign('current_lang_id', $current_lang_id);

        $this->view->assign('pay_types', $this->Memberships_model->getAllowedPaymentTypes());
        $this->view->assign('period_types', $this->Memberships_model->getAllowedPeriodTypes());

        $this->load->helper('users');
        $user_types = get_user_types();
        if (!empty($user_types)) {
            $this->view->assign('user_types', $user_types);
        }

        if (!empty($payment_types['option'])) {
            $this->view->assign('payment_types', $payment_types['option']);
        }

        $this->load->model('Services_model');

        $services = $this->Services_model->get_service_list(
                array('where' => array('type' => 'membership', 'id_membership' => $membership_id)));
        $this->view->assign('services', $services);

        $this->view->setHeader($header);

        $this->view->render('edit');
    }

    /**
     * Create membership
     *
     * @param integer $membership_id membership idnetifier
     *
     * @return void
     */
    public function create()
    {
        $this->edit(0);
    }

    /**
     * Remove membership
     *
     * @param integer $membership_id membership identifier
     *
     * @return void
     */
    public function delete($membership_id)
    {
        if (empty($membership_id)) {
            $membership_ids = (array) $this->input->post('ids', true);
        } else {
            $membership_ids = array($membership_id);
        }

        if (!empty($membership_ids)) {
            $this->Memberships_model->deleteMemberships($membership_ids);

            $message = l('success_item_delete', Memberships_model::MODULE_GID);
            $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
        }

        $url = site_url() . 'admin/memberships/index';
        $this->view->setRedirect($url);
    }

    /**
     * Activate membership
     *
     * @param integer $membership_id membership identifier
     *
     * @return void
     */
    public function activate($membership_id)
    {
        $validation = $this->Memberships_model->activate($membership_id);
        
        if (!empty($validation['errors'])) {
            foreach ($validation['errors'] as $error) {
                $this->system_messages->addMessage(View::MSG_ERROR, l($error, 'memberships'));
            }
        } else {       
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_item_activate', Memberships_model::MODULE_GID));
        }
            
        $url = site_url() . 'admin/memberships/index';
        $this->view->setRedirect($url);
    }

    /**
     * Deactivate membership
     *
     * @param integer $membership_id membership identifier
     *
     * @return void
     */
    public function deactivate($membership_id)
    {
        $this->Memberships_model->deactivate($membership_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS,
                                           l('success_item_deactivate', Memberships_model::MODULE_GID));
        $url = site_url() . 'admin/memberships/index';
        $this->view->setRedirect($url);
    }

    /**
     * Activate membership service
     *
     * @param integer $membership_id membership identifier
     * @param integer $service_id    service template identifier
     *
     * @return void
     */
    public function activate_service($membership_id, $service_id)
    {
        $membership_id = intval($membership_id);
        $service_id = intval($service_id);
        $redirect_url = site_url() . 'admin/memberships/edit/' . $membership_id;
        if (!$membership_id || !$service_id) {
            $this->view->setRedirect($redirect_url);
            return false;
        }

        $membership_data = $this->Memberships_model->formatMembership(
                $this->Memberships_model->getMembershipById($membership_id)
        );

        if (empty($membership_data['services_array'][$service_id]['is_active'])) {
            if (!isset($membership_data['services_array'][$service_id])) {
                $membership_data['services_array'][$service_id] = array('is_active' => 0, 'params' => array());
            }

            $membership_data['services_array'][$service_id]['is_active'] = 1;

            if ($membership_data['is_active']) {
                $this->load->model('Services_model');
                $this->Services_model->updateServiceStatus($service_id, 1);
            }
        }

        $save_data = array(
            'services' => $membership_data['services_array'],
        );
        $validate_data = $this->Memberships_model->validateMembership($membership_id, $save_data);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $this->Memberships_model->save($membership_id, $validate_data['data']);
            $this->system_messages->addMessage(View::MSG_SUCCESS,
                                               l('success_item_service_activate', Memberships_model::MODULE_GID));
        }

        $this->view->setRedirect($redirect_url);
    }

    /**
     * Deactivate membership service
     *
     * @param integer $membership_id membership identifier
     * @param integer $service_id    service template identifier
     *
     * @return void
     */
    public function deactivate_service($membership_id, $service_id)
    {
        $membership_id = intval($membership_id);
        $service_id = intval($service_id);

        $membership_data = $this->Memberships_model->formatMembership(
                $this->Memberships_model->getMembershipById($membership_id));

        if ($membership_data['is_active']) {
            $this->load->model('Services_model');
            $this->Services_model->updateServiceStatus($service_id, 0);
        }

        $membership_data['services_array'][$service_id]['is_active'] = 0;

        $success = array();
        if (!$this->Memberships_model->hasServices($membership_data)) {
            $deactivation_result = $this->Memberships_model->deactivate($membership_id);
            if (empty($deactivation_result['errors'])) {
                $success[] = l('success_item_deactivate', Memberships_model::MODULE_GID);
            }
        }

        $save_data = array(
            'services' => $membership_data['services_array'],
        );
        $validate_data = $this->Memberships_model->validateMembership($membership_id, $save_data);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $this->Memberships_model->save($membership_id, $validate_data['data']);
            $success[] = l('success_item_service_deactivate', Memberships_model::MODULE_GID);
        }
        if (!empty($success)) {
            $this->system_messages->addMessage(View::MSG_SUCCESS, $success);
        }

        $url = site_url() . 'admin/memberships/edit/' . $membership_id;
        $this->view->setRedirect($url);
    }

    /**
     * Activate membership
     *
     * @return void
     */
    public function ajax_activate()
    {
        $membership_id = filter_input(INPUT_POST, 'id', FILTER_VALIDATE_INT);
        if (!$membership_id) {
            $this->view->assign('errors', l('error_item_activate', Memberships_model::MODULE_GID));
        } else {
            $result = $this->Memberships_model->activate($membership_id);
            if (empty($result['errors'])) {
                $this->view->assign('success', l('success_item_activate', Memberships_model::MODULE_GID));
            } else {
                $errors = array();
                foreach ($result['errors'] as $error) {
                    $errors[] = l('error_' . $error, Memberships_model::MODULE_GID) ? : $error;
                }
                $this->view->assign('errors', $errors);
            }
        }
        $this->view->render();
    }

    /**
     * Deactivate membership
     *
     * @return void
     */
    public function ajax_deactivate()
    {
        $membership_id = filter_input(INPUT_POST, 'id', FILTER_VALIDATE_INT);
        $result = $this->Memberships_model->deactivate($membership_id);
        if (empty($result['errors'])) {
            $this->view->assign('success', l('success_item_deactivate', Memberships_model::MODULE_GID));
        } else {
            $errors = array();
            foreach ($result['errors'] as $error) {
                $errors[] = l('error_' . $error, Memberships_model::MODULE_GID) ? : $error;
            }
            $this->view->assign('errors', $errors);
        }
        $this->view->render();
    }

    /**
     * Activate/deactivate service (internal)
     *
     * @param boolean $status status of service activity
     *
     * @return void
     */
    private function serviceActivity($status)
    {
        if ($status) {
            $action = 'activate';
        } else {
            $action = 'deactivate';
        }
        $membership_id = filter_input(INPUT_POST, 'membership_id', FILTER_VALIDATE_INT);
        $service_id = filter_input(INPUT_POST, 'service_id', FILTER_VALIDATE_INT);
        if (!$membership_id) {
            $errors = l('error_service_' . $action, Memberships_model::MODULE_GID);
            $this->view->assign('errors', $errors);
        } else {
            $result = $this->Memberships_model->{$action . 'Service'}((int) $membership_id, (int) $service_id);
            if (empty($result['errors'])) {
                $success = l('success_item_service_' . $action, Memberships_model::MODULE_GID);
                $this->view->assign('success', $success);
            } else {
                $errors = array();
                foreach ($result['errors'] as $error) {
                    $errors[] = l('error_' . $error, Memberships_model::MODULE_GID) ? : $error;
                }
                $this->view->assign('errors', $errors);
            }
        }
        
        $this->view->render();
    }

    /**
     * Activate service (internal)
     *
     * @param boolean $status status of service activity
     *
     * @return void
     */
    public function ajax_activate_service()
    {
        $this->serviceActivity(true);
    }

    /**
     * Deactivate service (internal)
     *
     * @param boolean $status status of service activity
     *
     * @return void
     */
    public function ajax_deactivate_service()
    {
        $this->serviceActivity(false);
    }

}
