<?php

namespace Pg\Modules\Menu\Models;

/**
 * Menu module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Menu install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Menu
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Menu_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        // admin menu
        'admin_menu' => array(
            "action" => "none",
            "name" => "Admin menu",
            "items" => array(
                'interface-items' => array(
                    "action" => "none",
                    "items" => array(
                        'admin-menus-item' => array("action" => "create", 'link' => 'admin/menu', 'status' => 1, 'sorter' => 4),
                    ),
                ),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators_methods = array(
        array("module" => 'menu', 'method' => 'index', 'is_default' => 1),
    );

    /**
     * Constructor
     *
     * @return Menu_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->ci->load->model('Install_model');
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('menu', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        ///// install moderators permissions
        $this->ci->load->model("Moderators_model");

        foreach ($this->moderators_methods as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Import languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('menu', 'moderators', $langs_ids);

        ///// install moderators permissions
        $this->ci->load->model("Moderators_model");
        $params['where']['module'] = 'menu';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method['method']]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model("Moderators_model");
        $params['where']['module'] = 'menu';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        ///// delete moderation methods in moderators
        $this->ci->load->model("Moderators_model");
        $params['where']['module'] = 'menu';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of conjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        // Remove old indicators
        $this->ci->load->model('Cronjob_model');
        $cron_data = array(
            'name' => 'Remove old menu indicators',
            'module' => 'menu',
            'model' => 'Indicators_model',
            'method' => 'delete_old',
            'cron_tab' => '0 3 * * *',
            'status' => '1',
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);
    }

    /**
     * Uninstall data of conjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array();
        $cron_data['where']['module'] = 'menu';
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        $lang_dm_data = array(
            'module' => 'menu',
            'model' => 'Indicators_model',
            'method_add' => 'lang_dedicate_module_callback_add',
            'method_delete' => 'lang_dedicate_module_callback_delete',
        );
        $this->ci->pg_language->add_dedicate_modules_entry($lang_dm_data);
        $this->ci->load->model("menu/models/Indicators_model");
        foreach ($this->ci->pg_language->languages as $value) {
            $this->ci->Indicators_model->lang_dedicate_module_callback_add($value['id']);
        }

        return;
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        $lang_dm_data['where'] = array(
            'module' => 'indicators',
            'model' => 'Indicators_model',
        );
        $this->ci->pg_language->delete_dedicate_modules_entry($lang_dm_data);
    }

}
