<?php

namespace Pg\Modules\Mobile\Models;

use Pg\Libraries\Setup;

/**
 * Mobile module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Mobile install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mobile
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Mobile_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;
    private $modules_data;

    /**
     * Constructor
     *
     * @return Mobile_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        // Mobile url can't be handled by the menu module because of the modifications which it may perform 
        // so we made it "external"
        $this->modules_data = Setup::getModuleData('mobile', Setup::TYPE_MODULES_DATA);
        $this->modules_data['menu']['user_footer_menu']['items']['footer-menu-mobile-item']['link'] = site_url() . 'm';
    }

    /**
     * Install data of menu mobile
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]['id'] = linked_install_set_menu($gid, $menu_data['action'],
                                                                              isset($menu_data['name']) ? $menu_data['name'] : '');
            linked_install_process_menu_items($this->modules_data['menu'], 'create', $gid, 0,
                                              $this->modules_data['menu'][$gid]['items']);
        }
    }

    /**
     * Import languages of menu mobile
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('mobile', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu'], 'update', $gid, 0,
                                              $this->modules_data['menu'][$gid]['items'], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu mobile
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], 'export', $gid, 0,
                                                      $this->modules_data['menu'][$gid]['items'], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array('menu' => $return);
    }

    /**
     * Uninstall data of menu mobile
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->modules_data['menu'][$gid]['items']);
            }
        }
    }

    /**
     * Install data of uploads module
     *
     * @return void
     */
    public function install_uploads()
    {
        // upload config
        $this->ci->load->model("uploads/models/Uploads_config_model");

        $watermark_ids = array();

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = array(
                "gid" => $upload_data["gid"],
                "name" => $upload_data["name"],
                "min_height" => isset($upload_data["min_height"]) ? $upload_data["min_height"] : 0,
                "min_width" => isset($upload_data["min_width"]) ? $upload_data["min_width"] : 0,
                "max_height" => $upload_data["max_height"],
                "max_width" => $upload_data["max_width"],
                "max_size" => $upload_data["max_size"],
                "name_format" => $upload_data["name_format"],
                "file_formats" => serialize((array) $upload_data["file_formats"]),
                "default_img" => $upload_data["default_img"],
                "date_add" => date("Y-m-d H:i:s"),
            );
            $config_id = $this->ci->Uploads_config_model->save_config(null, $config_data);

            $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid("image-wm");
            $wm_id = isset($wm_data["id"]) ? $wm_data["id"] : 0;

            foreach ((array) $upload_data["thumbs"] as $thumb_gid => $thumb_data) {
                if (isset($thumb_data["watermark"])) {
                    if (!isset($watermark_ids[$thumb_data["watermark"]])) {
                        $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid($thumb_data["watermark"]);
                        $watermark_ids[$thumb_data["watermark"]] = isset($wm_data["id"]) ? $wm_data["id"] : 0;
                    }
                    $watermark_id = $watermark_ids[$thumb_data["watermark"]];
                } else {
                    $watermark_id = 0;
                }

                $thumb_data["config_id"] = $config_id;
                $thumb_data["prefix"] = $thumb_gid;
                $thumb_data["effect"] = "none";
                $thumb_data["watermark_id"] = $watermark_id;

                $validate_data = $this->ci->Uploads_config_model->validate_thumb(null, $thumb_data);
                if (!empty($validate_data["errors"])) {
                    continue;
                }
                $this->ci->Uploads_config_model->save_thumb(null, $validate_data["data"]);
            }
        }
    }

    /**
     * Uninstall data of uploads module
     *
     * @return void
     */
    public function deinstall_uploads()
    {
        $this->ci->load->model("uploads/models/Uploads_config_model");

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = $this->ci->Uploads_config_model->get_config_by_gid($upload_data["gid"]);
            if (!empty($config_data["id"])) {
                $this->ci->Uploads_config_model->delete_config($config_data["id"]);
            }
        }
    }

    /**
     * Install data of social networking module
     *
     * @return void
     */
    public function install_social_networking()
    {
        // add social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        foreach ((array) $this->modules_data['social_networking'] as $social_networking_data) {
            $validation_data = $this->ci->Social_networking_pages_model->validate_page(null, $social_networking_data);
            if (!empty($validation_data['errors'])) {
                continue;
            }
            $this->ci->Social_networking_pages_model->save_page(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall data of social networking module
     *
     * @return void
     */
    public function deinstall_social_networking()
    {
        // delete social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        $this->ci->Social_networking_pages_model->delete_pages_by_controller('mobile');
    }

    protected function setPaths()
    {
        $mobile_path = SITE_PHYSICAL_PATH . 'm/';
        $files = array(
            array(
                'path' => $mobile_path . 'index.html',
                'replace' => array(
                    '[site_root]' => '/' . SITE_SUBFOLDER . 'm/',
                ),
            ),
            array(
                'path' => $mobile_path . 'scripts/app.js',
                'replace' => array(
                    '[site_url]' => preg_replace('#https?:#', '', SITE_VIRTUAL_PATH),
                ),
            ),
        );
        foreach ($files as $file) {
            $file_contents = file_get_contents($file['path']);
            if ($file_contents) {
                $file_contents = str_replace(
                        array_keys($file['replace']), array_values($file['replace']), $file_contents
                );
                file_put_contents($file['path'], $file_contents);
            }
        }
    }

    protected function resetPaths()
    {
        $mobile_path = SITE_PHYSICAL_PATH . 'm/';
        $files = array(
            array(
                'path' => $mobile_path . 'index.html',
                'replace' => array(
                    '/' . SITE_SUBFOLDER . 'm/' => '[site_root]',
                ),
            ),
            array(
                'path' => $mobile_path . 'scripts/app.js',
                'replace' => array(
                    SITE_VIRTUAL_PATH => '[site_url]',
                ),
            ),
        );
        foreach ($files as $file) {
            $file_contents = file_get_contents($file['path']);
            if ($file_contents) {
                $file_contents = str_replace(array_keys($file['replace']), array_values($file['replace']),
                                                                                        $file_contents);
                file_put_contents($file['path'], $file_contents);
            }
        }
    }

    public function _arbitrary_installing()
    {
        $this->setPaths();
    }

    public function _arbitrary_deinstalling()
    {
        $this->resetPaths();
    }

}
