<?php

namespace Pg\Modules\Moderation\Models;

/**
 * Moderation module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('MODERATION_BADWORDS_TABLE', DB_PREFIX . 'moderation_badwords');

/**
 * Moderation badwords model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Moderation
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Moderation_badwords_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Badwords settings cahce
     *
     * @var array
     */
    public $badwords_params_cache = array();

    /**
     * Constructor
     *
     * @return Moderation_badwords_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->ci = &get_instance();
    }

    /**
     * Return badwords from data source as array
     *
     * @return array
     */
    public function get_badwords()
    {
        $this->ci->db->select('id, original')->from(MODERATION_BADWORDS_TABLE)->order_by("search");
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            foreach ($result as $item) {
                $list[] = get_object_vars($item);
            }

            return $list;
        } else {
            return array();
        }
    }

    /**
     * Save badword to data source
     *
     * @param string $word bad word
     *
     * @return array
     */
    public function set_badword($word)
    {
        $errors = array();
        $word_arr = explode(" ", trim(strip_tags($word)));
        foreach ($word_arr as $word) {
            $data["original"] = trim($word);
            $data["original"] = str_replace(array("_", ",", ":", ";"), " ", $data["original"]);
            $data["original"] = preg_replace("/[\n\t]/u", "", $data["original"]);

            $data["search"] = mb_strtoupper(preg_replace("/[^\pL\s0-9\.]/u", "", $data["original"]), 'utf8');
            if (!strlen($data["original"]) || !strlen($data["search"])) {
                continue;
            }

            $data["search_len"] = strlen($data["search"]);
            $data["search_ord"] = ord($data["search"]);

            $similar_arr = $this->get_similar_words($data["search"]);
            if (count($similar_arr) > 0) {
                $errors[] = l('badwords_in_base_exists', 'moderation') . ": " . implode(", ", $similar_arr);
            } else {
                $this->ci->db->insert(MODERATION_BADWORDS_TABLE, $data);
            }
        }
        $this->badwords_params_cache = array();

        return $errors;
    }

    /**
     * Refresh settings of exists baswords
     *
     * @return void
     */
    public function remap_badwords()
    {
        $this->ci->db->select('id, search')->from(MODERATION_BADWORDS_TABLE);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            foreach ($result as $item) {
                $data["search_len"] = strlen($item->search);
                $data["search_ord"] = ord($item->search);
                $this->ci->db->where("id", $item->id);
                $this->ci->db->update(MODERATION_BADWORDS_TABLE, $data);
            }
        }
    }

    /**
     * Remove badword from data source by identifier
     *
     * @param integer $id badword identifier
     *
     * @return void
     */
    public function delete_badword($id)
    {
        $this->ci->db->where("id", intval($id));
        $this->ci->db->delete(MODERATION_BADWORDS_TABLE);
        $this->badwords_params_cache = array();
    }

    /**
     * Return similar words
     *
     * @param string $search_word input word
     *
     * @return array
     */
    public function get_similar_words($search_word)
    {
        $this->ci->db->select('id, original')->from(MODERATION_BADWORDS_TABLE)->where("search", $search_word);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            foreach ($result as $item) {
                $list[] = $item->original;
            }

            return $list;
        } else {
            return array();
        }
    }

    /**
     * Return badwords settings
     *
     * @return array
     */
    public function get_badwords_params()
    {
        if (empty($this->badwords_params_cache)) {
            $return = array();
            $this->ci->db->select("DISTINCT search_len")->from(MODERATION_BADWORDS_TABLE);
            $result = $this->ci->db->get()->result();
            if (!empty($result)) {
                foreach ($result as $item) {
                    $return["len"][] = $item->search_len;
                }
            }
            $this->ci->db->select("DISTINCT search_ord")->from(MODERATION_BADWORDS_TABLE);
            $result = $this->ci->db->get()->result();
            if (!empty($result)) {
                foreach ($result as $item) {
                    $return["ord"][] = $item->search_ord;
                }
            }
        }
        $this->badwords_params_cache = $return;

        return $this->badwords_params_cache;
    }

    /**
     * Search badwords in text
     *
     * @param string $text input text
     *
     * @return array
     */
    public function search_in_text($text)
    {
        $text = str_replace(array("_", ",", ":", ";"), " ", $text);
        $text = preg_replace("/[\n\t]/u", " ", $text);
        $text = mb_strtoupper(preg_replace("/[^\pL\s0-9\.]/u", "", $text), 'utf8');
        $text_array = preg_split("/[\s]+/", $text);
        $text_array = array_unique($text_array);

        $this->ci->db->select("id, search")->from(MODERATION_BADWORDS_TABLE)->where_in("search", $text_array);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            $return["text"] = " " . $text . " ";
            foreach ($result as $item) {
                $return["text"] = str_replace(" " . $item->search . " ",
                                              " <span class='bwmark'>" . $item->search . "</span> ", $return["text"]);
            }
            $return["count"] = count($result);
        } else {
            $return["text"] = $text;
            $return["count"] = 0;
        }

        return $return;
    }

    /**
     * Check exists badwords in text
     *
     * @param string $mtype moderation type
     * @param string $text  input text
     *
     * @return integer
     */
    public function check_badwords($mtype, $text)
    {
        $this->ci->load->model('moderation/models/Moderation_type_model');
        $type_data = $this->ci->Moderation_type_model->get_type_by_name($mtype);
        if (empty($type_data["check_badwords"])) {
            return 0;
        }
        $text = str_replace(array("_", ",", ":", ";"), " ", $text);
        $text = preg_replace("/[\n\t]/u", " ", $text);
        $text = mb_strtoupper(preg_replace("/[^\pL\s0-9\.]/u", "", $text), 'utf8');
        $text_array = preg_split("/[\s]+/", $text);
        $text_array_mb = array_unique($text_array);

        $this->ci->db->select("COUNT(*) as cnt")->from(MODERATION_BADWORDS_TABLE)->where_in("search", $text_array_mb);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

}
