<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Egold payment system driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Egold_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array();

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        "seller_id" => array("type" => "text", "content" => "string", "size" => "middle"),
        "seller_name" => array("type" => "text", "content" => "string", "size" => "middle"),
            //"secret_word" => array("type"=>"text", "content"=>"string", "size"=>"middle")
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
        "PAYEE_ACCOUNT" => "seller_id",
        "PAYMENT_AMOUNT" => "amount",
        "PAYMENT_ID" => "id_payment",
    );

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://www.e-gold.com/sci_asp/payments.asp';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = '';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        switch ($payment_data["currency_gid"]) {
            case "USD": $currency = 1;
                break;
            case "EUR": $currency = 85;
                break;
            case "GBP": $currency = 44;
                break;
            case "CAD": $currency = 2;
                break;
            case "JPY": $currency = 81;
                break;
            case "AUD": $currency = 61;
                break;
            default: $currency = 1;
        }

        $send_data = array(
            "PAYEE_ACCOUNT" => $system_settings["settings_data"]["seller_id"],
            "PAYEE_NAME" => $system_settings["settings_data"]["seller_name"],
            "PAYMENT_METAL_ID" => "1",
            "PAYMENT_UNITS" => $currency,
            "PAYMENT_AMOUNT" => $payment_data["amount"],
            "PAYMENT_ID" => $payment_data["id_payment"],
            "STATUS_URL" => site_url() . "payments/responce/egold",
            "PAYMENT_URL" => site_url(),
            "NOPAYMENT_URL" => site_url(),
        );

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "post");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => array(), "type" => "exit");

        foreach ($this->variables as $payment_var => $site_var) {
            $return["data"][$site_var] = isset($payment_data[$payment_var]) ? $this->ci->input->xss_clean($payment_data[$payment_var]) : "";
        }

        //// get status
        $return["data"]["status"] = 0;
        if (isset($payment_data['PAYMENT_ID'])) {
            $return["data"]["status"] = -1;
        } else {
            $return["data"]["status"] = 1;
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

}
