<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Scrill payment system driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Skrill_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array(
        'gid' => 'skrill',
        'name' => 'Skrill',
        'settings_data' => 'a:2:{s:9:"seller_id";s:9:"seller id";s:11:"seller_word";s:11:"secret word";}',
        'is_test' => 1,
        'in_use' => 0,
        'tarifs_type' => 0,
        'tarifs_editable' => 0,
        'disable' => 0,
        'date_add' => '',
    );

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        'seller_id' => array("type" => "text", "content" => "string", "size" => "middle"),
        'secret_word' => array("type" => "text", "content" => "string", "size" => "middle"),
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
        'pay_to_email' => 'seller_id',
        'merchant_id' => 'merchant_id',
        'mb_transaction_id' => 'id_payment',
        'mb_amount' => 'amount',
        'mb_currency' => 'currency',
        'md5sig' => 'hash',
        'status' => 'status',
    );

    /**
     * Available languages
     *
     * @var array
     */
    protected $available_languages = array(
        'EN', 'DE', 'ES', 'FR', 'IT', 'PL',
        'GR', 'RO', 'RU', 'TR', 'CN', 'CZ',
        'NL', 'DA', 'SV', 'FI',
    );

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://www.moneybookers.com/app/payment.pl';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = 'https://www.moneybookers.com/app/test_payment.pl';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        $send_data = array(
            'pay_to_email' => $system_settings["settings_data"]["seller_id"],
            'tansaction_id' => $payment_data["id_payment"],
            'amount' => $payment_data["amount"],
            'currency' => $payment_data["currency_gid"],
            'detail1_description' => l('field_payment_name', 'payments'),
            'detail1_text' => $payment_data["payment_data"]["name"],
            'return_url' => site_url(),
            'status_url' => site_url() . "payments/responce/skrill",
            'cancel_url' => site_url(),
        );

        $current_lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);
        $current_lang['code'] = strtoupper($current_lang['code']);
        if (in_array($current_lang['code'], $this->available_languages)) {
            $send_data['language'] = $current_lang['code'];
        }

        $user_id = $this->ci->session->userdata('user_id');

        $this->ci->load->model('Users_model');

        $user = $this->ci->Users_model->get_user_by_id($user_id);
        $send_data['pay_from_email'] = $user['contact_email'];
        $send_data['first_name'] = $user['fname'];
        $send_data['last_name'] = $user['sname'];
        $send_data['phone_number'] = $user['contact_phone'];

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "post");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => array(), "type" => "exit");

        foreach ($this->variables as $payment_var => $site_var) {
            $return['data'][$site_var] = isset($payment_data[$payment_var]) ? $this->ci->input->xss_clean($payment_data[$payment_var]) : "";
        }

        $error = false;

        if ($system_settings["settings_data"]["seller_id"] != $return['data']['seller_id']) {
            $error = true;
        }

        $this->ci->load->model("Payments_model");
        $site_payment_data = $this->ci->Payments_model->get_payment_by_id($return['data']['id_payment']);
        if (floatval($site_payment_data["amount"]) != floatval($return['data']['amount'])) {
            $error = true;
        }

        $server_side_hash = $return['data']['merchant_id'] . $return['data']['id_payment'] .
                strtoupper(md5($system_settings['settings_data']['secret_word'])) .
                $return['data']['amount'] . $return['data']['currency'] . $return['data']['status'];
        $server_side_hash = strtoupper(trim(md5($server_side_hash)));
        if ($server_side_hash != $return['data']['hash']) {
            $error = true;
        }

        if ($error) {
            $return["data"]["status"] = -1;
        } else {
            switch ($return["data"]["status"]) {
                case 2: $return["data"]["status"] = 1;
                    break;
                case 0: $return["data"]["status"] = 0;
                    break;
                default: $return["data"]["status"] = -1;
                    break;
            }
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

}
