<?php

namespace Pg\Modules\Services\Models;

/**
 * Services users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Users services model
 *
 * @package PG_Dating
 * @subpackage application
 *
 * @category	modules
 *
 * @copyright Pilot Group <http://www.pilotgroup.net/>
 * @author Alexander Batukhtin <abatukhtin@pilotgroup.net>
 *
 * @version $Revision: 2 $ $Date: 2012-09-20 09:47:07 +0300 (Пт, 20 сент 2012) $ $Author: abatukhtin $
 * */
if (!defined('SERVICES_USERS_TABLE')) {
    define('SERVICES_USERS_TABLE', DB_PREFIX . 'services_users');
}

/**
 * Services users model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Services
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Services_users_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Services properties in data source
     *
     * @var array
     */
    private $fields = array(
        'id',
        'id_user',
        'service_gid',
        'template_gid',
        'service_data',
        'template_data',
        'payment_data',
        'available_data',
        'service_end_date',
        'id_users_membership',
        'status',
        'date_created',
        'date_modified',
        'date_expired',
    );

    /**
     * Constructor
     *
     * @return users object
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return services count
     *
     * @param integer $user_id    user identifier
     * @param string  $count_name count name
     *
     * @return integer
     */
    public function get_count($user_id = 0, $count_name)
    {
        $count = 0;
        $services = $this->get_services_list(array('id_user' => $user_id, 'status' => '1'));
        foreach ($services as $id => $value) {
            $data = @unserialize($value['service_data_array']);
            if (isset($data[$count_name])) {
                $count = $count + $data[$count_name];
            }
        }

        return $count;
    }

    public function save_service($service_id = null, $attrs = array())
    {
        if (!empty($attrs["service_data"])) {
            $attrs["service_data"] = serialize($attrs["service_data"]);
        }
        if (!empty($attrs["template_data"])) {
            $attrs["template_data"] = serialize($attrs["template_data"]);
        }
        if (isset($attrs["payment_data"]) && is_array($attrs["payment_data"])) {
            $attrs["payment_data"] = serialize($attrs["payment_data"]);
        }
        if (isset($attrs["available_data"]) && is_array($attrs["available_data"])) {
            $attrs["available_data"] = serialize($attrs["available_data"]);
        }
        foreach ($attrs as $field => $attr) {
            if (!in_array($field, $this->fields)) {
                unset($attrs[$field]);
            }
        }
        if (is_null($service_id)) {
            $attrs["date_created"] = $attrs["date_modified"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(SERVICES_USERS_TABLE, $attrs);
            $service_id = $this->ci->db->insert_id();
        } else {
            $attrs["date_modified"] = date("Y-m-d H:i:s");
            $this->ci->db->where('id', $service_id);
            $this->ci->db->update(SERVICES_USERS_TABLE, $attrs);
        }

        return $service_id;
    }

    public function update_service($params, $attrs = array())
    {
        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            $this->ci->db->where($params["where"]);
        }
        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }
        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($attrs["service_data"]) && is_array($attrs["service_data"])) {
            $attrs["service_data"] = serialize($attrs["service_data"]);
        }
        if (!isset($attrs["date_modified"])) {
            $attrs["date_modified"] = date("Y-m-d H:i:s");
        }
        $this->ci->db->update(SERVICES_USERS_TABLE, $attrs);

        return $this->ci->db->affected_rows();
    }

    public function get_services_list($params = array(), $order_by = null, $filter_object_ids = null, $lang_id = '')
    {
        $data = array();
        $this->ci->db->select(implode(", ", $this->fields))->from(SERVICES_USERS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            $this->ci->db->where($params["where"]);
        }
        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }
        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }
        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }
        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();

        return $this->format_services($results, $lang_id);
    }

    public function format_services($user_services, $lang_id = '')
    {
        $result = array();
        foreach ($user_services as $user_service) {
            $id = $user_service['id'];
            $result[$id] = $user_service;

            if (!empty($user_service['service_data'])) {
                $result[$id]['service_data_array'] = @unserialize($user_service['service_data']);
                $result[$id]['service_data_array']["data_admin_array"] = unserialize($result[$id]['service_data_array']["data_admin"]);
            }

            if (!empty($user_service['template_data'])) {
                $this->ci->load->model('Services_model');
                $result[$id]['template_data_array'] = @unserialize($user_service['template_data']);

                if (!empty($result[$id]['template_data_array']["data_admin"])) {
                    $result[$id]['template_data_array']["data_admin_array"] = unserialize($result[$id]['template_data_array']["data_admin"]);
                }

                if (!empty($result[$id]['template_data_array']["data_user"])) {
                    $result[$id]['template_data_array']["data_user_array"] = unserialize($result[$id]['template_data_array']["data_user"]);
                }

                if (!empty($result[$id]['template_data_array']["data_admin"])) {
                    $result[$id]['template_data_array']["data_membership_array"] = unserialize($result[$id]['template_data_array']["data_membership"]);
                }
            }

            if (!empty($user_service['payment_data'])) {
                $result[$id]['payment_data_array'] = @unserialize($user_service['payment_data']);
            }

            if (!empty($user_service['available_data'])) {
                $result[$id]['available_data_array'] = @unserialize($user_service['available_data']);
            }

            if (!(empty($result[$id]['service_data']))) {
                $result[$id]['name'] = l('service_name_' . $result[$id]['service_data_array']['id'], 'services',
                                         $lang_id);
                if (empty($result[$id]['name']) && $result[$id]['template_data_array']) {
                    $result[$id]['name'] = $result[$id]['template_data_array']['name'];
                }
            } else {
                $this->ci->load->model('Services_model');
                $service = $this->ci->Services_model->format_service($this->ci->Services_model->get_service_by_gid($user_service['service_gid']));
                $result[$id]['name'] = l('service_name_' . $service['id'], 'services', $lang_id);
            }

            if ($user_service['id'] && strtotime($user_service['date_expired']) > 0) {
                $now = new \DateTime('now');
                $expires_date = new \DateTime($user_service['date_expired']);
                $result[$id]['left'] = $now->diff($expires_date, false);
                $days_left = 0;
                if ($result[$id]['left']->days) {
                    $days_left = $result[$id]['left']->days;
                }
                $days_left += (int) ($result[$id]['left']->h > 12);
                if ($result[$id]['left']->invert) {
                    $days_left = -$days_left;
                }
                $result[$id]['is_expired'] = $result[$id]['left']->invert;
                $result[$id]['date_expires'] = $expires_date->format('Y-m-d H:i:s');
                $result[$id]['days_left'] = $days_left;
            }

            $result[$id]['description'] = !(empty($result[$id]['service_data_array'])) ?
                    l('service_name_' . $result[$id]['service_data_array']['id'] . '_description', 'services', $lang_id) : '';

            $result[$id]['is_active'] = $user_service['status'];
        }

        return $result;
    }

    public function get_service_by_id($id)
    {
        $this->ci->db->select(implode(", ", $this->fields))->from(SERVICES_USERS_TABLE)->where('id', $id);
        $results = $this->format_services($this->ci->db->get()->result_array());

        return array_shift($results);
    }

    public function get_user_service_by_id($id_user, $id)
    {
        $this->ci->db->select(implode(", ", $this->fields))
                ->from(SERVICES_USERS_TABLE)
                ->where('id', $id)
                ->where('id_user', $id_user);

        $results = $this->format_services($this->ci->db->get()->result_array());

        return array_shift($results);
    }

    /**
     * Copy service to other templates
     *
     * Method for combined
     *
     * @param array $user_service  user service data
     * @param array $template_gids template GUIDs
     *                             return void
     */
    public function copyService($user_service, $template_gids)
    {
        foreach ($template_gids as $template_gid) {
            $user_service_data = array(
                'id_user' => $user_service['id'],
                'service_gid' => $user_service['service_gid'],
                'template_gid' => $template_gid,
                'service_data' => $user_service['service_data'],
                'template_data' => $user_service['template_data'],
                'payment_data' => $user_service['payment_data'],
                'id_users_package' => $user_service['users_package_id'],
                'id_users_membership' => $user_service['users_membership_id'],
                'status' => 1,
            );
            $this->ci->Services_users_model->save_service(null, $user_service_data);
        }
    }

    /**
     * Remove services of user's membership
     *
     * @param integer $user_membership_id user membership identifier
     *
     * @return void
     */
    public function deleteMembershipServices($user_membership_id)
    {
        $this->ci->db->where('id_users_membership', $user_membership_id);
        $this->ci->db->delete(SERVICES_USERS_TABLE);
    }

}
