<?php

namespace Pg\Modules\Start\Controllers;

/**
 * Start module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;
use Pg\Libraries\Acl\Action\Admin as AdminAction;
use Pg\Libraries\Acl\Resource\Admin as AdminResource;

/**
 * Start admin side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Start
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_start extends \Controller
{

    /**
     * Numeric settings
     *
     * @var array
     */
    private $numerics_settings = array(
        'start' => array(
            array('name' => 'admin_items_per_page', 'type' => 'int'),
            array('name' => 'index_items_per_page', 'type' => 'int'),
        ),
    );

    /**
     * Formats settings
     *
     * @var array
     */
    private $formats_settings = array(
        'start' => array(
            array('name' => 'use_phone_format', 'type' => 'checkbox'),
            array('name' => 'phone_format', 'type' => 'text'),
            array('name' => 'base_currency', 'type' => 'text'),
        ),
    );

    /**
     * Widgets settings
     *
     * @var array
     */
    private $widgets_settings = array(
    );

    /**
     * Other settings
     *
     * @var array
     */
    private $other_settings = array(
    );

    /**
     * Date format settings
     *
     * @var array
     */
    private $date_format_settings = array(
        array('name' => 'date_literal', 'type' => 'text'),
        array('name' => 'date_literal_short', 'type' => 'text'),
        array('name' => 'date_time_literal', 'type' => 'text'),
        array('name' => 'date_time_literal_short', 'type' => 'text'),
        array('name' => 'time_numeric', 'type' => 'text'),
    );

    /**
     * Date format templates
     *
     * @var array
     */
    private $date_formats_pages = [
        'date_literal' => ['',],
        'date_literal_short' => ['',],
        'date_time_literal' => ['',],
        'date_time_literal_short' => ['',],
        'time_numeric' => ['',],
    ];

    /**
     * Constructor
     *
     * @return Admin_start
     */
    public function __construct()
    {
        parent::__construct();

        $is_installed = array(
            'banners' => $this->pg_module->is_module_active('banners'),
            'listings' => $this->pg_module->is_module_active('listings'),
            'mailbox' => $this->pg_module->is_module_active('mailbox'),
            'moderation' => $this->pg_module->is_module_active('moderation'),
            'news' => $this->pg_module->is_module_active('news'),
            'payments' => $this->pg_module->is_module_active('payments'),
            'polls' => $this->pg_module->is_module_active('polls'),
            'services' => $this->pg_module->is_module_active('services'),
            'users' => $this->pg_module->is_module_active('users'),
        );

        if ($is_installed['listings']) {
            $this->numerics_settings['listings'] = array(
                array('name' => 'rows_per_page', 'type' => 'int'),
            );
        }
        
        if ($is_installed['polls']) {
            $this->widgets_settings['listings'][] = array('name' => 'use_poll_in_search', 'type' => 'checkbox');
            $this->widgets_settings['listings'][] = array('name' => 'show_mortgage_calc', 'type' => 'checkbox');
        }

        if ($is_installed['mailbox']) {
            $this->numerics_settings['mailbox'] = array(
                array('name' => 'items_per_page', 'type' => 'int'),
            );
        }

        if ($is_installed['news']) {
            $this->numerics_settings['news'] = array(
                array('name' => 'userside_items_per_page', 'type' => 'int'),
            );
        }

        if ($is_installed['payments']) {
            $this->numerics_settings["payments"] = array(
                array('name' => 'items_per_page', 'type' => 'int'),
            );
        }

        if ($is_installed['banners']) {
            $this->numerics_settings['banners'] = array(
                array('name' => 'items_per_page', 'type' => 'int'),
            );
        }
    }

    /**
     * Display home page information
     *
     * @return void
     */
    public function index()
    {
        if ($this->session->userdata("auth_type") != 'admin') {
            redirect(site_url() . "admin/ausers/login");
        }

        $this->view->setHeader(l("header_admin_homepage", 'start'));

        if ($this->session->userdata("user_type") == "moderator") {
            $this->view->render('index_moderator');
        } else {
            $this->view->render('index');
        }
    }

    /**
     * Display error page
     *
     * @param string $error_type     error type
     * @param string $request_method request method
     *
     * @return void
     */
    public function error($error_type = '', $request_method = '')
    {
        if ($request_method == 'ajax') {
            $this->view->assign('ajax', '1');
        }
        $this->view->render('error');
    }

    /**
     * Modules installer
     *
     * @return void
     */
    public function mod_login()
    {
        $data["action"] = site_url() . "admin/install/login";
        $this->view->assign("data", $data);

        $this->view->setHeader(l("header_modinstaller_login", 'start'));
        $this->view->render('modules_login');
    }

    /**
     * Manage menu
     *
     * @param string $menu_item_gid menu item guid
     *
     * @return void
     */
    public function menu($menu_item_gid)
    {
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', $menu_item_gid);

        // add link to menu
        $menu_data = $this->Menu_model->get_menu_by_gid('admin_menu');
        $menu_item = $this->Menu_model->get_menu_item_by_gid($menu_item_gid, $menu_data["id"]);

        $user_type = $this->session->userdata("user_type");
        if ($user_type == "admin") {
            $menu_data["check_permissions"] = false;
            $permissions = array();
        } else {
            $menu_data["check_permissions"] = true;
            $permissions = $this->session->userdata("permission_data");
        }
        $menu_items = $this->Menu_model->get_menu_active_items_list($menu_data["id"], $menu_data["check_permissions"],
                                                                    array(), $menu_item["id"], $permissions);

        $this->view->assign("menu", $menu_item);
        $this->view->assign("options", $menu_items);
        $this->view->setHeader($menu_item["value"]);
        $this->view->render('menu_list');

        return;
    }

    /**
     * Manage general settings
     *
     * @param string $section section name
     *
     * @return void
     */
    public function settings($section = "overview")
    {
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');

        if (!$this->pg_module->is_module_active("mailbox")) {
            unset($this->numerics_settings['mailbox']);
        }

        if (!$this->pg_module->is_module_active("news")) {
            unset($this->numerics_settings['news']);
        }

        if (!$this->pg_module->is_module_active("payments")) {
            unset($this->numerics_settings['payments']);
        }

        if (!$this->pg_module->is_module_active("listings")) {
            unset($this->numerics_settings['listings']);
            unset($this->widgets_settings['listings']);
            unset($this->other_settings['listings']);
        }

        if (!$this->pg_module->is_module_active("users")) {
            unset($this->widgets_settings['users']);
        }

        if ($this->input->post('btn_save')) {
            $errors = array();
            $post_data = $this->input->post('settings', true);
            if ($section == 'numerics' || $section == 'formats' || $section == 'widgets') {
                foreach ($this->{$section . '_settings'} as $module => $settings) {
                    foreach ($settings as $var) {
                        if (!isset($post_data[$module][$var['name']])) {
                            continue;
                        }
                        switch ($var['type']) {
                            case 'checkbox':
                                $value = !empty($post_data[$module][$var['name']]);
                                $this->pg_module->set_module_config($module, $var['name'], $value);
                                break;
                            case 'int':
                                $value = !empty($post_data[$module][$var['name']]) ? intval($post_data[$module][$var['name']]) : 0;
                                if (empty($value)) {
                                    $errors[] = l('error_numerics_empty', 'start', '', 'text',
                                                  array('field' => l($module . "_" . $var['name'] . "_field", "start")));
                                } else {
                                    $this->pg_module->set_module_config($module, $var['name'], $value);
                                }
                                break;
                            case 'email':
                                $value = trim(strip_tags($post_data[$section][$var['name']]));
                                if (!empty($value)) {
                                    $this->CI->config->load("reg_exps", true);
                                    $email_expr = $this->CI->config->item("email", "reg_exps");
                                    $chunks = explode(',', $value);
                                    foreach ($chunks as $chunk) {
                                        if (empty($chunk) || !preg_match($email_expr, trim($chunk))) {
                                            $errors[] = l("error_invalid_email", "start");
                                            break;
                                        }
                                    }
                                    if (!empty($errors)) {
                                        break;
                                    }
                                    $this->pg_module->set_module_config($section, $var['name'], $value);
                                } elseif ($section == 'listings' && $var['name'] == 'admin_moderation_emails') {
                                    if (isset($post_data['listings']["moderation_send_mail"])) {
                                        $moderation_send_mail = $post_data['listings']["moderation_send_mail"] ? 1 : 0;
                                    } else {
                                        $moderation_send_mail = $this->pg_module->get_module_config('listings',
                                                                                                    'moderation_send_mail');
                                    }
                                    if ($moderation_send_mail) {
                                        $errors[] = l("error_empty_email", "start");
                                    }
                                }
                                break;
                            default:
                                $value = $post_data[$module][$var['name']];
                                $this->pg_module->set_module_config($module, $var['name'], $value);
                                break;
                        }
                    }
                }
            } else {
                if ($section == 'listings') {
                    $operation_types_enabled = false;
                    $this->load->model('Listings_model');
                    $operation_types = array_reverse($this->Listings_model->operations_arr);
                    foreach ($operation_types as $operation_type) {
                        if ($post_data['for_' . $operation_type . '_enabled']) {
                            $operation_types_enabled = true;
                            break;
                        }
                    }
                    if (!$operation_types_enabled) {
                        $errors[] = l("error_empty_operation_type", "start");
                        unset($post_data);
                    }
                }

                $settings = $this->other_settings[$section];
                foreach ($settings as $var) {
                    if (!isset($post_data[$var['name']])) {
                        continue;
                    }
                    switch ($var['type']) {
                        case 'checkbox':
                            if ($section === 'listings') {
                                $value = (bool) $post_data[$var['name']];
                                if ($var['name'] === 'is_show_contact') {
                                    $this->pg_module->set_module_config('users', $var['name'], $value);
                                } elseif ($var['name'] === 'use_booking') {
                                    $this->pg_module->setModuleDisabled('listings_booking',
                                                                        !(bool) $post_data[$var['name']]);
                                } else {
                                    $this->pg_module->set_module_config($section, $var['name'], $value);
                                }
                            } else {
                                $value = !empty($post_data[$var['name']]);
                                $this->pg_module->set_module_config($section, $var['name'], $value);
                            }
                            break;
                        case 'int':
                            $value = !empty($post_data[$var['name']]) ? intval($post_data[$var['name']]) : 0;
                            if (empty($value)) {
                                $errors[] = l('error_numerics_empty', 'start', '', 'text',
                                              array('field' => l($section . "_" . $var['name'] . "_field", "start")));
                            } else {
                                $this->pg_module->set_module_config($section, $var['name'], $value);
                            }
                            break;
                        case 'email':
                            $value = trim(strip_tags($post_data[$var['name']]));
                            if (!empty($value)) {
                                $this->config->load("reg_exps", true);
                                $email_expr = $this->config->item("email", "reg_exps");
                                $chunks = explode(',', $value);
                                foreach ($chunks as $chunk) {
                                    if (empty($chunk) || !preg_match($email_expr, trim($chunk))) {
                                        $errors[] = l("error_invalid_email", "start");
                                        break;
                                    }
                                }
                                if (!empty($errors)) {
                                    break;
                                }
                                $this->pg_module->set_module_config($section, $var['name'], $value);
                            } elseif ($section == 'listings' && $var['name'] == 'admin_moderation_emails') {
                                if (isset($post_data["moderation_send_mail"])) {
                                    $moderation_send_mail = $post_data["moderation_send_mail"] ? 1 : 0;
                                } else {
                                    $moderation_send_mail = $this->pg_module->get_module_config('listings',
                                                                                                'moderation_send_mail');
                                }
                                if ($moderation_send_mail) {
                                    $errors[] = l("error_empty_email", "start");
                                }
                            }
                            break;
                        default:
                            $value = $post_data[$var['name']];
                            $this->pg_module->set_module_config($section, $var['name'], $value);
                            break;
                    }
                }
            }

            if (empty($errors)) {
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_numerics_data', 'start'));
                $this->view->setRedirect(site_url() . 'admin/start/settings/' . $section);
            } else {
                $this->system_messages->addMessage(View::MSG_ERROR, $errors);
            }
        }

        switch ($section) {
            case 'overview':
                foreach ($this->numerics_settings as $module => $settings) {
                    $settings_data["numerics"][$module]["name"] = l($module . "_settings_module", "start");
                    foreach ($settings as $var) {
                        $settings_data["numerics"][$module]['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($module . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config($module, $var['name']),
                        );
                    }
                }
                foreach ($this->formats_settings as $module => $settings) {
                    $settings_data["formats"][$module]["name"] = l($module . "_settings_module", "start");
                    foreach ($settings as $var) {
                        $settings_data["formats"][$module]['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($module . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config($module, $var['name']),
                        );
                    }
                }
                foreach ($this->widgets_settings as $module => $settings) {
                    $settings_data["widgets"][$module]["name"] = l($module . "_settings_module", "start");
                    foreach ($settings as $var) {
                        $settings_data["widgets"][$module]['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($module . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config($module, $var['name']),
                        );
                    }
                }
                foreach ($this->other_settings as $module => $settings) {
                    $settings_data["other"][$module]["name"] = l($module . "_settings_module", "start");
                    foreach ($settings as $var) {
                        if ('date_formats' === $module) {
                            $value = $this->pg_date->strftime($this->pg_date->get_format($var['name'], 'st'), time());
                        } else {
                            $value = $this->pg_module->get_module_config($module, $var['name']);
                        }
                        $settings_data["other"][$module]['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($module . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $value,
                        );
                    }
                }
                break;
            case 'numerics':
            case 'widgets':
                foreach ($this->{$section . '_settings'} as $module => $settings) {
                    $settings_data[$module]["name"] = l($module . "_settings_module", "start");
                    foreach ($settings as $var) {
                        $settings_data[$module]['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($module . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config($module, $var['name']),
                        );
                    }
                }
                break;
            case 'formats':
                foreach ($this->formats_settings as $module => $settings) {
                    $settings_data[$module]["name"] = l($module . "_settings_module", "start");
                    foreach ($settings as $var) {
                        $options = array();

                        if ($var['name'] == 'base_currency') {
                            if ($this->pg_module->is_module_active("currencies")) {
                                $var['type'] = 'select';
                                $this->load->model("Currencies_model");
                                $currencies = $this->Currencies_model->return_currencies();
                                foreach ($currencies as $currency) {
                                    $options[$currency['gid']] = $currency['gid'];
                                }
                            }
                        }

                        $value = $this->pg_module->get_module_config($module, $var['name']);

                        $settings_data[$module]['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($module . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $value,
                            "options" => $options,
                        );
                    }
                }

                $this->view->assign('date_formats_pages', $this->date_formats_pages);
                $settings_data["name"] = l("date_formats_settings_module", "start");
                foreach ($this->date_format_settings as $var) {
                    $settings_data['vars'][] = array(
                        "field" => $var['name'],
                        "field_name" => l("date_formats_" . $var['name'] . "_field", "start"),
                        "value" => $this->pg_date->strftime($this->pg_date->get_format($var['name'], 'st'), time()),
                    );
                }
                break;
            default:
                $settings_data["name"] = l($section . "_settings_module", "start");
                foreach ($this->other_settings[$section] as $var) {
                    if ($section == 'listings' && $var['name'] == 'admin_moderation_emails') {
                        $admin_moderation_email = array(
                            "field" => $var['name'],
                            "field_name" => l($section . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config('listings', 'admin_moderation_emails'),
                        );
                        $this->view->assign("admin_moderation_emails", $admin_moderation_email);
                    } elseif ($section == 'listings' && $var['name'] == 'is_show_contact') {
                        $settings_data['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($section . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config("users", $var['name']),
                        );
                    } elseif ($section == 'listings' && $var['name'] === 'use_booking') {
                        $settings_data['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($section . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->is_module_active('listings_booking'),
                        );
                    } else {
                        $settings_data['vars'][] = array(
                            "field" => $var['name'],
                            "field_name" => l($section . "_" . $var['name'] . "_field", "start"),
                            'type' => $var['type'],
                            "value" => $this->pg_module->get_module_config($section, $var['name']),
                        );
                    }
                }
                break;
        }

        $this->view->assign('section', $section);
        $this->view->assign('settings_data', $settings_data);
        $this->view->assign('numerics_settings', $this->numerics_settings);
        $this->view->assign('other_settings', $this->other_settings);
        $this->view->setHeader(l('header_settings_numerics_list', 'start'));
        $this->view->render('numerics_list');
    }

    /**
     * Load inline editor for multi-language data
     *
     * @param integer $is_textarea user textarea
     *
     * @return void
     */
    public function lang_inline_editor($is_textarea = 0)
    {
        $this->view->assign('langs', $this->pg_language->languages);
        $this->view->assign('is_textarea', $is_textarea);
        $this->view->render('helper_lang_inline_editor');

        return;
    }

    /**
     * Manage date formats
     *
     * @param integer $format_id format identifier
     *
     * @return void
     */
    public function date_formats($format_id)
    {
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');

        if ($this->input->post('btn_save')) {
            $tpl = $this->input->post('tpl', true);
            // Fill $data with used patterns
            foreach ($this->pg_date->available_formats[$format_id] as $f_id => $available) {
                $value = $this->input->post($f_id, true);
                if (in_array($value, $available)) {
                    $data[$f_id] = $value;
                }
            }
            $generic_tpl = $this->pg_date->create_generic_tpl($tpl, $data);
            $this->pg_date->save_format($generic_tpl, $format_id);
        } else {
            $generic_tpl = $this->pg_module->get_module_config('start', 'date_format_' . $format_id);
        }

        $format = array(
            'current' => $this->pg_date->parse_generic_template($generic_tpl, 'js'),
            'available' => $this->pg_date->available_formats[$format_id],
            'name' => l('date_formats_' . $format_id . '_field', 'start'),
            'gid' => $format_id,
        );
        $this->view->setHeader(l('header_settings_numerics_list', 'start'));
        $this->view->assign('section', 'formats');
        $this->view->assign('format', $format);
        $this->view->assign('settings_name', l('date_formats_settings_module', 'start'));
        $this->view->assign('other_settings', $this->other_settings);
        $this->view->render('date_formats');
    }

    /**
     * Load example of date format usage by ajax
     *
     * @return void
     */
    public function ajax_get_example()
    {
        $uf_tpl = $this->input->get('tpl', true);
        $format_id = $this->input->get('format_id', true);
        if (empty($uf_tpl) || empty($format_id)) {
            return false;
        }

        // Fill $data with used patterns
        foreach ($this->pg_date->available_formats[$format_id] as $f_id => $available) {
            $value = $this->input->get($f_id, true);
            if (in_array($value, $available)) {
                $data[$f_id] = $value;
            }
        }
        $generic_tpl = $this->pg_date->create_generic_tpl($uf_tpl, $data);

        $this->view->output($this->pg_date->strftime($generic_tpl, null, 'generic'));

        $this->view->render();
    }

    /**
     * Display site contacts
     *
     * @return void
     */
    public function contacts()
    {
        $languages = $this->pg_language->languages;

        foreach ($languages as $lid => $lang_data) {
            $data['site_contacts'][$lid] = $this->pg_language->pages->get_string('start', 'site_contacts', $lid);
        }

        if ($this->input->post("btn_save")) {
            foreach ($languages as $lid => $lang_data) {
                $post_data['site_contacts'][$lid] = $this->input->post('site_contacts_' . $lid);
            }

            $this->load->model('Start_model');
            $validate_data = $this->Start_model->validate_system_settings($post_data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                if (!empty($validate_data['data']['site_contacts'])) {
                    $this->pg_language->pages->set_string_langs('start', 'site_contacts',
                                                                $validate_data['data']['site_contacts'],
                                                                array_keys($languages));
                }
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "start"));
            }

            $data = array_merge($data, $post_data);
        }

        $this->load->plugin('fckeditor');
        foreach ($languages as $lid => $lang_data) {
            $page_data["site_contacts_fck"][$lid] = create_editor("site_contacts_" . $lid,
                                                                  isset($data['site_contacts'][$lid]) ? $data['site_contacts'][$lid] : '',
                                                                        700, 400, 'Middle');
        }

        $this->view->assign('languages', $languages);
        $this->view->assign("current_lang_id", $this->pg_language->current_lang_id);
        $this->view->assign('page_data', $page_data);
        $this->view->setHeader(l("admin_site_contacts", "start"));

        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');

        $this->view->render('contacts');
    }

    /**
     * Default user on site
     *
     * @return void
     */
    public function user()
    {
        /*if ($this->pg_module->is_module_active("users")) {
            return $this->view->setRedirect(site_url() . "admin");
        }*/
        $this->load->model('Start_model');
        if ($this->input->post("btn_save")) {
            $post_data = array();
            foreach ($this->Start_model->fields_user as $field) {
                $post_data[$field] = $this->input->post($field, true);
            }
            $validate_data = $this->Start_model->validateUserDefault($post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $user = $post_data;
            } else {
                if ($this->input->post("user_logo_delete")) {
                    $this->Start_model->deleteUserLogo();
                }
                $this->Start_model->setUserDefault($validate_data["data"]);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "start"));
            }
        }
        $user = $this->Start_model->getUserDefaultOnSite();

        $user_content = $this->Start_model->getEditUserInfo($user);
        $this->view->assign("user_content", $user_content);

        $user["action"] = site_url() . "admin/start/user";
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');
        $this->view->setHeader(l("admin_site_contacts_edit", "start"));
        $this->view->assign('user', $user);
        $this->view->render('user_default');
    }

    public function geolocation()
    {
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');

        if ($this->input->post('btn_save')) {
            $geolocation_onoff = intval($this->input->post('geolocation_onoff', true));
            $this->pg_module->set_module_config('start', 'geolocation_onoff', $geolocation_onoff);
        } else {
            $geolocation_onoff = $this->pg_module->get_module_config('start', 'geolocation_onoff');
        }
        $this->view->assign('geolocation_onoff', $geolocation_onoff);

        $this->view->setHeader(l("header_settings_geolocation", 'start'));
        $this->view->render('geolocation_settings');

        return;
    }

}
