<?php

namespace Pg\Modules\Start\Controllers;

/**
 * Start module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Start user side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Start
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Start extends \Controller
{

    /**
     * Constructor
     *
     * @return Start
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
    }

    /**
     * Display main page
     *
     * @return void
     */
    public function index()
    {
        $this->checkMobile();
        /* if($this->session->userdata("auth_type") == "user" && $this->session->userdata('user_id')){
          $this->view->setRedirect(site_url()."start/homepage");
          } */
        $this->view->render('index');
    }

    /**
     * Display home page for authorized user
     *
     * @return void
     */
    public function homepage()
    {
        $this->checkMobile();
        $user_type = $this->session->userdata('user_type');
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_overview_item');
        $this->view->assign('user_type', $user_type);
        $this->view->render('homepage');
    }

    /**
     * Display 404 error
     *
     * @return void
     */
    public function error()
    {
        header("HTTP/1.0 404 Not Found");
        $this->Menu_model->breadcrumbs_set_active(l('header_error', 'start'));
        $this->view->render('error');
    }

    /**
     * Display product version
     *
     * @return void
     */
    public function printVersion()
    {
        $this->view->output($this->pg_module->get_module_config('start', 'product_version'));
        $this->view->render();
    }

    /**
     * Test file upload
     *
     * @return void
     */
    public function testFileUpload()
    {
        $this->load->model("file_uploads/models/File_uploads_config_model");

        $this->view->assign('configs', $this->File_uploads_config_model->get_config_list());

        if ($this->input->post('btn_save') && $this->input->post('config')) {
            $file_name = 'file';

            if (isset($_FILES[$file_name]) && is_array($_FILES[$file_name]) && is_uploaded_file($_FILES[$file_name]["tmp_name"])) {
                $this->load->model("File_uploads_model");
                $return = $this->File_uploads_model->upload($this->input->post('config'), '', $file_name);

                if (!empty($return["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $return["errors"]);
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, $return["file"]);
                }
            }
        }

        $this->view->render('test_file_upload');
    }

    /**
     * Set demo user type
     *
     * @param string $type user type
     *
     * @return void
     */
    public function demo($type = null)
    {
        $this->session->set_userdata('demo_user_type', $type);
        $this->view->setRedirect();
    }

    /**
     * Get translating form
     *
     * @param integer $is_textarea true if field is textarea
     *
     * @return void
     */
    public function langInlineEditor($is_textarea = 0)
    {
        $this->view->assign('langs', $this->pg_language->languages);
        $this->view->assign('is_textarea', $is_textarea);
        $this->view->render('helper_lang_inline_editor');
        return;
    }

    /**
     * Check device is mobile
     *
     * @return void
     */
    protected function checkMobile()
    {
        if (!MOBILE_SERVER) {
            return;
        }

        $this->load->helper('cookie');
        if (get_cookie('mobile', true)) {
            return;
        }

        $this->load->library('mobile_detect');
        if (!$this->mobile_detect->isMobile()) {
            return;
        }

        set_cookie(array(
            'name' => 'mobile',
            'value' => '1',
            'expire' => 0,
            'domain' => COOKIE_SITE_SERVER,
            'path' => '/' . SITE_SUBFOLDER,
        ));

        if (preg_match('#http(s)://#i', MOBILE_SERVER)) {
            $this->view->setRedirect(MOBILE_SERVER);
        } else {
            $this->view->setRedirect(SITE_SERVER . MOBILE_SERVER);
        }
    }

    /**
     * Set demo theme and preset
     *
     * @param integer $theme_id  theme identifier
     * @param integer $preset_id dynamic block preset idetinfer
     *
     * @return void
     */
    public function theme($theme_id, $preset_id)
    {
        if ($theme_id) {
            // Deactive current user theme
            $attrs["active"] = 0;
            $this->db->where('theme_type', 'user');
            $this->db->update(THEMES_TABLE, $attrs);

            // Active new user theme
            $attrs["active"] = 1;
            $this->db->where('id', $theme_id);
            $this->db->update(THEMES_TABLE, $attrs);
        }

        // Apply preset
        $this->load->model('Dynamic_blocks_model');
        $area = $this->Dynamic_blocks_model->get_area_by_gid('index-page');
        if (!empty($area) && $preset_id) {
            $this->Dynamic_blocks_model->set_area_preset($area['id'], $preset_id);
        }

        $this->view->setRedirect();
    }

    public function startUser()
    {
        $this->load->model('Start_model');
        $user = $this->Start_model->getUserDefaultOnSite();

        $this->load->helper('seo');
        $seo_data = array(
            'id' => $user['id'],
            'user_type' => $user['user_type'],
            'name' => $user['output_name'],
            'section' => 'contacts',
        );

        if ($this->pg_module->is_module_active('seo')) {
            $lang_canonical = $this->pg_module->get_module_config('seo', 'lang_canonical');
        } else {
            $lang_canonical = true;
        }

        $seo_settings = $user;
        $seo_settings['canonical'] = rewrite_link('start', 'startUser', $seo_data, false, null, $lang_canonical);
        $seo_settings['image'] = $user['media']['user_logo']['thumbs']['big'];
        $seo_settings['section'] = 'contacts';
        $seo_settings['name'] = '<span itemprop="name">' . $user['output_name'] . '</span>';

        // pagination
        $page_data = array();
        $page_data["date_format"] = $this->pg_date->get_format('date_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $user_content = $this->Start_model->getViewUserInfo($user);
        $this->view->assign("user_content", $user_content);
        $this->pg_seo->set_seo_data($seo_settings);
        $this->Menu_model->breadcrumbs_set_parent("main-menu-pro-item");
        $this->Menu_model->breadcrumbs_set_active($user["output_name"], site_url() . "start/startUser/");
        $this->view->assign('user_contact_message', l('text_user_message', 'users'));
        $this->view->assign('user', $user);
        $this->view->render("user_view", "user", "start");
    }

    /**
     * Change user language
     *
     * @param integer $lang_id language identifier
     * @return void
     */
    public function changeLanguage($lang_id)
    {
        $lang_id = intval($lang_id);

        $old_code = $this->pg_language->languages[$this->pg_language->current_lang_id]["code"];
        $this->pg_language->current_lang_id = $lang_id;
        $code = $this->pg_language->languages[$lang_id]["code"];
        $_SERVER["HTTP_REFERER"] = str_replace("/" . $old_code . "/", "/" . $code . "/", $_SERVER["HTTP_REFERER"]);

        $this->session->set_userdata("lang_id", $lang_id);

        if (strpos($_SERVER["HTTP_REFERER"], site_url()) !== false) {
            $this->view->setRedirect($_SERVER["HTTP_REFERER"]);
        } else {
            $this->view->setRedirect();
        }
    }

    /**
     * Change user language by ajax
     *
     * @param integer $lang_id language identifier
     * @return void
     */
    public function ajaxChangeLanguage($lang_id)
    {
        $lang_id = intval($lang_id);

        $old_code = $this->pg_language->languages[$this->pg_language->current_lang_id]["code"];
        $this->pg_language->current_lang_id = $lang_id;
        $code = $this->pg_language->languages[$lang_id]["code"];
        $_SERVER["HTTP_REFERER"] = str_replace("/" . $old_code . "/", "/" . $code . "/", $_SERVER["HTTP_REFERER"]);

        $this->session->set_userdata("lang_id", $lang_id);

        $this->view->assign('code_old', $old_code);
        $this->view->assign('code_new', $code);
    }

    public function aclCheck()
    {
        $module = filter_input(INPUT_POST, 'module');
        $action = filter_input(INPUT_POST, 'action');
        $errors = [];
        if (empty($module)) {
            $errors[] = 'Empty module';
        }
        if (empty($action)) {
            $errors[] = 'Empty action';
        }
        if (empty($errors)) {
            $allowed = $this->acl->check(new \Pg\Libraries\Acl\Action\ViewPage(
                new \Pg\Libraries\Acl\Resource\Page(
                    ['module' => $module, 'controller' => $module, 'action' => $action,]
                )
            ), false);
        } else {
            $allowed = false;
        }
        $this->view->assign(View::MSG_ERROR, $errors);
        $this->view->assign('is_allowed', $allowed);
        $this->view->render();
    }


    public function ajaxValueToCurrencyFormat()
    {
        $params = $this->input->post('data', true);
        $return = array();
        if (!empty($params)) {
            $this->load->helper('start');
            $this->load->model('Currencies_model');   
            $currency = $this->Currencies_model->get_currency_default(true);
            $params['cur_gid'] = $currency['gid'];
            $return = currency_format_output($params);
        }
        $this->view->assign($return);
        $this->view->render();
    }

}
