<?php

namespace Pg\Modules\Users\Controllers;

/**
 * Users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Users extends \Controller
{

    /**
     * Actvity filters
     *
     * @var array
     */
    protected $filter_arr = array('all', 'not_active', 'active', 'not_confirm');

    /**
     * Constructor
     *
     * @return Admin_Users
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Users_model");
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "users_menu_item");
    }

    /**
     * Users management
     *
     * @param string  $user_type       user type
     * @param string  $filter          activity filter
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function index($user_type = null, $filter = "all", $order = "email", $order_direction = "ASC", $page = 1)
    {
        $current_settings = isset($_SESSION["users_list"]) ? $_SESSION["users_list"] : array();
        if (!isset($current_settings["user_type"])) {
            $current_settings["user_type"] = $user_type;
        }
        if (!isset($current_settings["filter"])) {
            $current_settings["filter"] = $filter;
        }
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = $order;
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = $order_direction;
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = $page;
        }

        $user_types = $this->Users_model->getUserTypes();

        if (!in_array($filter, $this->filter_arr)) {
            $filter = $this->filter_arr[0];
        }

        if (!$user_type) {
            $user_type = $current_settings["user_type"];
        }

        if ($this->input->post('filter-submit')) {
            $_SESSION['users_filters'] = array();

            $post_data['keyword'] = trim(strip_tags($this->input->post('keyword', true)));
            if ($post_data['keyword']) {
                $_SESSION['users_filters']['keyword'] = $post_data['keyword'];
                $_SESSION['users_filters']['keyword_mode'] = true;
            }
        }

        if ($this->input->post('filter-reset')) {
            $_SESSION['users_filters'] = array();
        }

        $users_filters = isset($_SESSION['users_filters']) ? $_SESSION['users_filters'] : array();

        if (!in_array($user_type, $user_types)) {
            $user_type = current($user_types);
        }
        $this->view->assign("user_type", $user_type);
        $current_settings["user_type"] = $user_type;
        foreach ($this->filter_arr as $value) {
            $filters = $users_filters;
            $filters["type"] = $user_type;
            if ($value != "all") {
                $filters[$value] = "1";
            }
            $filter_data[$value] = $this->Users_model->get_users_count_by_filters($filters);
        }

        $current_settings["filter"] = $filter;

        $this->view->assign("filter", $filter);
        $this->view->assign("filters", $this->filter_arr);
        $this->view->assign("filter_data", $filter_data);
        $current_settings["page"] = $page;

        if (!$order) {
            $order = $current_settings["order"];
        }
        $this->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $this->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        $users_count = $filter_data[$filter];

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "admin_items_per_page");
        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $users_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["users_list"] = $current_settings;

        $build_link = function($field) use ($user_type, $filter, $order, $order_direction) {
            return site_url() . "admin/users/index/$user_type/$filter/$field/" . (($order != $field xor $order_direction === 'DESC') ? 'ASC' : 'DESC');
        };
        $this->view->assign('sort_links', array(
            'name' => $build_link('name'),
            'email' => $build_link('email'),
            'account' => $build_link('account'),
            'listings' => $build_link('listings'),
            'reviews' => $build_link('reviews'),
            'date_created' => $build_link('date_created'),
        ));

        switch ($order) {
            case "name":
                switch ($user_type) {
                    case "company": $order = "company_name";
                        break;
                    default: $order = "CONCAT(fname, ' ', sname)";
                        break;
                }
                break;
            case "listings": $order = "listings_for_sale_count+listings_for_buy_count+listings_for_rent_count+listings_for_lease_count";
                break;
            case "reviews": $order = "review_count";
                break;
        }

        if ($users_count > 0) {
            switch ($order) {
                case 'date_created':
                    $order_array = array('id' => $order_direction);
                    break;
                default:
                    $order_array = array($order => $order_direction);
                    break;
            }

            $filters = $users_filters;
            $filters["type"] = $user_type;
            if ($filter != "all") {
                $filters[$filter] = "1";
            }
            $this->Users_model->set_format_settings('get_safe', false);
            $users = $this->Users_model->get_users_list_by_filters($filters, $page, $items_on_page, $order_array);
            $this->Users_model->set_format_settings('get_safe', true);
            $this->view->assign("users", $users);
        }

        $this->load->helper('navigation');

        $url = site_url() . "admin/users/index/" . $user_type . "/" . $filter . "/" . $order . "/" . $order_direction . "/";
        $page_data = get_admin_pages_data($url, $users_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $page_data['filter'] = $filters;
        $this->view->assign("page_data", $page_data);

        $this->load->model("users/models/Users_groups_model");
        $groups = $this->Users_groups_model->get_groups_list();
        $this->view->assign("groups", $groups);

        $base_currency_gid = $this->pg_module->get_module_config('start', 'base_currency');
        $this->view->assign('base_currency_gid', $base_currency_gid);

        $this->Menu_model->set_menu_active_item("admin_users_types_menu", "users_types_" . $user_type . "_item");
        $this->view->setHeader(l("admin_header_users_list", "users"));
        $this->view->render("list");
    }

    /**
     * Edit user data
     *
     * @param integer $user_id     user identifier
     * @param string  $section_gid section guid
     * @param string  $user_type   user type
     *
     * @return void
     */
    public function edit($user_id, $section_gid = 'contacts', $user_type = null)
    {
        $format_settings = array('get_safe' => false, 'get_company' => true);

        switch ($section_gid) {
            case 'contacts':
                $format_settings['get_field_editor'] = true;
                break;
            case 'seo':
                break;
        }

        $this->Users_model->set_format_settings($format_settings);

        if ($user_id) {
            $this->Users_model->set_format_settings(array('get_safe' => false, 'get_company' => true));
            $data = $this->Users_model->get_user_by_id($user_id);
            $this->Users_model->set_format_settings(array('get_safe' => true, 'get_company' => false));
        } else {
            $data["lang_id"] = $this->pg_language->current_lang_id;
            $data["user_type"] = $user_type;
            $data = $this->Users_model->format_user($data);
        }

        $this->Users_model->set_format_settings(array('get_safe' => true, 'get_company' => false, 'get_field_editor' => false));

        if ($this->input->post("btn_save")) {
            switch ($section_gid) {
                case 'contacts':
                    $post_data = $this->input->post("data", true);
                    $post_data["user_type"] = $data['user_type'];
                    $post_data["id_country"] = $this->input->post("id_country", true);
                    $post_data["id_region"] = $this->input->get_post("id_region", true);
                    $post_data["id_city"] = $this->input->get_post("id_city", true);
                    $post_data["id_district"] = $this->input->get_post("id_district", true);
                    $post_data["services"] = $this->input->post("services", true);

                    $agent_company = $post_data["agent_company"];
                    unset($post_data["agent_company"]);

                    if ($user_id) {
                        $post_data["id"] = $user_id;
                        if (!intval($this->input->post("update_password"))) {
                            unset($post_data["password"]);
                            unset($post_data["repassword"]);
                        }

                        if ($this->input->post("user_icon_delete")) {
                            $this->Users_model->delete_logo($user_id);
                        }
                    }

                    if ($post_data['services']['featured'] != $data['is_featured']) {
                        if ($post_data['services']['featured']) {
                            $post_data["featured_end_date"] = date("Y-m-d H:i:s",
                                                                   time() + $this->Users_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data["featured_end_date"] = "0000-00-00 00:00:00";
                        }
                    }

                    if ($post_data['services']['show_logo'] != $data['is_show_logo']) {
                        if ($post_data['services']['show_logo']) {
                            $post_data["show_logo_end_date"] = date("Y-m-d H:i:s",
                                                                    time() + $this->Users_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data["show_logo_end_date"] = "0000-00-00 00:00:00";
                        }
                    }

                    foreach ($this->Users_model->get_fields_for_select('user') as $field) {
                        $post_data[$field] = $this->input->post($field, true);
                    }

                    foreach ($this->Users_model->get_fields_for_select($data['user_type']) as $field) {
                        $post_data[$field] = $this->input->post($field, true);
                    }

                    $validate_data = $this->Users_model->validate_user($user_id, $post_data, 'user_icon');
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } elseif (!empty($validate_data['data'])) {
                        $save_data = $validate_data["data"];
                        if (!$user_id) {
                            $save_data['user_type'] = $data['user_type'];
                            $save_data['lang_id'] = $this->pg_language->current_lang_id;
                        }
                        $user_id = $this->Users_model->save_user($user_id, $save_data, 'user_icon', false);

                        if ($agent_company) {
                            $this->Users_model->save_agent_company($user_id, $agent_company, false, false);
                            $this->Users_model->save_agent_company_status($user_id, $agent_company);

                            $this->load->model("Linker_model");
                            $this->Linker_model->add_link("users_contacts", $user_id, $agent_company);
                            $this->Linker_model->add_link("users_contacts", $agent_company, $user_id);
                        }

                        $this->load->model('geomap/models/Geomap_model');
                        $this->load->model('geomap/models/Geomap_settings_model');

                        $map_gid = $this->Geomap_model->get_default_driver_gid();

                        $post_settings = $this->input->post('map', true);
                        $post_settings['lat'] = $validate_data['data']['lat'];
                        $post_settings['lon'] = $validate_data['data']['lon'];

                        $validate_data = $this->Geomap_settings_model->validate_settings($post_settings);
                        if (empty($validate_data["errors"])) {
                            $this->Geomap_settings_model->save_settings($map_gid, 0, $user_id, 'user_profile',
                                                                        $validate_data["data"]);
                        }

                        if (!$data['id']) {
                            $this->load->model("Notifications_model");
                            $user_data = $save_data;
                            $user_data['password'] = $post_data['password'];
                            $this->Notifications_model->send_notification($user_data['email'],
                                                                          "user_account_create_by_admin", $user_data,
                                                                          '', $user_data['lang_id']);
                        }

                        if ($user_id) {
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_user", "users"));
                        } else {
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_add_user", "users"));
                        }

                        $this->Users_model->set_format_settings(array('get_safe' => false, 'get_company' => true));
                        $post_data = $this->Users_model->get_user_by_id($user_id);
                        $this->Users_model->set_format_settings(array('get_safe' => true, 'get_company' => false));

                        $this->view->setRedirect(site_url() . "admin/users/edit/" . $user_id . "/" . $section_gid . "/" . $user_type);
                    }

                    $data = array_merge($data, $post_data);
                    break;
                case 'seo':
                    $this->load->model('Seo_advanced_model');
                    $seo_fields = $this->Seo_advanced_model->get_seo_fields();
                    foreach ($seo_fields as $key => $section_data) {
                        if ($this->input->post('btn_save_' . $section_data['gid'])) {
                            $post_data[$section_data['gid']] = $this->input->post($section_data['gid'], true);
                            $validate_data = $this->Seo_advanced_model->validate_seo_tags($user_id, $post_data);
                            if (!empty($validate_data["errors"])) {
                                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                            } else {
                                $user_data['id_seo_settings'] = $this->Seo_advanced_model->save_seo_tags($data['id_seo_settings'],
                                                                                                         $validate_data['data']);
                                if (!$data['id_seo_settings']) {
                                    $data['id_seo_settings'] = $user_data['id_seo_settings'];
                                    $this->Users_model->save_user($user_id, $user_data, false);
                                }
                                $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                                   l('success_settings_updated', 'seo'));

                                $this->view->setRedirect(site_url() . "admin/users/edit/" . $user_id . "/" . $section_gid . "/" . $user_type);
                            }

                            $data = array_merge($data, $post_data);
                            break;
                        }
                    }
                    break;
            }
        }

        switch ($section_gid) {
            case 'contacts':
                $this->view->assign("user_type", $data['user_type']);

                $weekdays = $this->pg_language->ds->get_reference("start", "week_day_full");
                $this->view->assign("weekdays", $weekdays);

                $dayhours = $this->pg_language->ds->get_reference("start", "dayhour-names");
                $this->view->assign("dayhours", $dayhours);

                if ($data["user_type"] == 'company' && !empty($data['id'])) {
                    $markers = array(
                        array(
                            "gid" => $data["id"],
                            "country" => $data["country"],
                            "region" => $data["region"],
                            "city" => $data["city"],
                            "address" => $data["address"],
                            "postal_code" => $data["postal_code"],
                            "lat" => (float) $data["lat"],
                            "lon" => (float) $data["lon"],
                            "info" => ($data['location'] ? $data['location'] : $data["output_name"] . ', ' . l($data['user_type'],
                                                                                                               'users')),
                            "dragging" => true,
                        ),
                    );
                    $this->view->assign("markers", $markers);

                    $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
                    $view_settings = array(
                        "class" => "",
                        "disable_smart_zoom" => 1,
                        "zoom_listener" => "get_user_zoom_data",
                        "type_listener" => "get_user_type_data",
                        "drag_listener" => "get_user_drag_data",
                        "lang" => $current_language["code"],
                    );
                    $this->view->assign("map_settings", $view_settings);

                    if ($user_id) {
                        $this->load->model('geomap/models/Geomap_model');
                        $this->load->model('geomap/models/Geomap_settings_model');
                        $map_gid = $this->Geomap_model->get_default_driver_gid();
                        $map_settings = $this->Geomap_settings_model->get_settings($map_gid, 0, $user_id, 'user_profile');
                        $map_settings["lat"] = (float) $data["lat"];
                        $map_settings["lon"] = (float) $data["lon"];
                        $this->view->assign('user_map_settings', $map_settings);
                    }
                }

                $this->load->model('Field_editor_model');
                $this->Field_editor_model->initialize('user');
                $user_type_sections = $this->Field_editor_model->get_section_list();

                if ($data['user_type']) {
                    $this->Field_editor_model->initialize($data['user_type']);
                    $fe_sections = $this->Field_editor_model->get_section_list();
                    $user_type_sections = array_merge($user_type_sections, $fe_sections);
                }

                if (!empty($user_type_sections)) {
                    $sections = array();
                    foreach ($user_type_sections as $section) {
                        $sections[] = $section['gid'];
                    }

                    $this->view->assign('sections_data', array_values($user_type_sections));

                    $params['where_in']['section_gid'] = $sections;
                    $fields_data = $this->Field_editor_model->get_form_fields_list($data, $params);
                    $this->view->assign('fields_data', $fields_data);
                }

                $use_phone_format = $this->pg_module->get_module_config("start", "use_phone_format");
                if ($use_phone_format) {
                    $phone_format = $this->pg_module->get_module_config("start", "phone_format");
                    $this->view->assign('phone_format', $phone_format);
                }

                // check pay services exists
                if ($this->pg_module->is_module_active("users_services")) {
                    $this->load->model('Users_services_model');
                    $users_services = $this->Users_services_model->get_active_services($data["user_type"]);
                    foreach ($users_services as $template) {
                        $this->view->assign("use_" . str_replace(array($data["user_type"] . '_', '_template'), '',
                                                                 $template["template_gid"]), $template["status"]);
                    }
                    $this->view->assign("use_services", !empty($users_services));
                } else {
                    $this->view->assign("use_services", false);
                }
                break;
            case 'seo':
                $this->load->model('Seo_advanced_model');
                $seo_fields = $this->Seo_advanced_model->get_seo_fields();
                $this->view->assign('seo_fields', $seo_fields);

                $languages = $this->pg_language->languages;
                $this->view->assign('languages', $languages);

                $current_lang_id = $this->pg_language->current_lang_id;
                $this->view->assign('current_lang_id', $current_lang_id);

                if ($data['id_seo_settings']) {
                    $seo_settings = $this->Seo_advanced_model->get_seo_tags($data['id_seo_settings']);
                    $this->view->assign('seo_settings', $seo_settings);
                }
                break;
        }

        $this->view->assign("langs", $this->pg_language->languages);
        $this->view->assign("data", $data);
        $this->view->assign("section_gid", $section_gid);

        $cur_set = $_SESSION["users_list"];
        $back_url = site_url() . "admin/users/index/" . $cur_set["user_type"] . "/" . $cur_set["filter"] . "/" . $cur_set["order"] . "/" . $cur_set["order_direction"] . "/" . $cur_set["page"] . "";
        $this->view->assign("back_url", $back_url);
        $this->view->setHeader(l("admin_header_users_list", "users"));
        $this->view->render("edit_form");
    }

    /**
     * Create new user
     *
     * @param string $user_type user type
     *
     * @return void
     */
    public function add($user_type = null)
    {
        $user_types = $this->Users_model->getUserTypes();
        if (!$user_type) {
            $user_type = current($user_types);
        }
        if(!in_array($user_type, $user_types))
        {
            show_404('admin/users/add');
        }
        $this->edit(null, 'contacts', $user_type);
    }

    /**
     * Remove user
     *
     * @param array/integer $users_ids users identifiers
     *
     * @return void
     */
    public function delete($users_ids = null)
    {
        if (!$users_ids) {
            $users_ids = $this->input->post('ids', true);
        }
        if (!empty($users_ids)) {
            foreach ((array) $users_ids as $user_id) {
                $this->Users_model->delete_user($user_id);
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_delete_user", "users"));
        }
        $cur_set = $_SESSION["users_list"];
        $url = site_url() . "admin/users/index/" . $cur_set["user_type"] . "/" . $cur_set["filter"] . "/" . $cur_set["order"] . "/" . $cur_set["order_direction"] . "/" . $cur_set["page"] . "";
        $this->view->setRedirect($url);
    }

    /**
     * Activate/deactivate user
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param array/integer $users_ids users identfiers
     * @param integer       $status    active status
     *
     * @return void
     */
    public function activate($users_ids, $status = 0)
    {
        if (!$users_ids) {
            $users_ids = $this->input->post('ids', true);
        }
        if (!empty($users_ids)) {
            foreach ((array) $users_ids as $user_id) {
                $this->Users_model->activate_user($user_id, $status);
            }
            if ($status) {
                $success_msg = l("success_activate_user", "users");
            } else {
                $success_msg = l("success_deactivate_user", "users");
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, $success_msg);
        }
        $cur_set = $_SESSION["users_list"];
        $url = site_url() . "admin/users/index/" . $cur_set["user_type"] . "/" . $cur_set["filter"] . "/" . $cur_set["order"] . "/" . $cur_set["order_direction"] . "/" . $cur_set["page"] . "";
        $this->view->setRedirect($url);
    }

    /**
     * Change group of user by ajax
     *
     * @param integer $user_id   user identifier
     * @param string  $group_gid group guid
     *
     * @return void
     */
    public function ajax_change_user_group($user_id, $group_gid)
    {
        $this->load->model("users/models/Users_groups_model");
        $group_gid = strval($group_gid);
        $group_data = $this->Users_groups_model->get_group_by_gid($group_gid);
        $group_id = (!empty($group_data)) ? $group_data["id"] : 0;

        $user_id = intval($user_id);
        if ($user_id && $group_id) {
            $save_data["group_id"] = $group_id;
            $user_id = $this->Users_model->save_user($user_id, $save_data);
        }
    }

    /**
     * Change group of users by ajax
     *
     * @param string $group_gid group guid
     *
     * @return void
     */
    public function ajax_change_users_group($group_gid)
    {
        $this->load->model("users/models/Users_groups_model");
        $group_gid = strval($group_gid);
        $group_data = $this->Users_groups_model->get_group_by_gid($group_gid);
        $group_id = (!empty($group_data)) ? $group_data["id"] : 0;

        $user_ids = $this->input->post("user_ids");
        if (!empty($user_ids) && $group_id) {
            $save_data["group_id"] = $group_id;
            foreach ($user_ids as $user_id) {
                $this->Users_model->save_user($user_id, $save_data);
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("error_user_successfully_change_group", "users"));
        }
    }

    /**
     * Groups management
     *
     * @param integer $page page of results
     *
     * @return void
     */
    public function groups($page = 1)
    {
        $this->load->model("users/models/Users_groups_model");

        $current_settings["page"] = $page;
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        $group_count = $this->Users_groups_model->get_groups_count();

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "admin_items_per_page");
        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $group_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["groups_list"] = $current_settings;

        if ($group_count > 0) {
            $groups = $this->Users_groups_model->get_groups_list($page, $items_on_page, array("date_created" => "DESC"));
            $this->view->assign("groups", $groups);
        }

        $this->load->helper('navigation');

        $url = site_url() . "admin/users/groups/";
        $page_data = get_admin_pages_data($url, $group_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $this->Menu_model->set_menu_active_item("admin_users_menu", "groups_list_item");
        $this->view->setHeader(l("admin_header_groups_list", "users"));
        $this->view->render("groups_list");
    }

    /**
     * Edit group
     *
     * @param string $group_id group identifier
     *
     * @return void
     */
    public function group_edit($group_id = null)
    {
        $this->load->model("users/models/Users_groups_model");
        if ($group_id) {
            $data = $this->Users_groups_model->get_group_by_id($group_id);
        } else {
            $data = array();
        }

        if ($this->input->post("btn_save")) {
            $post_data = array(
                "gid" => $this->input->post("gid", true),
            );

            $langs_data = $this->input->post("langs", true);
            $validate_data = $this->Users_groups_model->validate_group($group_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = array_merge($data, $validate_data["data"]);
            } else {
                $data = $validate_data["data"];
                $group_id = $this->Users_groups_model->save_group($group_id, $data, $langs_data);

                if ($group_id) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_group", "users"));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_add_group", "users"));
                }

                $url = site_url() . "admin/users/groups";
                $this->view->setRedirect($url);
            }
        }

        $data = $this->Users_groups_model->format_group($data);
        $data["langs"] = $this->Users_groups_model->_get_group_string_data($group_id);

        $this->view->assign("languages", $this->pg_language->languages);
        $this->view->assign("data", $data);

        $this->view->setHeader(l("admin_header_groups_edit", "users"));
        $this->view->render("group_edit_form");
    }

    /**
     * Groups management
     *
     * @param integer $group_id
     *
     * @return void
     */
    public function group_set_default($group_id)
    {
        if (!empty($group_id)) {
            $this->load->model("users/models/Users_groups_model");
            $this->Users_groups_model->set_default($group_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_defaulted_group", "users"));
        }
        $this->view->setRedirect(site_url() . "admin/users/groups/" . $_SESSION["groups_list"]["page"]);
    }

    /**
     * Remove group
     *
     * @param integer $group_id group identifier
     *
     * @return void
     */
    public function group_delete($group_id)
    {
        if (!empty($group_id)) {
            $this->load->model("users/models/Users_groups_model");
            $group_data = $this->Users_groups_model->get_group_by_id($group_id);
            if ($group_data["is_default"]) {
                $this->system_messages->addMessage(View::MSG_ERROR, l("error_cant_delete_default_group", "users"));
            } else {
                $this->Users_groups_model->delete_group($group_id);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_delete_group", "users"));
            }
        }
        $current_settings = $_SESSION["groups_list"];
        $url = site_url() . "admin/users/groups/" . $current_settings["page"] . "";
        $this->view->setRedirect($url);
    }

    /**
     * Approve/decline request from agent
     *
     * @param integer $company_id company identifier
     * @param boolean $approve    approve request
     * @param integer $agent_id   agent identifier
     *
     * @return void
     */
    public function agents_request($company_id, $approve = 1, $agent_id = null)
    {
        $has_error = false;
        $messages = array();
        if (!$agent_id) {
            $agent_id = $this->input->post("ids");
        }
        if (!empty($agent_id)) {
            $action = $approve ? "approve" : "decline";
            foreach ((array) $agent_id as $i) {
                $error = $this->Users_model->{$approve . "_agent"}($company_id, $i);
                if ($error) {
                    $has_error = true;
                    $messages[] = $error;
                } else {
                    $messages[] = l("success_" . $action . "_agent", "users");
                }
            }
            if ($has_error) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }
        $current_settings = $_SESSION["users_agents_list"];
        $url = site_url() . "admin/users/agents/" . $company_id;
        if ($current_settings["order"]) {
            $url .= "/" . $current_settings["order"];
            if ($current_settings["order_direction"]) {
                $url .= "/" . $current_settings["order_direction"];
                if ($current_settings["page"]) {
                    $url .= "/" . $current_settings["page"];
                }
            }
        }
        $this->view->setRedirect($url);
    }

    /**
     * Remove agent from company
     *
     * @param integer $company_id company identifier
     * @param integer $agent_id   agent identifier
     *
     * @return void
     */
    public function agents_delete($company_id, $agent_id = null)
    {
        $errors = false;
        $messages = array();
        if (!$agent_id) {
            $agent_id = $this->input->post("ids");
        }
        if (!empty($agent_id)) {
            foreach ((array) $agent_id as $i) {
                $error = $this->Users_model->delete_agent($company_id, $i);
                if ($error) {
                    $errors = true;
                    $messages[] = $error;
                } else {
                    $messages[] = l("success_deleted_agent", "users");
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }
        $current_settings = $_SESSION["users_agents_list"];
        $url = site_url() . "admin/users/agents/" . $company_id;
        if ($current_settings["order"]) {
            $url .= "/" . $current_settings["order"];
            if ($current_settings["order_direction"]) {
                $url .= "/" . $current_settings["order_direction"];
                if ($current_settings["page"]) {
                    $url .= "/" . $current_settings["page"];
                }
            }
        }
        $this->view->setRedirect($url);
    }

    /**
     * Get out agent from company
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function agent_get_out($user_id)
    {
        $user = $this->Users_model->get_user_by_id($user_id);
        $error = $this->Users_model->delete_agent($user["agent_company"], $user_id);
        if ($error) {
            $this->system_messages->addMessage(View::MSG_ERROR, $error);
        } elseif ($user["agent_status"]) {
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_agent_get_out_by_admin", "users"));
        } else {
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_agent_cancel_request_by_admin", "users"));
        }
        $this->view->setRedirect(site_url() . "admin/users/edit/" . $user_id);
    }

    /**
     * Set map settings for user object
     *
     * @param integer $user_id user identifier
     * @param array   $param   map settings
     *
     * @return void
     */
    public function ajax_set_map($user_id, $param)
    {
        if (!$this->Users_model->get_user_by_id($user_id)) {
            return;
        }

        $this->load->model('geomap/models/Geomap_model');
        $this->load->model('geomap/models/Geomap_settings_model');

        switch ($param) {
            case 'type': $settings['view_type'] = $this->input->post('type', true);
                break;
            case 'zoom': $settings['zoom'] = $this->input->post('zoom', true);
                break;
            case 'coordinates':
                $settings['lat'] = $this->input->post('lat', true);
                $settings['lon'] = $this->input->post('lon', true);
                break;
        }

        if (!$settings) {
            return;
        }

        $validate_data = $this->Geomap_settings_model->validate_settings($settings);
        if (empty($validate_data["errors"])) {
            $map_gid = $this->Geomap_model->get_default_driver_gid();
            $this->Geomap_settings_model->save_settings($map_gid, 0, $user_id, 'user_profile', $validate_data["data"]);
        }

        if ($param === 'coordinates') {
            $data['lat'] = $this->input->post('lat', true);
            $data['lon'] = $this->input->post('lon', true);
            $this->Users_model->save_user($user_id, $data);
        }

        return;
    }

    /**
     * Set map coordinates to user after geocoding
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function ajax_set_geocode_coordinates($user_id)
    {
        $user = $this->Users_model->get_user_by_id($user_id);

        if (!$user) {
            return;
        }

        if ((float) $user['lat'] > 0 || (float) $user['lot'] > 0) {
            return;
        }

        $data['lat'] = $this->input->post('lat');
        $data['lon'] = $this->input->post('lon');

        $validate_data = $this->Users_model->validate_user($data);
        if (!empty($validate_data["errors"])) {
            $this->view->assign('error', implode('<br>', $validate_data["errors"]));

            $this->view->render();

            return;
        } else {
            $this->Users_model->save_user($user_id, $validate_data['data']);
        }

        $this->view->render();
    }

    /**
     * Render export form block by ajax
     *
     * @return void
     */
    public function ajax_get_export_extend_form()
    {
        $user_type = strval($this->input->post('user_type', true));
        $this->load->model('Users_model');
        $this->load->model('users/models/Users_export_model');

        $this->view->output($this->Users_export_model->get_export_extend_form($user_type, 'admin_export_form_gid'));

        $this->view->render();
    }

    /**
     * Display users selector by ajax
     *
     * @param integer $max_select max selected users
     * @param string  $template   form template
     *
     * @return void
     */
    public function ajax_get_users_form($max_select = 1, $template = "default")
    {
        $selected = $this->input->post("selected", true);

        if (!empty($selected)) {
            $data["selected"] = $this->Users_model->get_users_list(null, null, array("unique_name" => "asc"), array(),
                                                                   $selected, true);
        } else {
            $data["selected"] = array();
        }
        $data["max_select"] = $max_select ? $max_select : 0;

        $this->view->assign("select_data", $data);
        $this->view->render("ajax_user_select_form_" . $template);
    }

}
