<?php

namespace Pg\Modules\Users\Models;

/**
 * Users module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

if (!defined('USERS_TABLE')) {
    define("USERS_TABLE", DB_PREFIX . "users");
}
if (!defined('USER_ACCOUNT_STAT_TABLE')) {
    define("USER_ACCOUNT_STAT_TABLE", DB_PREFIX . "user_account_list");
}

/**
 * Users main model
 *
 * @package     PG_RealEstate
 * @subpackage  Users
 *
 * @category    models
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_model extends \Model
{
    const USE_GROUPS = false;

    const GROUP_PROFESSIONALS = 'professionals';

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Available user types
     *
     * @var array
     */
    private $user_types = [];

    private $user_types_groups = [
        'types' => [],
        'groups' => [
            self:: GROUP_PROFESSIONALS => ['company', 'agent',],
        ],
    ];

    /**
     * Agents types
     *
     * @var array
     */
    public $agents_arr = array('company', 'agent',);

    /**
     * Shared user properties in data source
     *
     * @var array
     */
    protected $fields_general = array(
        "id",
        "fname",
        "sname",
        "unique_name",
        "company_name",
        "email",
        "email_restore",
        "email_restore_code",
        'phone_restore',
        'phone_restore_code',
        "phone",
        "user_type",
        "password",
        "status",
        "confirm",
        "lang_id",
        "user_open_id",
        "user_logo",
        "user_logo_moderation",
        "group_id",
        "account",
        "views",
        "date_created",
        "date_modified",
        // company's agents field
        "agent_count",
        // agents field for company data storage
        "agent_company",
        "agent_date",
        "agent_status",
        // services for companies and agents
        'is_featured',
        "featured_end_date",
        'is_show_logo',
        "show_logo_end_date",
        // company's geo data
        "id_country",
        "id_region",
        "id_city",
        "id_district",
        "address",
        "postal_code",
        "contact_email",
        "contact_phone",
        "lat",
        "lon",
        "roles",
    );

    /**
     * All user properties in data source
     *
     * @var array
     */
    protected $fields_all = array(
        // general fields
        "id",
        "fname",
        "sname",
        "unique_name",
        "company_name",
        "email",
        "email_restore",
        "email_restore_code",
        'phone_restore',
        'phone_restore_code',
        "phone",
        "user_type",
        "password",
        "status",
        "confirm",
        "lang_id",
        "user_open_id",
        "user_logo",
        "user_logo_moderation",
        "group_id",
        "account",
        "confirm_code",
        "id_currency",
        "views",
        "date_created",
        "date_modified",
        // company's agents field
        "agent_count",
        // agents field for company data storage
        "agent_company",
        "agent_date",
        "agent_status",
        // services for companies and agents
        'is_featured',
        "featured_end_date",
        'is_show_logo',
        "show_logo_end_date",
        // contacts fields
        "contact_email",
        "contact_phone",
        // company's geo data
        "id_country",
        "id_region",
        "id_city",
        "id_district",
        "address",
        "postal_code",
        "lat",
        "lon",
        'id_seo_settings',
        "roles",
    );

    /**
     * Safe user properties in data source
     *
     * @var array
     */
    protected $safe_fields = array(
        'email',
        'phone',
        'password',
        'email_restore',
        'email_restore_code',
        'phone_restore',
        'phone_restore_code',
        'user_open_id',
        'confirm',
        'confirm_code',
    );

    /**
     * Contact user properties in data source
     *
     * @var array
     */
    protected $contact_fields = array(
        'contact_phone',
        'contact_email',
        'location',
        'id_country',
        'id_region',
        'region_code',
        'id_city',
        'address',
        'postal_code',
        'lat',
        'lon',
    );

    /**
     * User logo upload configuration (GUID)
     *
     * @var string
     */
    public $upload_config_id = "user-logo";

    /**
     * User logo moderation type (GUID)
     *
     * @var string
     */
    protected $moderation_type = "user_logo";

    /**
     * User data moderation type (GUID)
     *
     * @var string
     */
    protected $moderation_type_text = "users";

    /**
     * General field editor type (GUID)
     *
     * @var string
     */
    public $form_editor_type = "user";

    /**
     * Display sections of user profile
     *
     * @var array
     */
    public $display_sections = array("company" => 0, "reviews" => 1, "contacts" => 1, "map" => 1);

    /**
     * Schema of generating unique name
     *
     * @var array
     */
    protected $user_gid_scheme = array(
        'private' => array('str' => "[fname][sname]", 'fields' => array('fname', 'sname')),
        'agent' => array('str' => "[fname][sname]", 'fields' => array('fname', 'sname')),
        'company' => array('str' => "[company_name]", 'fields' => array('company_name')),
    );

    /**
     * Period of users services in days by default
     *
     * @var integer
     */
    public $service_period_default = 14;

    /**
     * Settings for formatting user
     *
     * @var array
     */
    protected $format_settings = array(
        'use_format' => true,
        'get_safe' => true,
        'get_contact' => false,
        'get_language' => false,
        'get_currency' => false,
        'get_group' => false,
        'get_company' => false,
        'get_field_editor' => false,
        'get_views' => false,
    );

    protected $old_settings = array();
    /**
     * Listings data properties
     *
     * @var array
     */
    protected $fields_listings = array(
        'listings_for_sale_count',
        'listings_for_buy_count',
        'listings_for_rent_count',
        'listings_for_lease_count',
    );

    /**
     * Reviews data properties
     *
     * @var array
     */
    protected $fields_reviews = array(
        "review_data",
        "review_count",
        "review_sorter",
        "review_value",
        "review_type",
    );

    /**
     * Properties from field editor by types
     *
     * @var array
     */
    protected $field_editor_fields = array();

    /**
     * Sections from field editor by types
     *
     * @var array
     */
    protected $field_editor_sections = array();

    /**
     * Quick search form from field editor (GUID)
     *
     * @var string
     */
    public $quick_search_form_gid = 'quick_search_form';

    /**
     * Quick search form of user from field editor (GUID)
     *
     * @var string
     */
    public $quick_search_form_user_gid = 'quick_search_form_13';

    /**
     * Quick search form of agent from field editor (GUID)
     *
     * @var string
     */
    public $quick_search_form_agent_gid = 'quick_search_form_14';

    /**
     * Quick search form of company from field editor (GUID)
     *
     * @var string
     */
    public $quick_search_form_company_gid = 'quick_search_form_15';

    /**
     * Quick search form of private from field editor (GUID)
     *
     * @var string
     */
    public $quick_search_form_private_gid = 'quick_search_form_16';

    /**
     * Class constructor
     *
     * @return Users_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        if (!self::USE_GROUPS) {
           $this->user_types_groups['groups'] = [];
        }

        if ($this->ci->pg_module->is_module_active("listings")) {
            $this->fields_general = array_merge($this->fields_general, $this->fields_listings);
            $this->fields_all = array_merge($this->fields_all, $this->fields_listings);
        }

        if ($this->ci->pg_module->is_module_active("reviews")) {
            $this->fields_general = array_merge($this->fields_general, $this->fields_reviews);
            $this->fields_all = array_merge($this->fields_all, $this->fields_reviews);
        }
    }

    /**
     * Get user types
     *
     * @return array
     */
    public function getUserTypes($group = null)
    {
        $this->ci->load->model('users/models/Users_types_model');
        // TODO: cache
        // TODO: remove one property
        $this->user_types = array_column($this->ci->Users_types_model->getTypes(), 'name');
        $this->user_types_groups['types'] = $this->user_types;
        if($group) {
            return $this->user_types_groups['groups'][$group];
        }
        return $this->user_types_groups['types'];
    }

    public function getUserTypesGroups()
    {
        return $this->user_types_groups;
    }

    /**
     * Return available user types with names
     *
     * @return array
     */
    public function getUserTypesNames()
    {
        $user_types = array();
        foreach ($this->getUserTypes() as $user_type) {
            $user_types[$user_type] = l($user_type, 'users');
        }

        return $user_types;
    }

    /**
     * Return user type by default
     *
     * @return string
     */
    public function get_user_type_default()
    {
        $user_types = $this->getUserTypes();

        if (empty($user_types)) {
            return '';
        }

        return current($user_types);
    }

    /**
     * Return user object from data source
     *
     * Method used pairs of field name/field value as arguments
     *
     * @return array/false
     */
    protected function _get_user()
    {
        $args = func_get_args();
        $this->ci->db->select(implode(", ", $this->fields_all))->from(USERS_TABLE);

        $count = floor(count($args) / 2);
        for ($i = 0; $i < $count; ++$i) {
            $this->ci->db->where($args[$i * 2], $args[$i * 2 + 1]);
        }

        $result = $this->ci->db->get()->result_array();
        if (empty($result)) {
            return false;
        } else {
            $result = $this->format_users($result);

            return $result[0];
        }
    }

    /**
     * Return user data from data source by identifier
     *
     * @param integer $user_id user identifier
     *
     * @return array/false
     */
    public function get_user_by_id($user_id)
    {
        return $this->_get_user("id", $user_id);
    }

    /**
     * Return user data from data source by unique name
     *
     * @param string $login unique name
     *
     * @return array/false
     */
    public function get_user_by_login($login)
    {
        return $this->_get_user("unique_name", $login);
    }

    /**
     * Return user data from data source by confirm code
     *
     * @param string $code confirm code
     *
     * @return array/false
     */
    public function get_user_by_confirm_code($code)
    {
        return $this->_get_user("confirm_code", $code);
    }

    /**
     * Return user data from data source by e-mail and password
     *
     * @param string $email    user email
     * @param string $password user password
     *
     * @return array/false
     */
    public function get_user_by_email_password($email, $password)
    {
        return $this->_get_user("email", $email, "password", $password);
    }

    /**
     * Return user data from data source by e-mail
     *
     * @param string $email user email
     *
     * @return array/false
     */
    public function get_user_by_email($email)
    {
        return $this->_get_user("email", $email);
    }

    /**
     * Return user data from data source by e-mail restore code
     *
     * @param string $code email restore code
     *
     * @return array/false
     */
    public function get_user_by_email_restore_code($code)
    {
        return $this->_get_user("email_restore_code", $code);
    }

    /**
     * Return user data from data source by phone restore code
     *
     * @param string $code phone restore code
     *
     * @return array/false
     */
    public function get_user_by_phone_restore_code($code)
    {
        return $this->_get_user("phone_restore_code", $code);
    }

    /**
     * Return user data from data source by openID
     *
     * @param string $open_id openID key
     *
     * @return array/false
     */
    public function get_user_by_open_id($open_id)
    {
        return $this->_get_user("user_open_id", $user_open_id);
    }

    /**
     * Check user password
     *
     * @param integer $user_id  user identifier
     * @param string  $password user password
     *
     * @return boolean
     */
    public function check_user_password($user_id, $password)
    {
        return $this->_get_user("id", $user_id, "password", $password, false) ? true : false;
    }

    /**
     * Encode password string
     *
     * @param string $password password string
     *
     * @return string
     */
    public function encode_password($password)
    {
        return md5($password);
    }

    /**
     * Encode confirm code
     *
     * @param string $confirm confirm code
     *
     * @return string
     */
    public function encode_confirm($confirm)
    {
        return md5($confirm);
    }

    /**
     * Return sql criteria for user searching as array
     *
     * @param array $filters filters data
     *
     * @return array
     */
    protected function _get_search_criteria($filters)
    {
        $filters = array('data' => $filters, 'table' => USERS_TABLE, 'type' => '');

        $use_optimization = $this->ci->pg_module->get_module_config('users', 'use_optimization');
        if ($use_optimization) {
            $this->ci->load->model('optimize/models/Optimize_users_model');
            $filters = $this->ci->Optimize_users_model->get_filters($filters);
        }

        $params = array();

        $params['table'] = !empty($filters['table']) ? $filters['table'] : USERS_TABLE;

        // By keyword
        if (!empty($filters['data']['keyword'])) {
            $boolean_mode = false;

            if (isset($filters['data']['keyword_mode'])) {
                $boolean_mode = true;
            }

            $filters['data']['keyword'] = trim($filters['data']['keyword']);

            $fe_criteria = array();

            $user_types = $this->getUserTypes();
            array_unshift($user_types, 'user');

            $this->ci->load->model('Field_editor_model');

            foreach ($user_types as $user_type) {
                $this->ci->Field_editor_model->initialize($user_type);
                $fe_criteria_type = array();
                $temp_criteria = $this->ci->Field_editor_model->return_fulltext_criteria($filters['data']['keyword'], $boolean_mode);
                $field_editor_table = USERS_TABLE;
                $fe_table_name = substr($field_editor_table, strlen(DB_PREFIX));
                if (!empty($temp_criteria[$fe_table_name]['where_sql'])) {
                    $fe_criteria[] = str_replace('search_field', $field_editor_table . '.search_field', $temp_criteria[$fe_table_name]['where_sql']);
                }
            }
            if (!empty($fe_criteria)) {
                $fe_criteria = array('where_sql' => array('(' . implode(' OR ', array_unique($fe_criteria)) . ')'));
                if (!empty($filters['data']['field_editor'])) {
                    $filters['data']['field_editor'][0] = array_merge($filters['data']['field_editor'][0], $user_types);
                    $filters['data']['field_editor'][1] = array_merge($filters['data']['field_editor'][1], $fe_criteria);
                } else {
                    $filters['data']['field_editor'] = array($user_types, $fe_criteria);
                }
            }
        }

        // Field editor forms
        if (isset($filters['data']['field_editor_data'])) {
            $field_editor_data = $filters['data']['field_editor_data'];
            unset($filters['data']['field_editor_data']);

            $fe_criteria = array();

            if (isset($filters['data']['type'])) {
                $user_type = $filters['data']['type'];
            } else {
                $user_type = current($this->getUserTypes());
            }

            $this->ci->load->model('field_editor/models/Field_editor_forms_model');
            foreach ($field_editor_data as $form_name => $form_data) {
                foreach ($form_data as $field_gid => $value) {
                    if (isset($value['range']) && is_array($value['range'])) {
                        if (isset($value['range']['min'])) {
                            $form_data[$field_gid . '_min'] = $value['range']['min'];
                        }
                        if (isset($value['range']['max'])) {
                            $form_data[$field_gid . '_max'] = $value['range']['max'];
                        }
                        unset($form_data[$field_gid]);
                    }
                }

                $form_gid = $this->{$form_name . '_user_gid'};
                $temp = $this->ci->Field_editor_forms_model->get_search_criteria($form_gid, $form_data, 'user');
                if (!empty($temp)) {
                    $fe_criteria = array_merge($fe_criteria, $temp);
                }

                $form_gid = $this->{$form_name . '_' . $user_type . '_gid'};
                $temp = $this->ci->Field_editor_forms_model->get_search_criteria($form_gid, $form_data, $user_type);
                if (!empty($temp)) {
                    $fe_criteria = array_merge($fe_criteria, $temp);
                }
            }
            if (!empty($fe_criteria)) {
                if (isset($filters['data']['field_editor'])) {
                    if (!in_array('user', $filters['data']['field_editor'][0])) {
                        $filters['data']['field_editor'][0][] = 'user';
                    }
                    if (!in_array($user_type, $filters['data']['field_editor'][0])) {
                        $filters['data']['field_editor'][0][] = $user_type;
                    }
                    $filters['data']['field_editor'][1] = array_merge($filters['data']['field_editor'][1], $fe_criteria);
                } else {
                    $filters['data']['field_editor'] = array(array('user', $user_type), $fe_criteria);
                }
            }
        }

        if (empty($filters['data'])) {
            return $params;
        }

        $fields = array_flip($this->fields_all);
        foreach ($filters['data'] as $filter_name => $filter_data) {
            if (!is_array($filter_data)) {
                $filter_data = trim($filter_data);
            }
            switch ($filter_name) {
                case 'active':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('status' => 1, 'confirm' => 1)));
                    break;
                case 'not_active':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('status' => 0)));
                    break;
                case 'not_confirm':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('confirm' => 0)));
                    break;
                case 'country':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('id_country' => $filter_data)));
                    break;
                case 'region':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('id_region' => $filter_data)));
                    break;
                case 'city':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('id_city' => $filter_data)));
                    break;
                case 'district':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('id_district' => $filter_data)));
                    break;
                case 'type':
                    if (!$filter_data || !in_array($filter_data, $this->getUserTypes())) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('user_type' => $filter_data)));
                    break;
                case 'zip':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('postal_code' => $filter_data)));
                    break;
                case 'keyword_all':
                    if (empty($filter_data)) {
                        break;
                    }
                    $keyword_o = $this->ci->db->escape($filter_data);
                    $keyword_w = $this->ci->db->escape('%' . $filter_data . '%');
                    $params = array_merge_recursive($params, array('where_sql' => array("(fname LIKE " . $keyword_w . " OR sname LIKE " . $keyword_w . " OR company_name LIKE " . $keyword_o . " OR email LIKE " . $keyword_o . " OR phone LIKE " . $keyword_o . ")")));
                    break;
                case 'name':
                    if (empty($filter_data)) {
                        break;
                    }
                    $name_o = $this->ci->db->escape($filter_data);
                    $name_w = $this->ci->db->escape('%' . $filter_data . '%');
                    $params = array_merge_recursive($params, array('where_sql' => array("(company_name=" . $name_o . " OR fname LIKE " . $name_w . " OR sname LIKE " . $name_w . " OR CONCAT(fname, ' ', sname) LIKE " . $name_w . ")")));
                    break;
                case 'not_user':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array('id !=' => $filter_data)));
                    break;
                // By field editor
                case 'field_editor':
                    if (empty($filter_data)) {
                        break;
                    }
                    foreach ($filter_data[0] as $key => $user_type) {
                        $filter_data[0][$key] = USERS_TABLE;
                    }
                    $filter_data[0] = array_unique($filter_data[0]);
                    foreach ($filter_data[0] as $key => $field_editor_table) {
                        $filter_data[0][$key] = array('table' => $field_editor_table, 'condition' => $params['table'] . '.id=' . $field_editor_table . '.id', 'type' => 'LEFT');
                    }
                    $params = array_merge_recursive($params, array('join' => $filter_data[0]), $filter_data[1]);
                    break;
                // By field
                default:
                    if (isset($fields[$filter_name])) {
                        if (is_array($filter_data)) {
                            $params = array_merge_recursive($params, array('where_in' => array($params['table'] . '.' . $filter_name => $filter_data)));
                        } else {
                            $params = array_merge_recursive($params, array('where' => array($params['table'] . '.' . $filter_name => $filter_data)));
                        }
                    }
                    break;
            }
        }

        return $params;
    }

    /**
     * Return users objects from data source as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param string  $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filters identifiers
     * @param boolean $formatted         format results
     * @param boolean $all               get all properties
     *
     * @return array
     */
    public function get_users_list($page = null, $items_on_page = null, $order_by = null, array $params = array(), array $filter_object_ids = null, $formatted = true, $all = false)
    {
        $table = isset($params['table']) ? $params['table'] : USERS_TABLE;

        $this->ci->db->select($table . '.' . implode(", " . $table . '.', $all ? $this->fields_all : $this->fields_general));
        $this->ci->db->from($table);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            $temp_tables = array();
            foreach ($params['join'] as $join) {
                if ($join['table'] == $table || in_array($join['table'], $temp_tables)) {
                    continue;
                }
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
                $temp_tables[] = $join['table'];
            }
        }

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $filter_object_ids = array_unique($filter_object_ids);
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields_all) || $field == "CONCAT(fname, ' ', sname)" ||
                    $field == "listings_for_sale_count+listings_for_buy_count+" .
                    "listings_for_rent_count+listings_for_lease_count") {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return ($formatted) ? $this->format_users($results) : $results;
        }

        return [];
    }

    /**
     * Return users objects from data source as associated array by identifier
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param string  $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filters identifiers
     * @param boolean $formatted         format results
     *
     * @return array
     */
    public function get_users_list_by_key($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null, $formatted = true)
    {
        $list = $this->get_users_list($page, $items_on_page, $order_by, $params, $filter_object_ids, $formatted);
        if (!empty($list)) {
            foreach ($list as $l) {
                $data[$l["id"]] = $l;
            }

            return $data;
        } else {
            return array();
        }
    }

    /**
     * Return number of users objects in data source
     *
     * @param array $params            sql criteria
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_users_count($params = array(), $filter_object_ids = null)
    {
        $table = isset($params['table']) ? $params['table'] : USERS_TABLE;

        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from($table);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            $temp_tables = array();
            foreach ($params['join'] as $join) {
                if ($join['table'] == $table || in_array($join['table'], $temp_tables)) {
                    continue;
                }
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
                $temp_tables[] = $join['table'];
            }
        }

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $result = $this->ci->db->get()->result();

        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Return filtered users objects from data source as array
     *
     * @param array   $filters       filters data
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param string  $order_by      sorting data
     * @param boolean $formatted     format results
     * @param boolean $all           get all properties
     *
     * @return array
     */
    public function get_users_list_by_filters($filters = array(), $page = null, $items_on_page = null, $order_by = null, $formatted = true, $all = false)
    {
        $params = $this->_get_search_criteria($filters);

        return $this->get_users_list($page, $items_on_page, $order_by, $params, array(), $formatted, $all);
    }

    /**
     * Return number of filtered users objects in data source
     *
     * @param array $filters filters data
     *
     * @return integer
     */
    public function get_users_count_by_filters($filters = array())
    {
        $params = $this->_get_search_criteria($filters);

        return $this->get_users_count($params);
    }

    /**
     * Return filtered users objects to build pagination block
     *
     * @param array   $filters   filters data
     * @param integer $page      page of results
     * @param integer $limits    items per page
     * @param string  $order_by  sorting data
     * @param boolean $formatted format results
     *
     * @return array/false
     */
    public function get_users_pagination($filters = array(), $page = null, $limits = null, $order_by = null, $formatted = true)
    {
        $params = $this->_get_search_criteria($filters);

        $table = isset($params['table']) ? $params['table'] : USERS_TABLE;

        $this->ci->db->select($table . '.id');
        $this->ci->db->from($table);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                //if(in_array($field, $this->fields_general)){
                $this->ci->db->order_by($field . " " . $dir);
                //}
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits + ($page > 1 ? 4 : 2), $limits * ($page - 1) - ($page > 1 ? 2 : 0));
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $user_types = array();
            $extended = array();
            foreach ($results as $r) {
                if (in_array($r["user_type"], $this->getUserTypes())) {
                    $user_types[$r["user_type"]][] = $r["id"];
                }
            }
            foreach ($user_types as $user_type => $ids) {
                $rs = $this->ci->db->select(implode(", ", $this->fields_general))->from(USERS_TABLE)->where_in("id", $ids)->get()->result_array();
                foreach ($rs as $r) {
                    $extended[$r["id"]] = $r;
                }
            }
            foreach ($results as $r) {
                $r = isset($extended[$r["id"]]) ? array_merge($r, $extended[$r["id"]]) : $r;
                $data[] = $r;
            }
            if ($formatted) {
                $data = $this->format_users($data);
            }

            return $data;
        }

        return false;
    }

    /**
     * Return objects of active users from data source as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param string  $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filters identifiers
     * @param boolean $formatted         format results
     *
     * @return array
     */
    public function get_active_users_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null, $formatted = true)
    {
        $params["where"]["status"] = 1;
        $params["where"]["confirm"] = 1;

        return $this->get_users_list($page, $items_on_page, $order_by, $params, $filter_object_ids, $formatted);
    }

    /**
     * Return number of active users in data source
     *
     * @param array $params            sql parameters
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_active_users_count($params = array(), $filter_object_ids = null)
    {
        $params["where"]["status"] = 1;
        $params["where"]["confirm"] = 1;

        return $this->get_users_count($params, $filter_object_ids);
    }

    /**
     * Return objects of blocked users from data source as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param string  $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filters identifiers
     * @param boolean $formatted         format results
     *
     * @return array
     */
    public function get_not_active_users_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null, $formatted = true)
    {
        $params["where"]["status"] = 0;

        return $this->get_users_list($page, $items_on_page, $order_by, $params, $filter_object_ids, $formatted);
    }

    /**
     * Return number of blocked users in data source
     *
     * @param array $params            sql criteria
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_not_active_users_count($params = array(), $filter_object_ids = null)
    {
        $params["where"]["status"] = 0;

        return $this->get_users_count($params, $filter_object_ids);
    }

    /**
     * Save user object to data source
     *
     * @param integer $user_id             user identifier
     * @param array   $attrs               user data
     * @param string  $file_name           upload file name
     * @param boolean $use_icon_moderation use moderation for upload logo
     *
     * @return integer
     */
    public function save_user($user_id = null, $attrs = array(), $file_name = '', $use_icon_moderation = true)
    {
        if (!$user_id) {
            $user_old_data = array();
        } else {
            $format_settings = $this->format_settings;
            $this->format_settings = array('use_format', false);
            $user_old_data = $this->get_user_by_id($user_id);
            $this->format_settings = $format_settings;
        }

        if (!empty($attrs)) {
            if (isset($attrs['roles'])) {
                $attrs['roles'] = $this->rolesEncode($attrs['roles']);
            }
            if (is_null($user_id)) {
                if (!isset($attrs["group_id"])) {
                    $this->ci->load->model("users/models/Users_groups_model");
                    $attrs["group_id"] = $this->ci->Users_groups_model->get_default_group_id();
                }
                $attrs["date_created"] = $attrs["date_modified"] = date("Y-m-d H:i:s");
                $this->ci->db->insert(USERS_TABLE, $attrs);
                $user_id = $this->ci->db->insert_id();
            } else {
                $attrs["date_modified"] = date("Y-m-d H:i:s");
                $this->ci->db->where("id", $user_id);
                $this->ci->db->update(USERS_TABLE, $attrs);
            }
        }

        if (!empty($file_name) && !empty($user_id) && !empty($_FILES[$file_name]) && is_uploaded_file($_FILES[$file_name]["tmp_name"])) {
            $this->ci->load->model("Uploads_model");

            $user = $this->get_user_by_id($user_id);
            if ($user['user_logo']) {
                $this->delete_logo($user_id);
            }

            $img_return = $this->ci->Uploads_model->upload($this->upload_config_id, $user_id . "/", $file_name);
            if (empty($img_return["errors"])) {
                if ($use_icon_moderation) {
                    if ($this->ci->pg_module->is_module_active("moderation")) {
                        $this->ci->load->model("Moderation_model");
                        $mstatus = intval($this->ci->Moderation_model->get_moderation_type_status($this->moderation_type));
                    } else {
                        $mstatus = 2;
                    }
                } else {
                    $mstatus = 2;
                }
                if ($mstatus != 2) {
                    $this->ci->Moderation_model->add_moderation_item($this->moderation_type, $user_id);
                    if ($mstatus == 1) {
                        $img_data["user_logo"] = $img_return["file"];
                        $img_data["user_logo_moderation"] = "";
                    } else {
                        $img_data["user_logo_moderation"] = $img_return["file"];
                    }
                } else {
                    $img_data["user_logo"] = $img_return["file"];
                }

                $this->save_user($user_id, $img_data);
            }
        }

        $format_settings = $this->format_settings;
        $this->format_settings = array('use_format', false);
        $user_new_data = $this->get_user_by_id($user_id);
        $this->format_settings = $format_settings;

        // update fulltext
        $this->ci->load->model('Field_editor_model');
        $this->ci->Field_editor_model->initialize('user');
        $this->ci->Field_editor_model->update_fulltext_field($user_id);

        // update extern data
        $this->ci->load->model('users/models/Users_events_model');
        $this->ci->Users_events_model->onsave_user($user_old_data, $user_new_data);

        return $user_id;
    }

    /**
     * Assign agent to company
     *
     * @param integer $id_user      user identifier
     * @param integer $id_company   company identifier
     * @param boolean $approve_wait awaiting approval
     *
     * @return boolean
     */
    public function save_agent_company($id_user, $id_company, $approve_wait = true, $add_indicator = true)
    {
        $this->set_format_settings('get_safe', false);
        $agent = $this->get_user_by_id($id_user);
        $this->set_format_settings('get_safe', true);

        if ($id_company == $agent['agent_company']) {
            return false;
        }

        $save_data = array(
            'agent_company' => $id_company,
            'agent_date' => date('Y-m-d H:i:s'),
            'agent_status' => 0,
        );
        $this->save_user($id_user, $save_data);

        $this->ci->load->model("Notifications_model");
        if (!empty($agent['agent_company'])) {
            $this->set_format_settings('get_safe', false);
            $old_company = $this->get_user_by_id($agent['agent_company']);
            $this->set_format_settings('get_safe', true);
            $this->update_agent_company_count($agent['agent_company']);
            $mail_data = array(
                'fname' => $old_company['fname'],
                'sname' => $old_company['sname'],
                'user_name' => $agent['output_name'],
                'company_name' => $old_company['output_name'],
            );
            $this->ci->Notifications_model->send_notification(
                $old_company["email"], ($agent['agent_status'] ? "agent_unjoin" : "agent_cancel_request"), $mail_data, '', $old_company["lang_id"]
            );
        }

        if ($id_company != 0 && $approve_wait) {
            $this->set_format_settings('get_safe', false);
            $company = $this->get_user_by_id($id_company);
            $this->set_format_settings('get_safe', true);
            $mail_data = array(
                'fname' => $company['fname'],
                'sname' => $company['sname'],
                'user_name' => $agent['output_name'],
                'company_name' => $company['output_name'],
            );
            $this->ci->Notifications_model->send_notification($company["email"], "agent_send_request", $mail_data, '', $company["lang_id"]);
        }

        if ($add_indicator) {
            $this->ci->load->model('menu/models/Indicators_model');
            if ($id_company) {
                $this->ci->Indicators_model->add('new_agent_item', $id_user, $id_company);
                $this->ci->Indicators_model->add('all_agent_item', 'request_' . $id_user, $id_company);
            } elseif (0 === $id_company) {
                $this->ci->Indicators_model->delete('new_agent_item', $id_user, true);
                $this->ci->Indicators_model->delete('all_agent_item', 'request_' . $id_user, true);
            }
        }

        return true;
    }

    /**
     * Change company of agent request
     *
     * @param integer $id_user    user identifier
     * @param integer $id_company company identifier
     * @param string  $status     request status
     *
     * @return void
     */
    public function save_agent_company_status($id_user, $id_company, $status = 'accept')
    {
        $this->set_format_settings('get_safe', false);
        $data['agent'] = $this->get_user_by_id($id_user);
        $data['company'] = $this->get_user_by_id($id_company);
        $this->set_format_settings('get_safe', true);

        $mail_data = array(
            'fname' => $data['agent']['fname'],
            'sname' => $data['agent']['sname'],
            'user_name' => $data['agent']['output_name'],
            'company_name' => $data['company']['output_name'],
        );
        $this->ci->load->model("Notifications_model");

        if ($status == 'accept') {
            $save_data = array(
                'agent_company' => $id_company,
                'agent_date' => date('Y-m-d H:i:s'),
                'agent_status' => 1,
            );
            $this->save_user($id_user, $save_data);
            $this->update_agent_company_count($id_company);

            $this->ci->load->model("Linker_model");
            $this->ci->Linker_model->add_link("users_contacts", $agent_id, $id_company);
            $this->ci->Linker_model->add_link("users_contacts", $id_company, $agent_id);

            $this->ci->Notifications_model->send_notification($data['agent']["email"], "agent_request_accepted", $mail_data, '', $data['agent']["lang_id"]);
        } elseif ($status == 'decline') {
            $save_data = array(
                'agent_company' => 0,
                'agent_date' => date('Y-m-d H:i:s'),
                'agent_status' => 0,
            );
            $this->save_user($id_user, $save_data);
            $this->ci->Notifications_model->send_notification($data['agent']["email"], "agent_request_declined", $mail_data, '', $data['agent']["lang_id"]);
        } elseif ($status == 'delete') {
            $save_data = array(
                'agent_company' => 0,
                'agent_date' => date('Y-m-d H:i:s'),
                'agent_status' => 0,
            );
            $this->save_user($id_user, $save_data);
            $this->update_agent_company_count($id_company);
            $this->ci->Notifications_model->send_notification($data['agent']["email"], "agent_deleted", $mail_data, '', $data['agent']["lang_id"]);
        }
    }

    /**
     * Update number of agent in company
     *
     * @param integer $id_company company identifier
     *
     * @return void
     */
    public function update_agent_company_count($id_company)
    {
        $params["where"]["agent_company"] = $id_company;
        $params["where"]["agent_status"] = 1;
        $save_data['agent_count'] = $this->get_users_count($params);
        $this->save_user($id_company, $save_data);
    }

    /**
     * Remove logo image from data source
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function delete_logo($user_id)
    {
        $this->ci->load->model("Uploads_model");

        $user = $this->get_user_by_id($user_id);
        if ($user["user_logo_moderation"]) {
            $this->ci->Uploads_model->delete_upload($this->upload_config_id, $user_id . "/", $user['user_logo_moderation']);
            $save_data['user_logo_moderation'] = '';

            if ($this->ci->pg_module->is_module_active("moderation")) {
                $this->ci->load->model('Moderation_model');
                $this->ci->Moderation_model->delete_moderation_item_by_obj($this->moderation_type, $user_id);
            }
        } elseif ($user["user_logo"]) {
            $this->ci->Uploads_model->delete_upload($this->upload_config_id, $user_id . "/", $user["user_logo"]);
            $save_data["user_logo"] = "";
        }
        $this->save_user($user_id, $save_data);
    }

    /**
     * Activate/deactivate user object
     *
     * Available status values: 1 - activate, 0 -deactivate
     *
     * @param integer $user_id user identifier
     * @param integer $status  activity status
     *
     * @return void
     */
    public function activate_user($user_id, $status = 1)
    {
        $attrs["status"] = intval($status);
        $attrs["date_modified"] = date("Y-m-d H:i:s");
        $this->ci->db->where("id", $user_id);
        $this->ci->db->update(USERS_TABLE, $attrs);
    }

    /**
     * Generate unique guid by user data
     *
     * @param integer $id   user identifier
     * @param array   $data user data
     *
     * @return string
     */
    protected function _create_unique_gid($id, $data)
    {
        $scheme = $this->user_gid_scheme[$data['user_type']]['str'];

        $this->ci->load->library('Translit');
        $current_lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);
        $current_lang_code = $current_lang['code'];

        // [fname]
        if ($data["fname"]) {
            $name_replace = strtolower(strip_tags($data["fname"]));
            $name_replace = $this->ci->translit->convert($current_lang_code, $name_replace);
            $name_replace = preg_replace("/[\n\s\t]+/i", '-', $name_replace);
            $name_replace = preg_replace("/[^a-z0-9\-_]+/i", '', $name_replace);
            $name_replace = preg_replace("/[\-]{2,}/i", '-', $name_replace);
            $name_replace = ucfirst($name_replace);
        } else {
            $name_replace = "";
        }
        $scheme = str_replace("[fname]", $name_replace, $scheme);

        // [sname]
        if ($data["sname"]) {
            $name_replace = strtolower(strip_tags($data["sname"]));
            $name_replace = $this->ci->translit->convert($current_lang_code, $name_replace);
            $name_replace = preg_replace("/[\n\s\t]+/i", '-', $name_replace);
            $name_replace = preg_replace("/[^a-z0-9\-_]+/i", '', $name_replace);
            $name_replace = preg_replace("/[\-]{2,}/i", '-', $name_replace);
            $name_replace = ucfirst($name_replace);
        } else {
            $name_replace = "";
        }
        $scheme = str_replace("[sname]", $name_replace, $scheme);

        // [company_name]
        if (!empty($data["company_name"])) {
            $name_replace = strtolower(strip_tags($data["company_name"]));
            $name_replace = $this->ci->translit->convert($current_lang_code, $name_replace);
            $name_replace = preg_replace("/[\n\s\t]+/i", '-', $name_replace);
            $name_replace = preg_replace("/[^a-z0-9\-_]+/i", '', $name_replace);
            $name_replace = preg_replace("/[\-]{2,}/i", '-', $name_replace);
            $name_replace = ucfirst($name_replace);
        } else {
            $name_replace = "";
        }
        $scheme = str_replace("[company_name]", $name_replace, $scheme);

        $param["where"]["unique_name"] = $scheme;
        if (!empty($id)) {
            $param["where"]["id !="] = $id;
        }
        $gid_counts = $this->get_users_count($param);
        if ($gid_counts > 0) {
            if (strlen($scheme) > 96) {
                $scheme = substr($scheme, 0, 96);
            }
            $scheme = $scheme . "-" . rand(100, 999);
        }

        return $scheme;
    }

    /**
     * Validate settings of site mode
     *
     * @param array $data settings data
     *
     * @return array
     */
    public function validate_settings($data)
    {
        $return = array("errors" => array(), "data" => array());

        $fields = array();

        foreach ($this->getUserTypes() as $user_type) {
            if (isset($data[$user_type . '_login_enabled'])) {
                $return["data"][$user_type . '_login_enabled'] = intval($data[$user_type . '_login_enabled']) ? 1 : 0;
                if (!$return["data"][$user_type . '_login_enabled']) {
                    $data[$user_type . '_register_enabled'] = 0;
                    $data[$user_type . '_register_mail_confirm'] = 0;
                }
            }
            if (isset($data[$user_type . '_register_enabled'])) {
                $return["data"][$user_type . '_register_enabled'] = intval($data[$user_type . '_register_enabled']) ? 1 : 0;
                if ($return["data"][$user_type . '_register_enabled'] == 1 && !isset($return["data"][$user_type . '_login_enabled'])) {
                    $return["data"][$user_type . '_login_enabled'] = 1;
                }
            }
            if (isset($data[$user_type . '_register_mail_confirm'])) {
                $return["data"][$user_type . '_register_mail_confirm'] = intval($data[$user_type . '_register_mail_confirm']) ? 1 : 0;
                if ($return["data"][$user_type . '_register_mail_confirm'] == 1 && !isset($return["data"][$user_type . '_login_enabled'])) {
                    $return["data"][$user_type . '_login_enabled'] = 1;
                }
                if ($return["data"][$user_type . '_register_mail_confirm'] == 1 && !isset($return["data"][$user_type . '_register_enabled'])) {
                    $return["data"][$user_type . '_register_enabled'] = 1;
                }
            }
            if (isset($data[$user_type . '_search_enabled'])) {
                $return["data"][$user_type . '_search_enabled'] = intval($data[$user_type . '_search_enabled']) ? 1 : 0;
            }
        }

        return $return;
    }

    /**
     * Validate user data for saving to data source
     *
     * @param integer $id        user identifier
     * @param array   $data      user data
     * @param string  $file_name upload name
     *
     * @return array
     */
    public function validate_user($id, $data, $file_name = '')
    {
        $return = array("errors" => array(), "data" => array());

        if ($id) {
            $this->set_format_settings('use_format', false);
            $user = $this->get_user_by_id($id);
            $this->set_format_settings('use_format', true);
        } else {
            $user = array();
        }

        $this->ci->config->load("reg_exps", true);

        $is_moderation_installed = $this->ci->pg_module->is_module_active("moderation");

        if (isset($data["id"])) {
            $return["data"]["id"] = intval($data["id"]);
        }

        if (isset($data["roles"])) {
            $return["data"]["roles"] = $data["roles"];
        }

        // user type
        if (isset($data['user_type'])) {
            $return["data"]["user_type"] = trim(strip_tags($data["user_type"]));
            if (empty($return["data"]["user_type"]) || !in_array($return["data"]["user_type"], $this->getUserTypes())) {
                $return["errors"][] = l("error_user_type_incorrect", "users");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['user_type']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_user_type', 'users'), l("error_badwords_field", "users"));
                    } else {
                        $user['user_type'] = $return['data']['user_type'];
                    }
                } else {
                    $user['user_type'] = $return['data']['user_type'];
                }
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_user_type_incorrect", "users");
        }

        if (isset($data["confirm"])) {
            $return["data"]["confirm"] = intval($data["confirm"]);
        }

        if (isset($data['confirm_code'])) {
            $return["data"]["confirm_code"] = trim(strip_tags($data["confirm_code"]));
        }

        if (isset($data["status"])) {
            $return["data"]["status"] = intval($data["status"]);
        }

        if (isset($data["lang_id"])) {
            $return["data"]["lang_id"] = intval($data["lang_id"]);
        }

        if (isset($data["id_currency"])) {
            $return["data"]["id_currency"] = intval($data["id_currency"]);
        }

        if (isset($data["group_id"])) {
            $return["data"]["group_id"] = intval($data["group_id"]);
        }

        if (isset($data["agent_date"])) {
            $return["data"]["agent_date"] = $data["agent_date"];
        }

        // email
        if (isset($data['email'])) {
            $return["data"]["email"] = trim(strip_tags($data["email"]));

            $email_expr = $this->ci->config->item("email", "reg_exps");
            if (empty($return["data"]["email"]) || !preg_match($email_expr, $return["data"]["email"])) {
                $return["errors"][] = l("error_email_incorrect", "users");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['email']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_email', 'users'), l("error_badwords_field", "users"));
                    }
                }
            }

            $eparams["where"]["email"] = $return["data"]["email"];
            if ($id) {
                $eparams["where"]["id <>"] = $id;
            }
            $count = $this->get_users_count($eparams);
            if ($count > 0) {
                $return["errors"][] = l("error_email_already_exists", "users");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_email", "users");
        }

        // fname
        if (isset($data['fname'])) {
            $return["data"]["fname"] = trim(strip_tags($data["fname"]));
            if (empty($return["data"]["fname"])) {
                $return["errors"][] = l("error_empty_fname", "users");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['fname']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_fname', 'users'), l("error_badwords_field", "users"));
                    }
                }
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_fname", "users");
        }

        // sname
        if (isset($data['sname'])) {
            $return["data"]["sname"] = trim(strip_tags($data["sname"]));
            if (empty($return["data"]["sname"])) {
                $return["errors"][] = l("error_empty_sname", "users");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['sname']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_sname', 'users'), l("error_badwords_field", "users"));
                    }
                }
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_sname", "users");
        }

        // phone
        if (isset($data['phone'])) {
            $return["data"]["phone"] = trim(strip_tags($data["phone"]));
            if (empty($return["data"]["phone"])) {
                $return["errors"][] = l("error_empty_phone", "users");
            } else {
                $this->ci->load->helper("start");
                $phone_format = get_phone_format('regexp');
                if ($phone_format !== false && !preg_match($phone_format, $return["data"]["phone"])) {
                    $return["errors"][] = l("error_phone_incorrect", "users");
                } else {
                    if ($is_moderation_installed) {
                        $this->ci->load->model("moderation/models/Moderation_badwords_model");
                        $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['phone']);
                        if ($bw_count) {
                            $return['errors'][] = str_replace('[field]', l('field_phone', 'users'), l("error_badwords_field", "users"));
                        }
                    }
                }
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_phone", "users");
        }

        // password
        if (isset($data['password'])) {
            $return["data"]["password"] = trim($data["password"]);
            $data["password"] = trim(strip_tags($data["password"]));

            if ($return["data"]["password"] != $data["repassword"]) {
                $return["errors"][] = l("error_pass_repass_not_equal", "users");
            } else {
                $password_expr = $this->ci->config->item("password", "reg_exps");
                if (!preg_match($password_expr, $return["data"]["password"])) {
                    $return["errors"][] = l("error_password_incorrect", "users");
                }
                $return["data"]["password"] = $this->encode_password($return["data"]["password"]);
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_password_empty", "users");
        }

        if (isset($data["password_old"]) && $id) {
            $data["password_old"] = $this->encode_password(trim($data["password_old"]));

            $format_settings_old = $this->format_settings;
            $this->format_settings = array('use_format' => false);
            $userdata = $this->get_user_by_id($id);
            $this->format_settings = $format_settings_old;
            if ($data["password_old"] != $userdata['password']) {
                $return["errors"][] = l("error_password_old_invalid", "users");
            }
        }

        // logo
        if (isset($data["user_logo"])) {
            $return["data"]["user_logo"] = strip_tags($data["user_logo"]);
        }

        if (isset($data["user_logo_moderation"])) {
            $return["data"]["user_logo_moderation"] = strip_tags($data["user_logo_moderation"]);
        }

        if (isset($data["contact_email"])) {
            $return["data"]["contact_email"] = trim(strip_tags($data["contact_email"]));
            $email_expr = $this->ci->config->item("email", "reg_exps");
            if (!empty($return["data"]["contact_email"]) && !preg_match($email_expr, $return["data"]["contact_email"])) {
                $return["errors"][] = l("error_contact_email_incorrect", "users");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['contact_email']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_contact_email', 'users'), l("error_badwords_field", "users"));
                    }
                }
            }
        }

        if (isset($data["contact_phone"])) {
            $return["data"]["contact_phone"] = trim(strip_tags($data["contact_phone"]));
            if (!empty($return["data"]["contact_phone"])) {
                $this->ci->load->helper("start");
                $phone_format = get_phone_format('regexp');
                if ($phone_format !== false && !preg_match($phone_format, $return["data"]["contact_phone"])) {
                    $return["errors"][] = l("error_contact_phone_incorrect", "users");
                } else {
                    if ($is_moderation_installed) {
                        $this->ci->load->model("moderation/models/Moderation_badwords_model");
                        $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['contact_phone']);
                        if ($bw_count) {
                            $return['errors'][] = str_replace('[field]', l('field_contact_phone', 'users'), l("error_badwords_field", "users"));
                        }
                    }
                }
            }
        }

        if (isset($data["account"])) {
            $return["data"]["account"] = intval($data["account"]);
        }

        if (isset($data['featured_end_date'])) {
            $value = strtotime($data['featured_end_date']);
            if ($value > 0) {
                $return['data']['featured_end_date'] = date("Y-m-d", $value);
                $return['data']['is_featured'] = 1;
            } else {
                $return['data']['featured_end_date'] = '0000-00-00 00:00:00';
                $return['data']['is_featured'] = 0;
            }
        }

        if (isset($data['show_logo_end_date'])) {
            $value = strtotime($data['show_logo_end_date']);
            if ($value > 0) {
                $return['data']['show_logo_end_date'] = date("Y-m-d", $value);
                $return['data']['is_show_logo'] = 1;
            } else {
                $return['data']['show_logo_end_date'] = '0000-00-00 00:00:00';
                $return['data']['is_show_logo'] = 0;
            }
        }

        switch ($user['user_type']) {
            case 'private':
                // standart contact fields
                break;
            case 'company':
                // company name (required)
                if (isset($data['company_name'])) {
                    $return["data"]["company_name"] = trim(strip_tags($data["company_name"]));
                    if (empty($return["data"]["company_name"])) {
                        $return["errors"][] = l("error_empty_company_name", "users");
                    } else {
                        if ($is_moderation_installed) {
                            $this->ci->load->model("moderation/models/Moderation_badwords_model");
                            $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['company_name']);
                            if ($bw_count) {
                                $return['errors'][] = str_replace('[field]', l('field_company', 'users'), l("error_badwords_field", "users"));
                            }
                        }
                    }
                } elseif (!$id) {
                    $return["errors"][] = l("error_empty_company_name", "users");
                }

                if (isset($data["id_country"])) {
                    $return["data"]["id_country"] = strval($data["id_country"]);
                }

                if (isset($data["id_region"])) {
                    $return["data"]["id_region"] = intval($data["id_region"]);
                }

                if (isset($data["id_city"])) {
                    $return["data"]["id_city"] = intval($data["id_city"]);
                }

                if (isset($data["id_district"])) {
                    $return["data"]["id_district"] = intval($data["id_district"]);
                }

                if (isset($data["address"])) {
                    $return["data"]["address"] = trim(strip_tags($data["address"]));
                    if (!empty($return["data"]["address"])) {
                        if ($is_moderation_installed) {
                            $this->ci->load->model("moderation/models/Moderation_badwords_model");
                            $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['address']);
                            if ($bw_count) {
                                $return['errors'][] = str_replace('[field]', l('field_address', 'users'), l("error_badwords_field", "users"));
                            }
                        }
                    }
                }

                if (isset($data["postal_code"])) {
                    $return["data"]["postal_code"] = trim(strip_tags($data["postal_code"]));
                    if (!empty($return["data"]["postal_code"])) {
                        if ($is_moderation_installed) {
                            $this->ci->load->model("moderation/models/Moderation_badwords_model");
                            $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['postal_code']);
                            if ($bw_count) {
                                $return['errors'][] = str_replace('[field]', l('field_postal_code', 'users'), l("error_badwords_field", "users"));
                            }
                        }
                    }
                }

                if (isset($data["lat"])) {
                    $return["data"]["lat"] = (float) $data["lat"];
                }

                if (isset($data["lon"])) {
                    $return["data"]["lon"] = (float) $data["lon"];
                }

                if (isset($data["agent_count"])) {
                    $return["data"]["agent_count"] = intval($data["agent_count"]);
                }
                break;
            case 'agent':
                if (isset($data["agent_company"])) {
                    $return["data"]["agent_company"] = intval($data["agent_company"]);
                    if (empty($return["data"]["agent_company"])) {
                        $return["errors"][] = l("error_empty_agent_company", "users");
                    }
                }

                if (isset($data["agent_status"])) {
                    $return["data"]["agent_status"] = $data["agent_status"] ? 1 : 0;
                }

                if (isset($data["lat"])) {
                    $return["data"]["lat"] = (float) $data["lat"];
                }

                if (isset($data["lon"])) {
                    $return["data"]["lon"] = (float) $data["lon"];
                }
                break;
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date('Y-m-d H:i:s', $value);
            } else {
                $return['data']['date_created'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date('Y-m-d H:i:s', $value);
            } else {
                $return['data']['date_modified'] = '0000-00-00 00:00:00';
            }
        }

        // field editor validate
        $this->ci->load->model('Field_editor_model');
        $this->ci->Field_editor_model->initialize('user');
        $params = array('where' => array('editor_type_gid' => 'user'));
        $validate_data = $this->ci->Field_editor_model->validate_fields_for_save($params, $data);
        $fields = $this->ci->Field_editor_model->get_fields_list($params);
        foreach ($fields as $field_gid => $field_data) {
            if ($field_data['field_type'] != 'text' && $field_data['field_type'] != 'textarea') {
                continue;
            }
            if (isset($data[$field_data['field_name']]) && !empty($data[$field_data['field_name']])) {
                $data[$field_data['field_name']] = trim(strip_tags($data[$field_data['field_name']]));
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $data[$field_data['field_name']]);
                    if ($bw_count) {
                        $message = str_replace('[field]', $field_data['name'], l('error_badwords_field', 'users'));
                        $validate_data['errors'][] = $message;
                    }
                }
            }
        }
        if (!empty($validate_data['errors'])) {
            $return['errors'] = array_merge($return['errors'], $validate_data['errors']);
        } else {
            foreach ($validate_data['data'] as $field => $value) {
                $return['data'][$field] = $value;
            }
        }

        if (isset($user['user_type'])) {
            $this->ci->load->model('Field_editor_model');
            $this->ci->Field_editor_model->initialize($user['user_type']);
            $params = array('where' => array('editor_type_gid' => $user['user_type']));
            $validate_data = $this->ci->Field_editor_model->validate_fields_for_save($params, $data);
            $fields = $this->ci->Field_editor_model->get_fields_list($params);
            foreach ($fields as $field_gid => $field_data) {
                if ($field_data['field_type'] != 'text' && $field_data['field_type'] != 'textarea') {
                    continue;
                }
                if (isset($data[$field_data['field_name']]) && !empty($data[$field_data['field_name']])) {
                    $data[$field_data['field_name']] = trim(strip_tags($data[$field_data['field_name']]));
                    if ($is_moderation_installed) {
                        $this->ci->load->model('moderation/models/Moderation_badwords_model');
                        $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $data[$field_data['field_name']]);
                        if ($bw_count) {
                            $message = str_replace('[field]', $field_data['name'], l('error_badwords_field', 'users'));
                            $validate_data['errors'][] = $message;
                        }
                    }
                }
            }
            if (!empty($validate_data['errors'])) {
                $return['errors'] = array_merge($return['errors'], $validate_data['errors']);
            } else {
                foreach ($validate_data['data'] as $field => $value) {
                    $return['data'][$field] = $value;
                }
            }
        }

        if (isset($data['id_seo_settings'])) {
            $return['data']['id_seo_settings'] = intval($data['id_seo_settings']);
        }

        // create unique_name
        $allow_create_unique_name = false;
        if (!empty($return['data']['user_type'])) {
            $allow_create_unique_name = true;
            if (isset($this->user_gid_scheme[$return['data']['user_type']]['fields'])) {
                foreach ($this->user_gid_scheme[$return['data']['user_type']]['fields'] as $scheme_field) {
                    if (!isset($return["data"][$scheme_field])) {
                        $allow_create_unique_name = false;
                    }
                }
            } else {
                $allow_create_unique_name = false;
            }
        }
        if ($allow_create_unique_name) {
            $return["data"]["unique_name"] = $this->_create_unique_gid($id, $return["data"]);
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_unique_name", "users");
        }

        if (isset($data["unique_name"])) {
            $return["data"]["unique_name"] = trim(strip_tags($data["unique_name"]));
            if (empty($return["data"]["unique_name"])) {
                $return["errors"][] = l("error_empty_unique_name", "users");
            }
        }

        // validate logo
        if ($file_name && isset($_FILES[$file_name]) && is_array($_FILES[$file_name]) && is_uploaded_file($_FILES[$file_name]["tmp_name"])) {
            $this->ci->load->model("Uploads_model");
            $validate_image = $this->ci->Uploads_model->validate_upload($this->upload_config_id, $file_name);
            if (!empty($validate_image["error"])) {
                $return["errors"] = array_merge($return["errors"], (array) $validate_image["error"]);
            }
        }

        return $return;
    }

    /**
     * Remove user object from data source by identifier
     *
     * Listings objects won't be deleted
     *
     * @param integer $user_id user identifier
     *
     * @return array
     */
    public function delete_user($user_id)
    {
        $data = $this->get_user_by_id($user_id);
        $this->ci->db->where("id", $user_id);
        $this->ci->db->delete(USERS_TABLE);

        // update agents
        if ($data['user_type'] == 'agent' && $data['agent_status']) {
            $this->update_agent_company_count($data['agent_company']);
        }

        // delete uploads
        $this->ci->load->model("Uploads_model");
        if ($data["user_logo_moderation"]) {
            $this->ci->Uploads_model->delete_upload($this->upload_config_id, $user_id . "/", $data["user_logo_moderation"]);
        }
        if ($data["user_logo"]) {
            $this->ci->Uploads_model->delete_upload($this->upload_config_id, $user_id . "/", $data["user_logo"]);
        }

        // delete moderation
        if ($this->ci->pg_module->is_module_active("moderation")) {
            $this->ci->load->model("Moderation_model");
            $this->ci->Moderation_model->delete_moderation_item_by_obj($this->moderation_type, $user_id);
        }

        // delete links
        $this->delete_user_contacts($user_id);

        // delete seo settings
        if ($data['id_seo_settings'] != 0) {
            $this->ci->load->model('Seo_model');
            $this->ci->Seo_model->delete_seo_tags($data['id_seo_settings']);
        }

        // delete external data
        $this->ci->load->model('users/models/Users_events_model');
        $this->ci->Users_events_model->ondelete_user($data);
    }

    /**
     * Validate upload file of user logo
     *
     * @param string $file_name upload name
     *
     * @return array
     */
    public function validate_logo($file_name)
    {
        $return = array('errors' => array(), 'data' => array());
        if (isset($_FILES[$file_name]) && is_array($_FILES[$file_name]) && is_uploaded_file($_FILES[$file_name]["tmp_name"])) {
            $this->ci->load->model("Uploads_model");
            $validate_image = $this->ci->Uploads_model->validate_upload($this->upload_config_id, $file_name);
            $return["errors"] = $validate_image["error"];
        }

        return $return;
    }

    /**
     * Add roles to user
     * @param mixed $user User id or array of user data including roles
     * @param array $new_roles
     * @return array
     */
    public function addRoles($user, array $new_roles)
    {
        if (is_int($user)) {
            $db_user = $this->get_user_by_id($user);
            $user = $this->save_user($db_user['id'], [
                'roles' => array_unique(array_merge($this->rolesDecode($db_user['roles']), $new_roles))
            ]);
        } else {
            $user['roles'] = array_unique(array_merge($this->rolesDecode($user['roles']), $new_roles));
        }
        return $user;
    }

    /**
     * Get user roles
     * @param int $user_id
     * @return array
     */
    public function getUserRoles($user_id)
    {
        $user = $this->get_user_by_id($user_id);

        if (empty($user)) {
            return [];
        }

        return $this->rolesDecode($user['roles']);
    }

    /**
     * Decode roles
     * @param string $roles
     * @return array
     * @throws \InvalidArgumentException
     */
    private function rolesDecode($roles)
    {
        if (is_string($roles)) {
            $roles = explode(',', $roles);
        } elseif (!is_array($roles)) {
            throw new \InvalidArgumentException('(users) Wrong roles format');
        }
        return $roles;
    }

    /**
     * Encode roles
     * @param array $roles
     * @return string
     * @throws \InvalidArgumentException
     */
    private function rolesEncode($roles)
    {
        if (is_array($roles)) {
            $roles = implode(',', $roles);
        } elseif (!is_string($roles)) {
            throw new \InvalidArgumentException('(users) Wrong roles format');
        }
        return $roles;
    }

    /**
     * Format users data
     *
     * @param array $users users data
     *
     * @return array
     */
    public function format_users($users)
    {
        if (empty($this->format_settings['use_format']) || empty($users)) {
            return $users;
        }

        $users_ids = array();
        $users_types_ids = array();
        $user_for_location = array();

        $reference = $this->ci->pg_language->ds->get_reference('start', 'week_day_full');
        $weekdays = $reference['option'];
        $reference = $this->ci->pg_language->ds->get_reference('start', 'dayhour-names');
        $dayhours = $reference['option'];

        $group_ids = array();
        $company_ids = array();
        $language_ids = array();
        $currency_ids = array();

        if ($this->format_settings['get_currency']) {
            if (!$this->ci->pg_module->is_module_active('currencies')) {
                $this->format_settings['get_currency'] = false;
            }
        }

        $this->ci->load->model('Uploads_model');

        foreach ($users as $key => $user) {
            // postfix for images path
            if (!empty($user['id'])) {
                if (!isset($users_types_ids['user'])) {
                    $users_types_ids['user'] = array();
                }

                if (!isset($users_types_ids[$user['user_type']])) {
                    $users_types_ids[$user['user_type']] = array();
                }

                $users_ids[] = $user['id'];
                $users_types_ids[$user['user_type']][] = $user['id'];
                $users_types_ids['user'][] = $user['id'];
                $user['postfix'] = $user["id"];
                $user['user_type_str'] = l($user['user_type'], 'users');
                if ($user['user_type'] == 'company') {
                    $user['output_name'] = $user['company_name'];
                } else {
                    $user['output_name'] = trim($user['fname'] . ' ' . $user['sname']);
                }

                $user['name'] = $user['output_name'];
                $user['user_logo_alt'] = l('text_user_logo', 'users', null, 'text', array('user_name' => $user['output_name']));

                switch ($user['user_type']) {
                    case 'private':
                        $user['is_map'] = 0;
                        break;
                    case 'company':
                        if (!empty($user["working_days"]) && !empty($user['working_days'])) {
                            $user['working_days'] = explode(',', $user['working_days']);
                            $user['working_days_str'] = implode(', ', array_intersect_key($weekdays, array_flip($user['working_days'])));
                        }

                        if (!empty($user['working_hours_begin'])) {
                            $user['working_hours_begin_text'] = $dayhours[$user['working_hours_begin']];
                        } else {
                            $user['working_hours_begin_text'] = '';
                        }
                        if (!empty($user['working_hours_end'])) {
                            $user['working_hours_end_text'] = $dayhours[$user['working_hours_end']];
                        } else {
                            $user['working_hours_end_text'] = '';
                        }
                        if (!empty($user['working_hours_begin']) && !empty($user['working_hours_end'])) {
                            $user['working_hours_str'] = $dayhours[$user['working_hours_begin']] . ' - ' . $dayhours[$user['working_hours_end']];
                        }

                        if (!empty($user['lunch_time_begin'])) {
                            $user['lunch_time_begin_text'] = $dayhours[$user['lunch_time_begin']];
                        } else {
                            $user['lunch_time_begin_text'] = '';
                        }
                        if (!empty($user['lunch_time_end'])) {
                            $user['lunch_time_end_text'] = $dayhours[$user['lunch_time_end']];
                        } else {
                            $user['lunch_time_end_text'] = '';
                        }
                        if (!empty($user['lunch_time_begin']) && !empty($user['lunch_time_end'])) {
                            $user['lunch_time_str'] = $dayhours[$user['lunch_time_begin']] . ' - ' . $dayhours[$user['lunch_time_end']];
                        }

                        $user['is_map'] = !((float) $user['lat'] == 0 && (float) $user['lon'] == 0);

                        $user_for_location[$user['id']] = array(
                            'country' => $user['id_country'],
                            'region' => $user['id_region'],
                            'city' => $user['id_city'],
                            'district' => $user['id_district'],
                            'address' => $user['address'],
                            'zip_code' => $user['postal_code'],
                        );
                        break;
                    case 'agent':
                        if ($user['agent_company']) {
                            $company_ids[] = $user['agent_company'];
                            if (!$user['agent_status']) {
                                $user['agent_status_str'] = l('agent_status_not_approved', 'users');
                            }
                        }
                        $user['is_map'] = 0;
                        break;
                }

                if ($this->format_settings['get_language']) {
                    if ($user['lang_id']) {
                        $language_ids[] = $user['lang_id'];
                    }
                }

                if ($this->format_settings['get_currency']) {
                    if ($user['id_currency']) {
                        $currency_ids[] = $user['id_currency'];
                    }
                }

                if ($this->format_settings['get_group']) {
                    $group_ids[] = $user['group_id'];
                }

                if ($this->format_settings['get_safe']) {
                    foreach ($this->safe_fields as $field_name) {
                        unset($user[$field_name]);
                    }
                }
            }

            if (isset($user['roles'])) {
                $user['roles'] = $this->rolesDecode($user['roles']);
            }

            $user['media']['default_logo'] = $this->ci->Uploads_model->format_default_upload($this->upload_config_id);
            if (!empty($user['user_logo'])) {
                $user['media']['user_logo'] = $this->ci->Uploads_model->format_upload($this->upload_config_id, $user["postfix"], $user["user_logo"]);
            } else {
                $user['media']['user_logo'] = $user['media']['default_logo'];
            }

            if (!empty($user['user_logo_moderation'])) {
                $user['media']['user_logo_moderation'] = $this->ci->Uploads_model->format_upload($this->upload_config_id, $user["postfix"], $user["user_logo_moderation"]);
            } else {
                $user['media']['user_logo_moderation'] = $this->ci->Uploads_model->format_default_upload($this->upload_config_id);
            }

            if (!empty($user['status'])) {
                $user['status_output'] = l('active_user', 'users');
            } else {
                $user['status_output'] = l('inactive_user', 'users');
            }

            if (isset($user['review_sorter'])) {
                $user['review_sorter'] = round($user['review_sorter'], 1);
            }

            if (isset($user['review_value'])) {
                $user['review_value'] = round($user['review_value'], 1);
            }

            $users[$key] = $user;
        }

        if (!empty($user_for_location)) {
            $this->ci->load->helper('countries');

            $user_locations = address_output_format($user_for_location);
            $user_locations_data = get_location_data($user_for_location);
            $lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);

            foreach ($users as $key => $user) {
                $users[$key]['country'] = (isset($user_locations_data["country"][$user["id_country"]])) ? $user_locations_data["country"][$user["id_country"]]["name"] : "";
                $users[$key]['region'] = (isset($user_locations_data["region"][$user["id_region"]])) ? $user_locations_data["region"][$user["id_region"]]["name"] : "";
                $users[$key]['region_code'] = (isset($user_locations_data["region"][$user["id_region"]])) ? $user_locations_data["region"][$user["id_region"]]["code"] : "";
                $users[$key]['city'] = (isset($user_locations_data["city"][$user["id_city"]])) ? $user_locations_data["city"][$user["id_city"]]["name"] : "";
                $users[$key]['district'] = (isset($user_locations_data["district"][$user["id_district"]])) ? $user_locations_data["district"][$user["id_district"]]["name"] : "";
                $users[$key]['location'] = (isset($user_locations[$user["id"]])) ? $user_locations[$user["id"]] : '';
            }
        }

        // get field_editor
        if ($this->format_settings['get_field_editor'] && !empty($users_types_ids)) {
            $field_editor_data = array();
            foreach ($users_types_ids as $user_type_gid => $user_type_data) {
                $fields_for_select = $this->get_fields_for_select($user_type_gid);
                if (empty($fields_for_select)) {
                    continue;
                }
                $field_editor_data = $this->_get_field_editor_data($user_type_gid, $fields_for_select, $user_type_data);
                foreach ($users as $key => $user) {
                    if (!isset($field_editor_data[$user['id']])) {
                        continue;
                    }
                    foreach ($field_editor_data[$user['id']] as $name => $value) {
                        if (isset($user[$name])) {
                            continue;
                        }
                        $user[$name] = $value;
                        $users[$key] = $user;
                    }
                }
            }

            $this->ci->load->model('Field_editor_model');
            $params = array('where_in' => array('editor_type_gid' => array_keys($users_types_ids)));
            $fields_data = $this->ci->Field_editor_model->format_list_fields_for_view($params, $users);
            foreach ($users as $key => $user) {
                if (!isset($fields_data[$key])) {
                    continue;
                }
                $users[$key]['field_editor'] = array();
                foreach ($this->field_editor_sections['user'] as $field_editor_section) {
                    foreach ($fields_data[$key] as $field_gid => $field_data) {
                        if ($field_data['section_gid'] != $field_editor_section) {
                            continue;
                        }
                        $users[$key]['field_editor'][$field_gid] = $field_data;
                    }
                }
                foreach ($this->field_editor_sections[$user['user_type']] as $field_editor_section) {
                    foreach ($fields_data[$key] as $field_gid => $field_data) {
                        if ($field_data['section_gid'] != $field_editor_section) {
                            continue;
                        }
                        $users[$key]['field_editor'][$field_gid] = $field_data;
                    }
                }
            }
        }

        if ($this->format_settings['get_contact']) {
            if ($this->ci->session->userdata('auth_type') == 'user') {
                $user_id = $this->ci->session->userdata('user_id');
                $user_type = $this->ci->session->userdata('user_type');
                $is_show_contact = 1;
            } else {
                $user_id = 0;
                $user_type = null;
                $is_show_contact = !$this->ci->pg_module->get_module_config("listings", "is_show_contact");
            }

            if ($this->ci->pg_module->is_module_active('users_services')) {
                $this->ci->load->model('Users_services_model');
                $is_contact_services_active = $this->ci->Users_services_model->is_active_contact_services($user_type);
            } else {
                $is_contact_services_active = false;
            }

            $this->ci->load->helper('users');
            $is_login_enabled = is_login_enabled();

            foreach ($users as $key => $user) {
                $user['no_access_contact'] = !$is_login_enabled && $is_contact_services_active;
                if ($user['no_access_contact']) {
                    foreach ($this->contact_fields as $field_name) {
                        unset($user[$field_name]);
                    }
                    if (!isset($user['field_editor'])) {
                        unset($user['field_editor']);
                    }
                    $user['is_contact'] = false;
                } elseif ($is_contact_services_active) {
                    if ($user_id != $user['id']) {
                        $user['is_contact'] = $this->ci->Users_services_model->is_available_contact($user_id, $user['id']);
                        if (!$user['is_contact']) {
                            foreach ($this->contact_fields as $field_name) {
                                if (!empty($user[$field_name])) {
                                    $user[$field_name] = l('please_buy', 'users');
                                }
                            }
                            if (!empty($user['field_editor'])) {
                                foreach ($user['field_editor'] as $field_gid => $field_data) {
                                    if (!empty($user['field_editor'][$field_gid]['value_str'])) {
                                        $user['field_editor'][$field_gid]['value_str'] = l('please_buy', 'users');
                                    }
                                    if (!empty($user['field_editor'][$field_gid]['value'])) {
                                        $user['field_editor'][$field_gid]['value'] = l('please_buy', 'users');
                                    }
                                    if (!empty($user['field_editor'][$field_gid]['value_original'])) {
                                        $user['field_editor'][$field_gid]['value_original'] = l('please_buy', 'users');
                                    }
                                }
                            }
                        }
                        $user['is_company'] = $this->ci->Users_services_model->is_available_contact($user_id, $user['agent_company']);
                    } else {
                        $user['is_contact'] = true;
                    }
                } else {
                    $user['is_contact'] = true;
                }

                $user['is_show_contact'] = $is_show_contact;

                $users[$key] = $user;
            }
        }

        if ($this->format_settings['get_group'] && !empty($group_ids)) {
            $groups_by_id = array();
            $this->ci->load->model('users/models/Users_groups_model');
            $groups = $this->ci->Users_groups_model->get_groups_list(null, null, null, array('where_in' => array('id' => $group_ids)));
            foreach ($groups as $group) {
                $groups_by_id[$group['id']] = $group;
            }
            foreach ($users as $key => $user) {
                $user['group_output'] = isset($groups_by_id[$user['group_id']]) ? $groups_by_id[$user['group_id']]['group_name'] : '';
                $users[$key] = $user;
            }
        }

        if ($this->format_settings['get_company'] && !empty($company_ids)) {
            $companies = $this->get_users_list_by_key(null, null, null, array(), $company_ids);
            foreach ($users as $key => $user) {
                if ($user['user_type'] != 'agent' || !$user['agent_company']) {
                    continue;
                }
                $user['company'] = isset($companies[$user['agent_company']]) ? $companies[$user['agent_company']] : $this->format_default_user($user['agent_company']);
                $users[$key] = $user;
            }
        }

        if ($this->format_settings['get_language'] && !empty($language_ids)) {
            $languages = $this->ci->pg_language->return_langs();
            foreach ($users as $key => $user) {
                if (isset($languages[$user['lang_id']])) {
                    $user['language'] = $languages[$user['lang_id']];
                    $user['language_output'] = $user['language']['name'];
                } else {
                    $user['language_output'] = '';
                }
                $users[$key] = $user;
            }
        }

        if ($this->format_settings['get_currency'] && !empty($currency_ids)) {
            $this->ci->load->model('Currencies_model');
            $currencies = $this->ci->Currencies_model->return_currencies();
            foreach ($users as $key => $user) {
                if (isset($currencies[$user['id_currency']])) {
                    $user['currency'] = $currencies[$user['id_currency']];
                    $user['currency_output'] = $user['currency']['name'];
                } else {
                    $user['currency_output'] = '';
                }
                $users[$key] = $user;
            }
        }

        // all views
        if (!empty($this->format_settings['get_views'])) {
            $this->ci->load->model('users/models/Users_visitor_model');
            $users_views = $this->ci->Users_visitor_model->get_views_count($users_ids);
            foreach ($users as $key => $user) {
                $user['views'] += isset($users_views[$user['id']]) ? $users_views[$user['id']] : 0;
                $users[$key] = $user;
            }
        }

        return $users;
    }

    /**
     * Format user object by default
     *
     * @param integer $id user identifier
     *
     * @return array
     */
    public function format_default_user($id)
    {
        $this->ci->load->model("Uploads_model");
        $data["postfix"] = $id;
        $data["output_name"] = "User is deleted";
        $data["media"]["user_logo"] = $this->ci->Uploads_model->format_default_upload($this->upload_config_id);
        $data["media"]["default_logo"] = $this->ci->Uploads_model->format_default_upload($this->upload_config_id);

        return $data;
    }

    /**
     * Format user object
     *
     * @param array $data user data
     *
     * @return array
     */
    public function format_user($data)
    {
        $return = $this->format_users(array(0 => $data));

        return $return[0];
    }

    /**
     * Return settings for rewriting seo url and metatags
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function get_seo_settings($method = "", $lang_id = "")
    {
        if (!empty($method)) {
            return $this->_get_seo_settings($method, $lang_id);
        } else {
            $actions = array("account", "login_form", "profile", "view", "restore", 'index', 'search');
            foreach ($this->getUserTypes() as $user_type) {
                $actions[] = "reg_" . $user_type;
            }
            $return = array();
            foreach ($actions as $action) {
                $return[$action] = $this->_get_seo_settings($action, $lang_id);
            }

            return $return;
        }
    }

    /**
     * Return settings for rewriting seo url and metatags (internal)
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    protected function _get_seo_settings($method, $lang_id = "")
    {
        switch ($method) {
            case "account":
                return array(
                    "templates" => array("unique_name", "name", "fname", "sname", "user_type", "user_type_str", "contact_email", "contact_phone"),
                    "url_vars" => array(),
                );
                break;
            case "login_form":
                return array(
                    "templates" => array(),
                    "url_vars" => array(),
                );
                break;
            case "restore":
                return array(
                    "templates" => array(),
                    "url_vars" => array(),
                );
                break;
            case "profile":
                return array(
                    "templates" => array("unique_name", "name", "fname", "sname", "user_type", "user_type_str", "contact_email", "contact_phone"),
                    "url_vars" => array(
                        "action" => array("action" => "literal"),
                    ),
                );
                break;
            case "index":
                return array(
                    "templates" => array('user_type', 'user_type_str'),
                    "url_vars" => array(
                        "user_type" => array("user_type" => "literal"),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case "search":
                return array(
                    "templates" => array('user_type', 'user_type_str'),
                    "url_vars" => array(
                        "user_type" => array("user_type" => "literal"),
                        "keyword" => array("keyword" => "literal"),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case "view":
                return array(
                    "templates" => array("name", "fname", "sname", "user_type", "user_type_str", "unique_name", "contact_email", "contact_phone", "location"),
                    "url_vars" => array(
                        "id" => array("id" => "numeric"),
                        "section" => array("section" => "literal"),
                    ),
                    "optional" => array(
                        "user_type" => array("user_type" => "literal"),
                        "name" => array("name" => "literal"),
                    ),
                );
                break;
            default:
                foreach ($this->getUserTypes() as $user_type) {
                    if ($method == "reg_" . $user_type) {
                        return array(
                            "templates" => array(),
                            "url_vars" => array(),
                        );
                        break;
                    }
                }
                break;
        }
    }

    /**
     * Transform seo values from query string to module variables
     *
     * @param string $var_name_from variable name in url
     * @param string $var_name_to   variable name in method
     * @param mixed  $value         variable value
     *
     * @return mixed
     */
    public function request_seo_rewrite($var_name_from, $var_name_to, $value)
    {
        $user_data = array();

        if ($var_name_from == $var_name_to) {
            switch ($var_name_to) {
                case "name":
                case "type":
                    return '';
                    break;
                default:
                    return $value;
                    break;
            }
        }

        if ($var_name_from == "login") {
            $user_data = $this->get_user_by_login($value);
        }

        if ($var_name_to == "id") {
            return $user_data["id"];
        }
    }

    /**
     * Return urls of pages for xml sitemap
     *
     * @return array
     */
    public function get_sitemap_xml_urls()
    {
        $this->ci->load->helper("seo");

        $lang_canonical = true;

        if ($this->ci->pg_module->is_module_active('seo')) {
            $lang_canonical = $this->ci->pg_module->get_module_config('seo', 'lang_canonical');
        }

        if ($lang_canonical) {
            $default_lang_id = $this->ci->pg_language->get_default_lang_id();
            $default_lang_code = $this->ci->pg_language->get_lang_code_by_id($default_lang_id);
            $langs[$default_lang_id] = $default_lang_code;
        } else {
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $langs[$lang_id] = $lang_data['code'];
            }
        }

        $return = array();

        $this->ci->load->helper("users");

        $is_login_enabled = is_login_enabled();
        if ($is_login_enabled) {
            $user_settings = $this->pg_seo->get_settings('user', 'users', 'login_form');
            if (!$user_settings['noindex']) {
                $this->ci->load->helper('users');
                if (is_login_enabled()) {
                    foreach ($langs as $lang_id => $lang_code) {
                        $return[] = array(
                            "url" => rewrite_link('users', 'login_form', array(), false, $lang_code, $lang_canonical),
                            "priority" => 0.3,
                        );
                    }
                }
            }

            $user_settings = $this->pg_seo->get_settings('user', 'users', 'restore');
            if (!$user_settings['noindex']) {
                foreach ($langs as $lang_id => $lang_code) {
                    $return[] = array(
                        "url" => rewrite_link('users', 'restore', array(), false, $lang_code, $lang_canonical),
                        "priority" => 0.3,
                    );
                }
            }

            $user_types = $this->getUserTypes();
            foreach ($user_types as $user_type) {
                $user_settings = $this->pg_seo->get_settings('user', 'users', 'reg_' . $user_type);
                if (!$user_settings['noindex']) {
                    $register_enabled = $this->ci->pg_module->get_module_config("users", $user_type . "_register_enabled");
                    if (!$register_enabled) {
                        continue;
                    }
                    foreach ($langs as $lang_id => $lang_code) {
                        $return[] = array(
                            "url" => rewrite_link('users', 'reg_' . $user_type, array(), false, $lang_code, $lang_canonical),
                            "priority" => 0.3,
                        );
                    }
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'users', 'index');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($user_types as $user_type) {
                    $return[] = array(
                        "url" => rewrite_link('users', 'index', array('user_type' => $user_type), false, $lang_code, $lang_canonical),
                        "priority" => 0.5,
                    );
                }
            }
        }

        /*$user_settings = $this->pg_seo->get_settings('user', 'users', 'view');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($user_types as $user_type) {
                    $order_array = array('id' => 'DESC');
                    $params = array('user_type' => $user_type);
                    $users = $this->get_active_users_list(1, 1000, $order_array, $params);
                    if (!empty($users)) {
                        foreach ($users as $user) {
                            $return[] = array(
                                "url" => rewrite_link("users", "view", $user, false, $lang_code, $lang_canonical),
                                "priority" => 0.6,
                            );
                        }
                    }
                }
            }
        }*/

        return $return;
    }

    /**
     * Return urls of pages for site map
     *
     * @return array
     */
    public function get_sitemap_urls()
    {
        $this->ci->load->helper("seo");
        $this->ci->load->helper("users");
        $auth = $this->ci->session->userdata("auth_type");
        $block = array();

        $items = array();

        $login_enabled = is_login_enabled();
        if ($login_enabled) {
            $items[] = array(
                "name" => l("header_login", "users"),
                "link" => rewrite_link("users", "login_form"),
                "clickable" => ($auth == "user") ? false : true,
            );

            foreach ($this->getUserTypes() as $user_type) {
                $register_enabled = $this->ci->pg_module->get_module_config("users", $user_type . "_register_enabled");
                if (!$register_enabled) {
                    continue;
                }
                $items[] = array(
                    "name" => ld_option('user_type', 'users', $user_type) . ' ' . l("header_register_as", "users"),
                    "link" => rewrite_link("users", "reg_" . $user_type),
                    "clickable" => ($auth == "user") ? false : true,
                );
            }

            if ($this->ci->pg_module->is_module_active("users_payments")) {
                $items[] = array(
                    "name" => l("link_edit_account", "users"),
                    "link" => rewrite_link("users", "account"),
                    "clickable" => ($auth == "user") ? true : false,
                );
            }

            if ($auth == "user") {
                $items[] = array(
                    "name" => l("link_edit_profile", "users"),
                    "link" => rewrite_link("users", "profile"),
                    "clickable" => ($auth == "user") ? true : false,
                );

                $items[] = array(
                    "name" => l("link_logout", "users"),
                    "link" => site_url() . "users/logout",
                    "clickable" => ($auth == "user") ? true : false,
                );
            }
        }

        $block[] = array(
            "name" => l("header_profile", "users"),
            "link" => rewrite_link("users", "profile"),
            "clickable" => ($auth == "user") ? true : false,
            "items" => $items,
        );

        $items = array();

        $professionals = ld('professionals', 'users');
        foreach ($professionals['option'] as $user_type => $name) {
            $search_enabled = $this->ci->pg_module->get_module_config("users", $user_type . "_search_enabled");
            if (!$search_enabled) {
                continue;
            }
            $items[] = array(
                "name" => $name,
                "link" => site_url() . 'users/index/' . $user_type,
                "clickable" => true,
            );
        }

        if (!empty($items)) {
            $block[] = array(
                "name" => l("header_search_users", "users"),
                "link" => site_url() . "users/index",
                "clickable" => true,
                "items" => $items,
            );
        }

        return $block;
    }

    /**
     * Return available banners places
     *
     * Callback method of banners module
     *
     * @return array
     */
    public function _banner_available_pages()
    {
        $return[] = array("link" => "users/profile", "name" => l("header_profile", "users"));
        $return[] = array("link" => "users/login_form", "name" => l("header_login", "users"));
        $return[] = array("link" => "users/reg_company", "name" => l("header_register_company", "users"));
        $return[] = array("link" => "users/reg_agent", "name" => l("header_register_agent", "users"));
        $return[] = array("link" => "users/account", "name" => l("header_edit_account", "users"));
        $return[] = array("link" => "users/index", "name" => l("header_search_users", "users"));
        $return[] = array("link" => "users/company_list", "name" => l("header_search_companies", "users"));
        $return[] = array("link" => "users/agent_list", "name" => l("header_search_agents", "users"));
        $return[] = array("link" => "users/view", "name" => l("header_view_profile", "users"));

        if ($this->ci->pg_module->is_module_active("users_private")) {
            $this->ci->load->model("Users_private_model");
            $pages = $this->ci->Users_private_model->_banner_available_pages();
            $return = array_merge($return, $pages);
        }

        return $return;
    }

    /**
     * Return content of last registered users block
     *
     * Сallback method of dynamic blocks module
     *
     * @param array   $params block parameters
     * @param string  $view   view mode
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_last_registered($params, $view = 'default', $width = 100)
    {
        if (!isset($params['with_logo'])) {
            $params['with_logo'] = 'yes';
        }
        $this->ci->view->assign('block_with_logo', $params['with_logo']);
        if (!isset($params['count'])) {
            $params['count'] = 8;
        }
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_featured_users', 'user', 'users');
    }

    /*
     * User account functions
     */

    /**
     * Return amount of user account
     *
     * @param integer $user_id user identifier
     *
     * @return float
     */
    public function get_user_account($user_id)
    {
        $data = $this->get_user_by_id($user_id);

        return isset($data["account"]) ? $data["account"] : 0;
    }

    /**
     * Update amount of user account
     *
     * @param array   $payment_data   payment data
     * @param integer $payment_status payment status
     *
     * @return void
     */
    public function update_user_account($payment_data, $payment_status)
    {
        if ($payment_status == 1) {
            $user_id = $payment_data["id_user"];
            $this->set_format_settings('get_safe', false);
            $user_data = $this->get_user_by_id($user_id);
            $this->set_format_settings('get_safe', true);
            $data["account"] = $user_data["account"] + $payment_data["amount"];
            $this->save_user($user_id, $data);
            $this->account_add_spend_entry($user_id, $payment_data["amount"], "add", l("account_funds_add_message", "users"));
            $this->ci->load->model("Notifications_model");
            $this->ci->Notifications_model->send_notification($user_data["email"], "users_update_account", $user_data, '', $user_data["lang_id"]);
        }
    }

    /**
     * Write off from account
     *
     * @param integer $id_user user identifier
     * @param float   $amount  amount value
     * @param string  $message message text
     *
     * @return string/boolean
     */
    public function write_off_user_account($id_user, $amount, $message)
    {
        $user_data = $this->get_user_by_id($id_user);
        $data["account"] = $user_data["account"] - $amount;
        if ($data["account"] < 0) {
            return l("error_money_not_sufficient", "users");
        } else {
            $this->save_user($id_user, $data);
            $this->account_add_spend_entry($id_user, $amount, "spend", $message);

            return true;
        }
    }

    /**
     * Add logging of spend transaction
     *
     * @param integer $id_user    user identifier
     * @param float   $price      price value
     * @param string  $price_type price type
     * @param string  $message    transaction message
     *
     * @return void
     */
    public function account_add_spend_entry($id_user, $price, $price_type = "add", $message = "")
    {
        if (empty($message)) {
            $message = ($price_type = "add") ? l("account_funds_add_message", "users") : l("account_funds_spend_message", "users");
        }

        $data = array(
            "id_user" => $id_user,
            "date_add" => date("Y-m-d H:i:s"),
            "price_type" => $price_type,
            "price" => $price,
            "message" => $message,
        );
        $this->ci->db->insert(USER_ACCOUNT_STAT_TABLE, $data);
    }

    /*
     * Moderation functions
     */

    /**
     * Return users objects for moderation
     *
     * @param array $object_ids users identifiers
     *
     * @return array
     */
    public function _moder_get_list($object_ids)
    {
        $params["where_in"]["id"] = $object_ids;

        $users = $this->get_users_list(null, null, null, $params);

        if (!empty($users)) {
            foreach ($users as $user) {
                if ($user['user_logo_moderation']) {
                    $user['media']['user_logo'] = $user['media']['user_logo_moderation'];
                }
                if (!isset($user['media']['user_logo']['thumbs']['small']) && !empty($user['media']['user_logo']['thumbs'])) {
                    $user['media']['user_logo']['thumbs']['small'] = array_pop($user['media']['user_logo']['thumbs']);
                }
                $return[$user["id"]] = $user;
            }

            return $return;
        } else {
            return array();
        }
    }

    /**
     * Set moderation status to user object
     *
     * @param integer $object_id user identifier
     * @param integer $status    moderation status
     *
     * @return void
     */
    public function _moder_set_status($object_id, $status)
    {
        $user = $this->get_user_by_id($object_id);
        $backup_user = array();
        switch ($status) {
            case 0:
                if ($user["user_logo_moderation"]) {
                    $backup_user["user_logo_moderation"] = "";
                } else {
                    $backup_user["user_logo"] = "";
                }
                break;
            case 1:
                if ($user["user_logo_moderation"]) {
                    $backup_user["user_logo"] = $user["user_logo_moderation"];
                    $backup_user["user_logo_moderation"] = "";
                }
                break;
        }
        $this->save_user($object_id, $backup_user);
    }

    /**
     * Return user type
     *
     * @param integer $user_id user identifier
     *
     * @return string
     */
    public function get_user_type_by_id($user_id)
    {
        $this->set_format_settings('use_format', false);
        $data = $this->get_user_by_id($user_id);
        $this->set_format_settings('use_format', true);
        if (isset($data["user_type"])) {
            return $data["user_type"];
        } else {
            return;
        }
    }

    /*
     * Contacts functions
     */

    /**
     * Add contact link to user object
     *
     * @param integer $id_user    user identifier
     * @param integer $id_contact contact identifier
     *
     * @return void
     */
    public function add_contact($id_user, $id_contact)
    {
        $this->ci->load->model("Linker_model");
        $this->ci->Linker_model->add_link("users_contacts", $id_user, $id_contact);
        $this->ci->Linker_model->add_link("users_contacts", $id_contact, $id_user);

        return;
    }

    /**
     * Remove contact link from user object
     *
     * @param integer $id_user    user identifier
     * @param integer $id_contact contact identifier
     *
     * @return void
     */
    public function delete_contact($id_user, $id_contact)
    {
        $this->ci->load->model("Linker_model");
        $params["where_in"]["id_link_2"] = $params["where_in"]["id_link_1"] = array($id_user, $id_contact);
        $this->ci->Linker_model->delete_links("users_contacts", $params);

        return;
    }

    /**
     * Remove all contacts from user object
     *
     * @param integer $id_user user identifier
     *
     * @return void
     */
    public function delete_user_contacts($id_user)
    {
        $this->ci->load->model("Linker_model");
        $params["where"]["id_link_2"] = $params["where"]["id_link_1"] = $id_user;
        $this->ci->Linker_model->delete_links("users_contacts", $params);
    }

    /**
     * Return display sections of user profile
     *
     * @param array $data user data
     *
     * @return array
     */
    public function get_display_sections($data)
    {
        if (!$data['is_map'] || !$this->ci->pg_module->is_module_active("geomap")) {
            $this->display_sections["map"] = 0;
        }
        if (!$data["review_count"]) {
            $this->display_sections["reviews"] = 0;
        }

        return $this->display_sections;
    }

    /**
     * Build params for agents query
     * @param int $company_id
     * @param bool $agent_status
     * @param string $name
     * @param bool $status
     * @return array
     */
    private function getAgentsParams($company_id, $agent_status = 1, $name = null, $status = null)
    {
        $params = array(
            'where' => array(
                'agent_company' => $company_id,
                'agent_status' => $agent_status,
            ),
        );
        if (!is_null($status)) {
            $params['where']['status'] = (int) (bool) $status;
        }
        if ($name != null) {
            $name_parts = explode(' ', $name);
            foreach ($name_parts as $name_part) {
                $params['where_sql'][] = "(`fname` LIKE '%$name_part%' OR `sname` LIKE '%$name_part%')";
            }
        }
        return $params;
    }

    /**
     * Return agents objects by company from data source as array
     *
     * @param integer $company_id    company identifier
     * @param integer $agent_status  status of agent request
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param array   $order_by      sorting data
     * @param boolean $formatted     format results
     * @param string  $name          agent name
     * @param integer $status        activity status
     *
     * @return array
     */
    public function getAgentsList($company_id, $agent_status = 1, $page = 1, $items_on_page = null, $order_by = array(), $formatted = true, $name = null, $status = null)
    {
        $params = $this->getAgentsParams($company_id, $agent_status, $name, $status);
        return $this->get_users_list($page, $items_on_page, $order_by, $params, array(), $formatted);
    }

    /**
     * Return number of user agents in data source
     *
     * @param integer $company_id   company identifier
     * @param integer $agent_status status of agent request
     * @param string  $name         agent name
     * @param integer $status       activity status
     *
     * @return integer
     */
    public function getAgentsCount($company_id, $agent_status = 1, $name = null, $status = null)
    {
        $params = $this->getAgentsParams($company_id, $agent_status, $name, $status);
        $count = $this->get_users_count($params);
        return $count;
    }

    /**
     * Get company agents ids
     * @param int $company_id
     * @return array
     */
    public function getAgentsListIds($company_id)
    {
        $agents = $this->getAgentsList($company_id, 1, 1, null, [], false, null, 0);
        $ids = [];
        foreach ($agents as $agent) {
            $ids[] = $agent['id'];
        }
        return $ids;
    }

    /**
     * Approve agent request
     *
     * @param integer $company_id company identifier
     * @param integer $agent_id   agent identifier
     *
     * @return void
     */
    public function approve_agent($company_id, $agent_id)
    {
        $this->set_format_settings('get_safe', false);
        $company = $this->get_user_by_id($company_id);
        $agent = $this->get_user_by_id($agent_id);
        $this->set_format_settings('get_safe', true);

        $set["agent_status"] = 1;
        $this->save_user($agent_id, $set);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_agent_item', $agent_id, true);
        $this->ci->Indicators_model->delete('all_agent_item', 'request_' . $agent_id, true);
        $this->ci->Indicators_model->add('agent_status_item', $agent_id, $agent_id);
        $this->ci->Indicators_model->add('all_profile_item', 'agent_status_' . $agent_id, $agent_id);

        $data['fname'] = $agent['fname'];
        $data['sname'] = $agent['sname'];
        $data["company_name"] = $company["output_name"];
        $data["user_name"] = $agent["output_name"];

        $this->update_agent_company_count($company_id);

        $this->ci->load->model("Linker_model");
        $this->ci->Linker_model->add_link("users_contacts", $agent_id, $company_id);
        $this->ci->Linker_model->add_link("users_contacts", $company_id, $agent_id);

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($agent["email"], "agent_request_accepted", $data, '', $agent["lang_id"]);
    }

    /**
     * Decline agent request
     *
     * @param integer $company_id company identifier
     * @param integer $agent_id   agent identifier
     *
     * @return void
     */
    public function decline_agent($company_id, $agent_id)
    {
        $this->set_format_settings('get_safe', false);
        $company = $this->get_user_by_id($company_id);
        $agent = $this->get_user_by_id($agent_id);
        $this->set_format_settings('get_safe', true);

        $set["agent_company"] = 0;
        $this->save_user($agent_id, $set);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_agent_item', $agent_id, true);
        $this->ci->Indicators_model->delete('all_agent_item', 'request_' . $agent_id, true);
        $this->ci->Indicators_model->add('agent_status_item', $agent_id, $agent_id);
        $this->ci->Indicators_model->add('all_profile_item', 'agent_status_' . $agent_id, $agent_id);

        $data['fname'] = $agent['fname'];
        $data['sname'] = $agent['sname'];
        $data["company_name"] = $company["output_name"];
        $data["user_name"] = $agent["output_name"];

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($agent["email"], "agent_request_declined", $data, '', $agent["lang_id"]);
    }

    /**
     * Remove agent from company
     *
     * @param integer $company_id company identifier
     * @param integer $agent_id   agent identifier
     * @param boolean $by_agent   initialize by agent
     *
     * @return void
     */
    public function delete_agent($company_id, $agent_id, $by_agent)
    {
        $this->set_format_settings('get_safe', false);
        $company = $this->get_user_by_id($company_id);
        $agent = $this->get_user_by_id($agent_id);
        $this->set_format_settings('get_safe', true);

        $set["agent_company"] = 0;
        $set["agent_status"] = 0;
        $this->save_user($agent_id, $set);

        $mail_data["company_name"] = $company["output_name"];
        $mail_data["user_name"] = $agent["output_name"];

        $this->ci->load->model("Notifications_model");

        if ($by_agent) {
            if ($agent['agent_status']) {
                $company_template = 'agent_unjoin';
            } else {
                $company_template = 'agent_cancel_request';
            }
            $mail_data["fname"] = $company["fname"];
            $mail_data["sname"] = $company["sname"];
            $this->ci->Notifications_model->send_notification($company["email"], $company_template, $mail_data, '', $company["lang_id"]);
        } else {
            if ($agent['agent_status']) {
                $company_template = 'agent_unjoin';
                $agent_template = 'agent_deleted';

                $this->update_agent_company_count($company_id);
            } else {
                $company_template = 'agent_cancel_request';
                $agent_template = 'agent_request_declined';
            }

            $mail_data["fname"] = $company["fname"];
            $mail_data["sname"] = $company["sname"];
            $this->ci->Notifications_model->send_notification($company["email"], $company_template, $mail_data, '', $company["lang_id"]);

            //$mail_data["fname"] = $agent["fname"];
            //$mail_data["sname"] = $agent["sname"];
            //$this->CI->Notifications_model->send_notification($agent["email"], $agent_template, $mail_data, '', $agent["lang_id"]);
        }

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_agent_item', $agent_id, true);
        $this->ci->Indicators_model->delete('all_agent_item', 'request_' . $agent_id, true);
        $this->ci->Indicators_model->add('agent_status_item', $agent_id, $agent_id);
        $this->ci->Indicators_model->add('all_profile_item', 'agent_status_' . $agent_id, $agent_id);
    }

    /**
     * Clear end date of user service
     *
     * @param string $field        field name of user service end date
     * @param string $service_name service name
     *
     * @return integer
     */
    public function clear_end_date($field, $service_name)
    {
        $this->ci->db->select(implode(", ", $this->fields_general))
            ->from(USERS_TABLE)
            ->where($field . ' <', date('Y-m-d H:i:s'))
            ->where('UNIX_TIMESTAMP(' . $field . ') >', '86400');
        $results = $this->ci->db->get()->result_array();
        $clean = 0;
        if (!empty($results) && is_array($results)) {
            $clean = count($results);
            if ($clean > 0) {
                $data['is_' . substr($field, 0, -9)] = '0';
                $data[$field] = "0000-00-00 00:00:00";
                $this->ci->db->where($field . " <", date("Y-m-d H:i:s"))
                    ->where("UNIX_TIMESTAMP(" . $field . ") >", "86400")
                    ->update(USERS_TABLE, $data);

                // send notification
                $this->ci->load->model("Notifications_model");
                $this->set_format_settings('get_safe', false);
                $results = $this->format_users($results);
                $this->set_format_settings('get_safe', true);
                foreach ($results as $user_data) {
                    $mail_data = array(
                        'user' => $user_data['output_name'],
                        'name' => l($service_name, 'users_services', $user_data['lang_id']),
                    );
                    $this->ci->Notifications_model->send_notification($user_data['email'], "user_service_expired", $mail_data, '', $user_data['lang_id']);
                }
            }
        }

        return $clean;
    }

    /**
     * Install user fields of reviews
     *
     * @param array $fields fields data
     *
     * @return void
     */
    public function install_reviews_fields($fields = array())
    {
        if (empty($fields)) {
            return;
        }
        $this->ci->load->dbforge();
        $table_fields = $this->ci->db->list_fields(USERS_TABLE);
        foreach ((array) $fields as $field_name => $field_data) {
            if (!in_array($field_name, $table_fields)) {
                $this->ci->dbforge->add_column(USERS_TABLE, array($field_name => $field_data));
            }
        }
    }

    /**
     * Uninstall fields of reviews
     *
     * @param array $fields fields data
     *
     * @return void
     */
    public function deinstall_reviews_fields($fields = array())
    {
        if (empty($fields)) {
            return;
        }
        $this->ci->load->dbforge();
        $table_fields = $this->ci->db->list_fields(USERS_TABLE);
        foreach ($fields as $field_name) {
            if (in_array($field_name, $table_fields)) {
                $this->ci->dbforge->drop_column(USERS_TABLE, $field_name);
            }
        }
    }

    /**
     * Process event of reviews module
     *
     * @param string $action action name
     * @param array  $data   reviews data
     *
     * @return mixed
     */
    public function callback_reviews($action, $data)
    {
        switch ($action) {
            case 'update':
                $user_data['review_type'] = $data['type_gid'];
                $user_data['review_sorter'] = $data['review_sorter'];
                $user_data['review_value'] = $data['review_value'];
                $user_data['review_count'] = $data['review_count'];
                $user_data['review_data'] = serialize($data['review_data']);
                $this->save_user($data['id_object'], $user_data);
                break;
            case 'get_object':
                if (empty($data)) {
                    return array();
                }
                $users = $this->get_users_list_by_key(null, null, null, null, (array) $data);

                return $users;
                break;
        }
    }

    /**
     * Proccess events of spam module
     *
     * @param string  $action action name
     * @param integer $data   spam data
     *
     * @return mixed
     */
    public function spam_callback($action, $data)
    {
        switch ($action) {
            case "ban":
                $this->save_user((int) $data, array("banned" => 1));

                return "banned";

                //$this->delete_user((int)$data);
                //return "removed";
                break;
            case "unban":
                $this->save_user((int) $data, array("banned" => 0));

                return "unbanned";
                break;
            case "delete":
                $this->delete_user((int) $data);

                return "removed";
                break;
            case 'get_content':
                if (empty($data)) {
                    return array();
                }
                $users = $this->get_users_list(null, null, null, null, (array) $data);
                $return = array();
                foreach ($users as $user) {
                    $return[$user['id']] = $user['output_name'] . ', ' . l($user['user_type'], 'users');
                }

                return $return;
                break;
            case 'get_link':
                if (empty($data)) {
                    return array();
                }
                $users = $this->get_users_list(null, null, null, null, (array) $data);
                $return = array();
                foreach ($users as $user) {
                    $return[$user['id']] = site_url() . 'admin/users/edit/' . $user['id'];
                }

                return $return;
                break;
            case 'get_object':
                if (empty($data)) {
                    return array();
                }
                $users = $this->get_users_list_by_key(null, null, null, null, (array) $data);

                return $users;
                break;
        }
    }

    /**
     * Install properties from listings module
     *
     * @param array $fields fields data
     *
     * @return void
     */
    public function install_listings_fields($fields = array())
    {
        if (empty($fields)) {
            return;
        }
        $this->ci->load->dbforge();
        $table_fields = $this->ci->db->list_fields(USERS_TABLE);
        foreach ((array) $fields as $field_name => $field_data) {
            if (!in_array($field_name, $table_fields)) {
                $this->ci->dbforge->add_column(USERS_TABLE, array($field_name => $field_data));
            }
        }
    }

    /**
     * Uninstall properties from listings module
     *
     * @param array $fields fields data
     *
     * @return void
     */
    public function deinstall_listings_fields($fields = array())
    {
        if (empty($fields)) {
            return;
        }
        $this->ci->load->dbforge();
        $table_fields = $this->ci->db->list_fields(USERS_TABLE);
        foreach ($fields as $field_name) {
            if (in_array($field_name, $table_fields)) {
                $this->ci->dbforge->drop_column(USERS_TABLE, $field_name);
            }
        }
    }

    /**
     * Render content of pdf version
     *
     * @param string $content content of user profile
     * @param array  $data    user data
     *
     * @return void
     */
    public function get_pdf_file($content, $data)
    {
        $content = preg_replace("/<script(.*?)<\/script>/is", "", $content);
        $content = str_replace('<meta charset="utf-8">', '<meta http-equiv="content-type" content="text/html; charset=utf-8" />', $content);

        $current_lang_id = $this->ci->pg_language->current_lang_id;
        $current_lang = $this->ci->pg_language->get_lang_by_id($current_lang_id);

        if ($current_lang['code'] == 'ma' || $current_lang['code'] == 'zh') {
            $content = str_replace('</head>', '<style>@font-face {font-family:\'Noto Sans\';font-style: normal;font-weight: 400;src: url(' . site_url() . 'application/libraries/dompdf/lib/fonts/fireflysung.ttf) format(\'truetype\');}*{font-family: Noto Sans !important;}</style></head>', $content);
        }

        $content = str_replace("&nbsp;", " ", $content);

        $this->ci->load->library("dompdf");

        $this->ci->dompdf->set_paper("a4", "portrait");
        $this->ci->dompdf->load_html($content);
        $this->ci->dompdf->render();
        $file_name = $data["unique_name"] . ".pdf";
        $this->ci->dompdf->stream($file_name);
    }

    /**
     * Return profile completeness in percent
     *
     * @param array $user user data
     *
     * @return integer
     */
    public function get_profile_complete($user)
    {
        $complete_count = 0;
        $params_count = 3;

        if (!empty($user['fname']) != '') {
            $complete_count++;
        }
        if (!empty($user['sname']) != '') {
            $complete_count++;
        }
        if (!empty($user['user_logo']) != '') {
            $complete_count++;
        }

        switch (get($user['user_type'])) {
            case 'company':
                if (!empty($user['company_name'])) {
                    $complete_count++;
                }
                if (!empty($user['contact_email']) != '') {
                    $complete_count++;
                }
                if (!empty($user['contact_phone']) != '') {
                    $complete_count++;
                }
                if (!empty($user['id_country'])) {
                    $complete_count++;
                }
                if (!empty($user['id_region'])) {
                    $complete_count++;
                }
                if (!empty($user['id_city'])) {
                    $complete_count++;
                }
                if (!empty($user['address'])) {
                    $complete_count++;
                }
                if (!empty($user['postal_code'])) {
                    $complete_count++;
                }
                $params_count += 8;
                break;

            case 'agent':
                if (!empty($user['agent_company'])) {
                    $complete_count++;
                }
                $params_count += 1;
                break;
        }

        return ceil($complete_count * 100 / $params_count);
    }

    /**
     * Update statistics of user listings
     *
     * @param integer $user_id user identifier
     * @param string  $type    operation type
     * @param boolean $inc     icremental/decremental
     *
     * @return void
     */
    public function update_listings_stat($user_id, $type, $inc = true)
    {
        $name = 'listings_for_' . $type . '_count';
        if (!in_array($name, $this->fields_listings)) {
            return;
        }
        $this->ci->db->set($name, $name . " " . ($inc ? "+" : "-") . " 1", false);
        $this->ci->db->where("id", $user_id);
        $this->ci->db->update(USERS_TABLE);
    }

    /**
     * Return data for build seo link of search page
     *
     * @return array
     */
    public function get_search_link_data()
    {
        $current_settings = isset($_SESSION['users_list']) ? $_SESSION['users_list'] : array();

        $user_types = $this->getUserTypes();
        if (isset($current_settings['data']['type']) &&
            in_array($current_settings['data']['type'], $user_types)) {
            $user_type = $current_settings['data']['type'];
        } else {
            $user_type = current($user_types);
        }

        $keyword = '';
        if (isset($current_settings['data']['keyword'])) {
            $keyword = $current_settings['data']['keyword'];
        }

        $order = 'default';
        $order_direction = 'DESC';
        $page = 1;

        if (!empty($current_settings)) {
            $order = $current_settings['order'];
            $order_direction = $current_settings['order_direction'];
            $page = $current_settings['page'];
        }

        $action_data = array(
            'user_type' => $user_type,
            'keyword' => $keyword,
            'order' => $order,
            'order_direction' => $order_direction,
            'page' => $page,
        );

        return $action_data;
    }

    /**
     * Change settings for formatting user object
     *
     * @param string $name  parameter name
     * @param mixed  $value parameter value
     *
     * @return void
     */
    public function set_format_settings($name, $value = false)
    {
        if (!is_array($name)) {
            $name = array($name => $value);
        }
        foreach ($name as $key => $item) {
            $this->format_settings[$key] = $item;
        }
    }

    /**
     * Save user logo by local file
     *
     * @param integer $user_id             user identifier
     * @param string  $file_name           local file name
     * @param boolean $use_icon_moderation use moderation of logo upload
     *
     * @return array
     */
    public function save_local_user_logo($user_id, $file_name, $use_icon_moderation = true)
    {
        $this->ci->load->model('Uploads_model');
        $img_return = $this->ci->Uploads_model->upload_exist($this->upload_config_id, $user_id . "/", $file_name);

        if (empty($img_return["errors"])) {
            if ($use_icon_moderation) {
                if ($this->ci->pg_module->is_module_active("moderation")) {
                    $this->ci->load->model("Moderation_model");
                    $mstatus = intval($this->ci->Moderation_model->get_moderation_type_status($this->moderation_type));
                } else {
                    $mstatus = 2;
                }
            } else {
                $mstatus = 2;
            }
            if ($mstatus != 2) {
                $this->ci->Moderation_model->add_moderation_item($this->moderation_type, $user_id);
                if ($mstatus == 1) {
                    $img_data["user_logo"] = $img_return["file"];
                    $img_data["user_logo_moderation"] = "";
                } else {
                    $img_data["user_logo_moderation"] = $img_return["file"];
                }
            } else {
                $img_data["user_logo"] = $img_return["file"];
            }
            $this->save_user($user_id, $img_data);
        }

        return $img_return;
    }

    /**
     * Return properties data from field editor
     *
     * @param string $user_type_gid field editor type GUID
     *
     * @return array
     */
    public function get_fields_for_select($user_type_gid)
    {
        if (!isset($this->field_editor_fields[$user_type_gid])) {
            $this->ci->load->model('Field_editor_model');
            $this->ci->Field_editor_model->initialize($user_type_gid);
            $user_type_sections = array_keys($this->ci->Field_editor_model->get_section_list());
            if ($user_type_sections) {
                $user_type_fields = $this->ci->Field_editor_model->get_fields_for_select($user_type_sections);
            } else {
                $user_type_fields = array();
            }
            $this->field_editor_fields[$user_type_gid] = $user_type_fields;
            $this->field_editor_sections[$user_type_gid] = $user_type_sections;
        }

        return $this->field_editor_fields[$user_type_gid];
    }

    /**
     * Return properties data from field editor (internal)
     *
     * @param string $user_type_gid     property type GUID
     * @param array  $fields_for_select fields from field editor
     * @param array  $users_ids         users identificators
     *
     * @return array
     */
    protected function _get_field_editor_data($user_type_gid, $fields_for_select, $users_ids)
    {
        array_unshift($fields_for_select, 'id');
        $this->ci->db->select(implode(',', $fields_for_select));
        $this->ci->db->from(USERS_TABLE);
        $this->ci->db->where_in('id', (array) $users_ids);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $data[$r['id']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return criteria for used in fulltext search
     *
     * @param ineteger $user_id user identifier
     * @param array    $fields  fulltext fields
     *
     * @return array
     */
    public function get_fulltext_data($user_id, $fields)
    {
        $all_langs = $this->ci->pg_language->return_langs();
        $default_lang = $this->ci->pg_language->get_default_lang_id();

        $return = array(
            'main_fields' => array(),
            'fe_fields' => array(),
            'default_lang_id' => $default_lang,
            'object_lang_id' => $default_lang,
        );

        $format_settings = $this->format_settings;

        $this->format_settings = array(
            'use_format' => true,
            'get_safe' => false,
            'get_contact' => false,
            'get_language' => false,
            'get_currency' => false,
            'get_group' => false,
            'get_company' => false,
            'get_field_editor' => true,
        );
        $data = $this->get_user_by_id($user_id);
        $this->format_settings = $format_settings;

        $use_multilang_search = $this->ci->pg_module->get_module_config('field_editor', 'use_multilang_search');
        if ($use_multilang_search) {
            $langs = $all_langs;
        } else {
            if (in_array($data['lang_id'], $this->ci->pg_language->languages)) {
                $return['object_lang_id'] = $data['lang_id'];
            }

            $current_lang = $return['object_lang_id'];
            $langs[$current_lang] = $all_langs[$current_lang];
            if ($current_lang != $default_lang) {
                $langs[$default_lang] = $all_langs[$default_lang];
            };
        }

        $return['main_fields'] = array('name' => $data['output_name'], 'email' => $data['email']);

        switch ($data['user_type']) {
            case 'company':
                $this->ci->load->model('Countries_model');

                foreach ($langs as $lang) {
                    if ($data['id_country']) {
                        $country = $this->ci->Countries_model->get_country($data['id_country'], $lang['id']);
                        if (isset($country['name']) && !empty($country['name'])) {
                            $return['main_fields']['location_' . $lang['code']] = $country['name'];
                            if ($data['id_region']) {
                                $region = $this->ci->Countries_model->get_region($data['id_region'], $lang['id']);
                                if (isset($region['name']) && !empty($region['name'])) {
                                    $return['main_fields']['location_' . $lang['code']] .= ' ' . $region['name'];
                                    if ($data['id_city']) {
                                        $city = $this->ci->Countries_model->get_city($data['id_city'], $lang['id']);
                                        if (isset($city['name']) && !empty($city['name'])) {
                                            $return['main_fields']['location_' . $lang['code']] .= ' ' . $city['name'];
                                            if (isset($district['name']) && !empty($district['name'])) {
                                                $return['main_fields']['location_' . $lang['code']] .= ' ' . $district['name'];
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if ($data['address']) {
                    $return['main_fields']['address'] = $data['address'];
                }
                if ($data['postal_code']) {
                    $return['main_fields']['postal_code'] = $data['postal_code'];
                }
                break;
            case 'agent':
                if ($data['agent_company'] && $data['agent_status']) {
                    $format_settings = $this->format_settings;

                    $this->format_settings = array(
                        'use_format' => true,
                        'get_safe' => false,
                        'get_contact' => false,
                        'get_language' => false,
                        'get_currency' => false,
                        'get_group' => false,
                        'get_company' => false,
                        'get_field_editor' => false,
                    );
                    $company = $this->get_user_by_id($data['agent_company']);
                    $this->format_settings = $format_settings;

                    $return['main_fields']['company'] = $company['output_name'];
                }
                break;
        }

        foreach ($fields as $field) {
            if (empty($data[$field])) {
                continue;
            }
            $return['fe_fields'][$field] = $data[$field];
        }

        return $return;
    }

    public function applyFormatSettings($settings = array()){
        $this->old_settings = $this->format_settings;
        $this->set_format_settings($settings);
    }

    public function revertFormatSettings(){
        $this->format_settings = $this->old_settings;
    }

    /**
     * Return form content from field editor
     *
     * @param string  $user_type  user type
     * @param string  $form_gid   form GUID
     * @param boolean $admin_mode admin mode
     *
     * @return array
     */
    public function get_search_extend_form($user_type, $form_gid, $admin_mode = true)
    {
        $this->ci->load->model('Field_editor_model');
        $this->ci->Field_editor_model->initialize($user_type);
        $form_gid = $this->{$this->{$form_gid} . '_' . $user_type . '_gid'};
        $this->ci->load->model('field_editor/models/Field_editor_forms_model');
        $form = $this->ci->Field_editor_forms_model->get_form_by_gid($form_gid, $user_type);
        $form = $this->ci->Field_editor_forms_model->format_output_form($form);

        return $form['field_data'];
    }

}
