<?php

namespace Pg\Modules\Users_connections\Controllers;

/**
 * Users connections module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Social networking admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users connections
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_users_connections extends \Controller
{

    /**
     * Allow to remove social service
     *
     * @var boolean
     */
    protected $allow_service_delete = false;

    /**
     * Allow to create social widget page
     *
     * @var boolean
     */
    protected $allow_page_add = false;

    /**
     * Allow to edit social widget page
     *
     * @var boolean
     */
    protected $allow_page_edit = false;

    /**
     * Allow to remove social widget page
     *
     * @var boolean
     */
    protected $allow_page_delete = false;

    /**
     * Class constructor
     *
     * @return Admin_users_connections
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');
    }

    /**
     * Manage social services
     *
     * @return void
     */
    public function services()
    {
        $this->load->model('users_connections/models/Users_connections_services_model');
        $services = $this->Users_connections_services_model->get_services_list();
        $this->view->assign('services', $services);

        $this->view->assign('allow_add', $this->allow_service_add);
        $this->view->assign('allow_edit', $this->allow_service_edit);
        $this->view->assign('allow_delete', $this->allow_service_delete);

        $this->Menu_model->set_menu_active_item('admin_social_networking_menu', 'services_list_item');
        $this->view->setHeader(l('admin_header_services_list', 'users_connections'));
        $this->view->render('list_services');
    }

    /**
     * Edit social service data
     *
     * @param integer $service_id service identifier
     *
     * @return void
     */
    public function service_edit($service_id = null)
    {
        $this->load->model('users_connections/models/Users_connections_services_model');
        if ($service_id) {
            $data = $this->Users_connections_services_model->get_service_by_id($service_id);
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "name" => $this->input->post('name', true),
                "gid" => $this->input->post('gid', true),
                "authorize_url" => $this->input->post('authorize_url', true),
                "access_key_url" => $this->input->post('access_key_url', true),
                "app_enabled" => $this->input->post('app_enabled', true),
                "oauth_enabled" => $this->input->post('oauth_enabled', true),
            );
            $validate_data = $this->Users_connections_services_model->validate_service($post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = array_merge($data, $validate_data["data"]);
            } else {
                $data = $validate_data["data"];

                $result = $this->Users_connections_services_model->save_service($service_id, $data, $langs);
                if (isset($result['errors']) && is_array($result['errors']) && count($result['errors']) > 0) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $result['errors']);
                    redirect(site_url('admin/users_connections/service_edit/' . $service_id));
                } else {
                    if ($service_id) {
                        $message = l('success_updated_service', 'users_connections');
                    } else {
                        $message = l('success_added_service', 'users_connections');
                    }
                    $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
                    redirect(site_url('admin/users_connections/services'));
                }
            }
        }

        $this->view->assign('data', $data);

        $this->view->setHeader($data['name']);

        $this->view->render('edit_service');
    }

    /**
     * Remove social service
     *
     * @param integer $service_id service identifier
     *
     * @return void
     */
    public function service_delete($service_id = null)
    {
        $this->load->model('users_connections/models/Users_connections_services_model');
        $this->Users_connections_services_model->delete_service($service_id);
        $url = site_url('admin/users_connections/services');
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_deleted_service', 'users_connections'));
        redirect($url);
    }

    /**
     * Activate/deactivate social service
     *
     * Available status values: 1 - activate, 0 -deactivate
     *
     * @param integer $service_id service idnetifier
     * @param integer $status     active status
     *
     * @return void
     */
    public function service_active($service_id = null, $status = '1')
    {
        $this->load->model('Users_connections_model');
        $this->load->model('users_connections/models/Users_connections_services_model');

        $url = site_url('admin/users_connections/services');
        if ($service_id) {
            $data = $this->Users_connections_services_model->get_service_by_id($service_id);
        }
        if (!empty($data)) {
            $data['status'] = floor($status);
            $this->Users_connections_services_model->save_service($service_id, $data);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_updated_service', 'users_connections'));
        }
        redirect($url);
    }

    /**
     * Activate/deactivate authorization by oauth
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param integer $service_id service identifier
     * @param integer $status     active status
     *
     * @return void
     */
    public function oauth_active($service_id = null, $status = '1')
    {
        $this->load->model('Users_connections_model');
        $this->load->model('users_connections/models/Users_connections_services_model');

        $url = site_url('admin/users_connections/services');
        if ($service_id) {
            $data = $this->Users_connections_services_model->get_service_by_id($service_id);
        }
        if (!empty($data)) {
            if ($status == 1) {
                if (!$this->Users_connections_services_model->enabled) {
                    $this->system_messages->addMessage(View::MSG_ERROR, l('no_curl', 'users_connections'));
                    redirect(site_url('admin/users_connections/services/'));
                }
                if ($data['oauth_version'] == 2) {
                    // Проверка подключения
                    $result = $this->Users_connections_model->_check_oauth2_connection($data,
                                                                                       site_url('admin/users_connections/oauth_active/' . $service_id . '/1/'));
                } else {
                    $result = $this->Users_connections_model->_check_oauth_connection($data,
                                                                                      site_url('admin/users_connections/oauth_active/' . $service_id . '/1/'));
                }
                // Вывод ошибок
                if (isset($result['error']) && $result['error']) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $result['error']);
                    redirect($url);
                }
            }
            $data['oauth_status'] = floor($status);
            $this->Users_connections_services_model->save_service($service_id, $data);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_updated_service', 'users_connections'));
        }
        redirect($url);
    }

    /**
     * Edit social service application settings
     *
     * @param integer $service_id service identifier
     *
     * @return void
     */
    public function application($service_id = null)
    {
        $this->load->model('users_connections/models/Users_connections_services_model');
        $data = $this->Users_connections_services_model->get_service_by_id($service_id);
        if ($data) {
            if ($this->input->post('btn_save')) {
                $post_data = (array) $data;
                $post_data["app_key"] = $this->input->post('app_key', true);
                $post_data["app_secret"] = $this->input->post('app_secret', true);
                $data = $post_data + $data;
                $validate_data = $this->Users_connections_services_model->validate_service($data);
                if (!empty($validate_data["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    $data = array_merge($data, $validate_data["data"]);
                } else {
                    $data = $validate_data["data"];
                    $data['oauth_status'] = 0;
                    $this->Users_connections_services_model->save_service($service_id, $data);

                    $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                       l('success_updated_application', 'users_connections'));
                    $url = site_url('admin/users_connections/services/');
                    redirect($url);
                }
            }

            $this->view->assign('data', $data);

            $this->view->setHeader($data['name']);

            $this->view->render('edit_application');
        } else {
            redirect(site_url('admin/users_connections/services'));
        }
    }

}
