<?php

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking google service model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Google_service_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    public $CI;

    /**
     * Url of google+ service api
     *
     * @var string
     */
    public $api_url = 'https://www.googleapis.com/oauth2/v1/';

    /**
     * Class constructor
     *
     * @return Google_service_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Return user data from google+ as array
     *
     * @param integer $user_id    user identifier
     * @param string  $access_key access token
     *
     * @return array/false
     */
    public function get_user_data($user_id = 0, $access_key = '')
    {
        $this->CI->load->model('Users_connections_model');
        $params = array(
            'access_token' => $access_key,
        );
        $response = $this->CI->Users_connections_model->curl_get($this->api_url . 'userinfo', $params);
        $data = (array) @json_decode($response);
        if (isset($data['id']) && isset($data['email']) && isset($data['given_name']) && isset($data['family_name'])) {
            $user_data = array(
                'id'    => $data['id'],
                'email' => $data['email'],
                'fname' => $data['given_name'],
                'sname' => $data['family_name'],
            );

            return $user_data;
        }

        return false;
    }

    /**
     * Return parameters of authorization request
     *
     * @param array  $service_data service data
     * @param string $redirect     redirect url
     *
     * @return array
     */
    public function get_auth_params($service_data = array(), $redirect)
    {
        $app_key = isset($service_data['app_key']) ? $service_data['app_key'] : false;
        $params = array(
            'client_id'     => $app_key,
            'redirect_uri'  => site_url(),
            'state'         => str_replace(site_url(), '/', $redirect),
            'response_type' => 'code',
            'scope'         => 'https://www.googleapis.com/auth/userinfo.email https://www.googleapis.com/auth/userinfo.profile',
        );
        ksort($params);

        return $params;
    }

    /**
     * Return parameters for check token
     *
     * @param array  $service_data service data
     * @param string $redirect     redirect url
     *
     * @return array
     */
    public function get_token_params($service_data = array(), $redirect)
    {
        $app_key = isset($service_data['app_key']) ? $service_data['app_key'] : false;
        $app_secret = isset($service_data['app_secret']) ? $service_data['app_secret'] : false;
        $params = array(
            'client_id'     => $app_key,
            'client_secret' => $app_secret,
            'redirect_uri'  => site_url(),
            //'state' => str_replace(site_url(), '', $redirect),
            'grant_type' => 'authorization_code',
        );
        ksort($params);

        return $params;
    }
}
