<?php

namespace Pg\Modules\Users_private\Models;

use Pg\Libraries\Setup;

define('USERS_PRIVATE_INSTALLING', true);

/**
 * Users private module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users private install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users_private
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_private_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    protected $modules_data = [];

    /**
     * Constructor
     *
     * @return Users_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->modules_data = Setup::getModuleData('users_private', Setup::TYPE_MODULES_DATA);
        $this->ci = &get_instance();
    }

    protected function installMenu(array $menu_list)
    {
        $this->ci->load->helper("menu");

        foreach ($menu_list as $gid => $menu_data) {
            $menu_list[$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($menu_list, "create", $gid, 0, $menu_list[$gid]["items"]);
        }
    }

    protected function installMenuLangUpdate($langs_ids, array $menu_list)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users_private", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data (users_private)");
            return false;
        }

        $this->ci->load->helper("menu");
        foreach ($menu_list as $gid => $menu_data) {
            linked_install_process_menu_items($menu_list, "update", $gid, 0, $menu_list[$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    protected function installMenuLangExport(array $menu_list)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($menu_list as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($menu_list, "export", $gid, 0, $menu_list[$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    protected function deinstallMenu(array $menu_list)
    {
        $this->ci->load->helper("menu");

        foreach ($menu_list as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_list[$gid]["items"]);
            }
        }
    }

    /**
     * Install users_private data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        return $this->installMenu($this->modules_data['menu']);
    }

    /**
     * Import users_private languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        return $this->installMenuLangUpdate($langs_ids, $this->modules_data['menu']);
    }

    /**
     * Export users_private languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        return $this->installMenuLangExport();
    }

    /**
     * Uninstall users_private data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->deinstallMenu($this->modules_data['menu']);
    }

    /**
     * Install users_private data to menu module
     *
     * @return void
     */
    public function install_menu_users_deactivate()
    {
        return $this->installMenu($this->modules_data['menu_users_deactivate']);
    }

    /**
     * Import users_private languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_users_deactivate_lang_update($langs_ids = null)
    {
        return $this->installMenuLangUpdate($langs_ids, $this->modules_data['menu_users_deactivate']);
    }

    /**
     * Export users_private languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_users_deactivate_lang_export($langs_ids)
    {
        return $this->installMenuLangExport($this->modules_data['menu_users_deactivate']);
    }

    /**
     * Uninstall users_private data from menu module
     *
     * @return void
     */
    public function deinstall_menu_users_deactivate()
    {
        $this->deinstallMenu($this->modules_data['menu_users_deactivate']);
    }

    /**
     * Install users data to field editor module
     *
     * @return void
     */
    public function install_field_editor()
    {
        $this->ci->load->model("Field_editor_model");
        $this->ci->load->model("field_editor/models/Field_editor_forms_model");

        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            include MODULEPATH . 'users_private/install/users_fields_' . $type_gid . '_data.php';
            $this->ci->Field_editor_model->import_type_structure($type_gid, $fe_sections, $fe_fields, $fe_forms);
            unset($fe_sections);
            unset($fe_fields);
            unset($fe_form);
        }

        return true;
    }

    /**
     * Import users languages to field editor module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_field_editor_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users_private", "field_editor", $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty field editor langs data');
            return false;
        }
        $this->ci->load->model('Field_editor_model');
        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            include MODULEPATH . 'users/install/users_fields_' . $type_gid . '_data.php';
            $this->ci->Field_editor_model->initialize($type_gid);
            $this->ci->Field_editor_model->update_sections_langs($fe_sections, $langs_file);
            $this->ci->Field_editor_model->update_fields_langs($type_gid, $fe_fields, $langs_file);
        }
        return true;
    }

    /**
     * Export users languages from field editor module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_field_editor_lang_export($langs_ids = null)
    {
        $this->ci->load->model('Field_editor_model');
        $sections = $fields = array();
        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            $this->ci->Field_editor_model->initialize($type_gid);
            list($fe_sections, $fe_fields, $fe_forms) = $this->ci->Field_editor_model->export_type_structure($type_gid, 'application/modules/users/install/users_fields_' . $type_gid . '_data.php');
            if (!empty($fe_sections)) {
                $sections = array_merge($sections, $this->ci->Field_editor_model->export_sections_langs($fe_sections, $langs_ids));
            }
            if (!empty($fe_fields)) {
                $fields = array_merge($fields, $this->ci->Field_editor_model->export_fields_langs($type_gid, $fe_fields, $langs_ids));
            }
        }

        return array('field_editor' => array_merge($sections, $fields));
    }

    /**
     * Uninstall users data from field editor module
     *
     * @return void
     */
    public function deinstall_field_editor()
    {
        $this->ci->load->model("Field_editor_model");
        $this->ci->load->model("field_editor/models/field_editor_forms_model");

        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            $this->ci->Field_editor_model->initialize($type_gid);
            list($fe_sections, $fe_fields, $fe_forms) = $this->ci->Field_editor_model->export_type_structure($type_gid, MODULEPATH . 'users_private/install/users_fields_' . $type_gid . '_data.php');

            foreach ((array) $fe_forms as $form_gid => $form_data) {
                if ($form_data['data']["editor_type_gid"] != $type_gid) {
                    continue;
                }
                $this->ci->Field_editor_forms_model->delete_form_by_gid($form_data['data']['gid']);
            }
            foreach ((array) $fe_fields as $field_gid => $field_data) {
                if ($field_data['data']["editor_type_gid"] != $type_gid) {
                    continue;
                }
                $this->ci->Field_editor_model->delete_field_by_gid($field_data['data']['gid']);
            }
            foreach ((array) $fe_sections as $section_gid => $section_data) {
                if ($section_data['data']["editor_type_gid"] != $type_gid) {
                    continue;
                }
                $this->ci->Field_editor_model->delete_section_by_gid($section_data['data']['gid']);
            }
        }
    }

    /**
     * Install users data to social networking module
     *
     * @return void
     */
    public function install_social_networking()
    {
        // add social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        foreach ((array) $this->modules_data['social_networking'] as $social_networking_data) {
            $validation_data = $this->ci->Social_networking_pages_model->validate_page(null, $social_networking_data);
            if (!empty($validation_data['errors'])) {
                continue;
            }
            $this->ci->Social_networking_pages_model->save_page(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall users data from social networking module
     *
     * @return void
     */
    public function deinstall_social_networking()
    {
        // delete social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        $this->ci->Social_networking_pages_model->delete_pages_by_controller('users');
    }

    /**
     * Install users data to payments module
     *
     * @return void
     */
    public function install_payments()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            $this->modules_data['menu_payments'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu_payments'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import users languages to payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_payments_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users_private", "menu_payments", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty menu langs data for payments (users)");
            return false;
        }
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu_payments'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
        }
        return true;
    }

    /**
     * Export users languages from payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_payments_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu_payments'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }
        return array('menu_payments' => $return);
    }

    /**
     * Uninstall users data from payments module
     *
     * @return void
     */
    public function deinstall_payments()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }
    }

    /**
     * Install data of users_settings module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        // add seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:registration-as-private-person]',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'reg_private', $data);

        $xml_data['users']['reg_private'] = $this->ci->pg_seo->url_template_transform('users', 'reg_private', $data['url_template'], 'base', 'xml');

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        $this->addUserTypes();
        
        // DEMO
        //if (!INSTALL_DONE) {
            $this->add_demo_content();
        //}
    }

    protected function addUserTypes()
    {
        $this->ci->load->model('users/models/Users_types_model');
        $this->ci->Users_types_model->addTypes($this->modules_data['user_types']);
    }

    /**
     * Import languages of users_settings module
     *
     * @return boolean
     */
    public function _arbitrary_lang_install()
    {
        $langs_file = $this->ci->Install_model->language_file_read("users_private", "arbitrary");
        if (!$langs_file) {
            log_message("info", "Empty arbitrary langs data");
            return false;
        }

        $post_data = array(
            "title" => $langs_file["seo_tags_register_private_title"],
            "keyword" => $langs_file["seo_tags_register_private_keyword"],
            "description" => $langs_file["seo_tags_register_private_description"],
            "header" => $langs_file["seo_tags_register_private_header"],
            "og_title" => $langs_file["seo_tags_register_private_og_title"],
            "og_type" => $langs_file["seo_tags_register_private_og_type"],
            "og_description" => $langs_file["seo_tags_register_private_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "users", "reg_private", $post_data);
    }

    /**
     * Export languages of users_settings module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        $arbitrary_return = array();

        $settings = $this->ci->pg_seo->get_settings("user", "users", "reg_private", $langs_ids);
        $arbitrary_return["seo_tags_register_private_title"] = $settings["title"];
        $arbitrary_return["seo_tags_register_private_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_register_private_description"] = $settings["description"];
        $arbitrary_return["seo_tags_register_private_header"] = $settings["header"];
        $arbitrary_return["seo_tags_register_private_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_register_private_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_register_private_og_description"] = $settings["og_description"];

        return array("arbitrary" => $arbitrary_return);
    }

    /**
     * Unistall data of users_settings module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        $this->ci->pg_module->set_module_config("users", "private_login_enabled", 0);
        $this->ci->pg_module->set_module_config("users", "private_search_enabled", 0);

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'reg_private', $data);
    }

    /**
     * Install demo content of users module
     *
     * @return void
     */
    public function add_demo_content()
    {
        $demo_content = Setup::getModuleData('users_private', Setup::TYPE_DEMO_CONTENT);

        foreach ((array) $demo_content['users'] as $user_data) {
            $user_data['repassword'] = $user_data['password'];
            $validate_data = $this->ci->Users_model->validate_user(null, $user_data);
            if (!empty($validate_data['errors'])) {
                continue;
            }
            $this->ci->Users_model->save_user(null, $validate_data['data']);
        }

        if ($this->ci->pg_module->is_module_installed("reviews")) {
            $this->ci->load->model('Reviews_model');
            foreach ((array) $demo_content['reviews'] as $review) {
                $validate_data = $this->ci->Reviews_model->validate_review(null, $review);
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $this->ci->Reviews_model->save_review(null, $validate_data['data']);
            }
        }
assert(false);
        return true;
    }

}
