<?php

namespace Pg\Modules\Users_services\Controllers;

/**
 * Users services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users services api controller
 *
 * @package 	PG_RealEstate
 * @subpackage	Users services
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Users_services extends \Controller
{
    /**
     * Constructor
     *
     * @return Api_Users_services
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Users_services_model');
    }

    /**
     * Available paid services
     *
     * $_POST: string $filter contact|post|combined
     *
     * @return void
     */
    public function list_services()
    {
        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $data['usertype'] = $user_type;

        $available_filter = array('contact', 'post', 'combined');
        $filter = (string) $this->input->post('filter', true);
        if (!in_array($filter, $available_filter)) {
            $filter = '';
        }

        $this->load->model('Services_model');

        /* Contact services */
        if (!$filter || $filter == 'contact') {
            $service_template = array('contact_template');
            $contact_services = $this->Services_model->get_active_services($user_id, $service_template);
            $data['contact_services'] = $contact_services;
        }

        /* Post services */
        if (!$filter || $filter == 'post') {
            $service_template = array('post_template');
            $post_services = $this->Services_model->get_active_services($user_id, $service_template);
            $data['post_services'] = $post_services;
        }

        /* Combined services */
        if (!$filter || $filter == 'combined') {
            $service_template = (array) ('combined_template');
            $combined_services = $this->Services_model->get_active_services($user_id, $service_template);
            $data['combined_services'] = $combined_services;
        }

        // Featured service
        if (!$filter || $filter == 'featured') {
            $service_template = array('featured_template');
            $featured_services = $this->Services_model->get_active_services($user_id, $service_template);
            $data['featured_services'] = $featured_services;
        }

        // Show logo service
        if (!$filter || $filter == 'show_logo') {
            $service_template = array('show_logo_template');
            $show_logo_services = $this->Services_model->get_active_services($user_id, $service_template);
            $data['show_logo_services'] = $show_logo_services;
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Paid services of me
     *
     * @return void
     */
    public function my()
    {
        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $data = array(
            'user_id'  => $user_id,
            'usertype' => $user_type,
        );

        $order = array('id' => 'DESC');

        $params = array(
            "where" => array(
                "id_user"             => $user_id,
                "template_gid"        => "contact_template",
                "status"              => 1,
                "id_users_membership" => 0,
            ),
        );
        $current_services['contact'] = $this->Users_services_model->get_services_list($order, $params);

        $params = array(
            "where" => array(
                "id_user"             => $user_id,
                "template_gid"        => "post_template",
                "status"              => 1,
                "id_users_membership" => 0,
            ),
        );
        $current_services['post'] = $this->Users_services_model->get_services_list($order, $params);

        $params = array(
            "where" => array(
                "id_user"             => $user_id,
                "status"              => 1,
                "id_users_membership" => 0,
            ),
            "where_in" => array(
                "template_gid" => array("contact_template", "post_template"),
            ),
        );
        $current_services['combined'] = $this->Users_services_model->get_services_list($order, $params);

        $current_services['featured'] = $this->Users_services_model->is_user_featured($user_id);
        $current_services['show_logo'] = $this->Users_services_model->is_user_show_logo($user_id);

        $data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        $this->load->helper('date_format');

        foreach ($current_services['contact'] as $i => $contact_service) {
            $current_services['contact'][$i]['contact_service_end_date_output'] = tpl_date_format($contact_service['service_end_date'], $data['date_format']);
        }

        foreach ($current_services['combined'] as $i => $combined_service) {
            $current_services['combined'][$i]['contact_service_end_date_output'] = tpl_date_format($combined_service['service_end_date'], $data['date_format']);
        }

        if (!empty($current_services['featured']['is_featured'])) {
            $current_services['featured']['featured_end_date_output'] = tpl_date_format($current_services['featured']['featured_end_date'], $data['date_format']);
        }

        if (!empty($current_services['show_logo']['is_show_logo'])) {
            $current_services['show_logo']['show_logo_end_date_output'] = tpl_date_format($current_services['show_logo']['show_logo_end_date'], $data['date_format']);
        }

        $data['current_services'] = $current_services;

        $this->set_api_content('data', $data);
    }

    /**
     * Activate contact service
     *
     * @param integer $contact_id contact identifier
     *
     * @return void
     */
    public function activate_contact($contact_id)
    {
        $user_id = $this->session->userdata('user_id');

        $id_user_service = $this->input->post('id_user_service', true);

        $user_data = array('contact' => $contact_id);

        $user_service = $this->Users_services_model->get_service($id_user_service);

        if (empty($user_service)) {
            show404();
        }

        $validate_data = $this->Users_services_model->validate_contact_activate($user_service, $user_data);

        if (!empty($validate_data['errors'])) {
            $status = 0;
            $this->set_api_content('error', l('error_service_activating', 'users_services'));
        } else {
            $this->Users_services_model->service_contact_activate($user_service, $user_data);
            $status = 1;
            $this->set_api_content('messages', l('success_service_activating', 'users_services'));
        }

        $this->set_api_content('data', array('status' => $status));
    }
}
