<?php

namespace Pg\Modules\Users_services\Models;

use Pg\Modules\Services\Models\Services_users_model;

/**
 * Users services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * Users services model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users services
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_services_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * @return Users_services_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return number of services objects in data source
     *
     * @param integer $user_id    user identifier
     * @param string  $count_name count name
     *
     * @return integer
     */
    public function get_count($user_id = 0, $count_name)
    {
        $this->ci->load->model("services/models/Services_users_model");
        return $this->ci->Services_users_model->get_count($user_id, $count_name);
    }

    /**
     * Save service object to data source
     *
     * @param integer $service_id service identifier
     * @param array   $attrs      service data
     *
     * @return integer
     */
    public function save_service($service_id = null, $attrs = array())
    {
        $this->ci->load->model("services/models/Services_users_model");
        return $this->ci->Services_users_model->save_service($service_id, $attrs);
    }

    /**
     * Return services objects as array
     *
     * @param array $order_by          sorting data
     * @param array $params            filters data
     * @param array $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_services_list($order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->load->model("services/models/Services_users_model");
        return $this->ci->Services_users_model->get_services_list($params, $order_by, $filter_object_ids);
    }

    /**
     * Format service data
     *
     * @param array $data service data
     *
     * @return array
     */
    public function format_service($data)
    {
        return current($this->format_services(array($data)));
    }

    /**
     * Format services data
     *
     * @param array $data services data
     *
     * @return array
     */
    public function format_services($data)
    {
        $this->ci->load->model("services/models/Services_users_model");
        return $this->ci->Services_users_model->format_services($data);
    }

    /**
     * Return service data by identifier
     *
     * @param integer $user_service_id service identifier
     *
     * @return array
     */
    public function get_service($user_service_id)
    {
        $this->ci->load->model("services/models/Services_users_model");
        return $this->ci->Services_users_model->get_service_by_id($user_service_id);
    }

    private function findArrayListingsIds($data)
    {
        // TODO: Refactor this whole module to use polymorphism.
        $existent_keys = array_intersect(
            array('listing_id', 'id_listing', 'listing'), 
            array_keys($data)
        );
        foreach($existent_keys as $key) {
            if(is_array($data[$key]) && !empty($data[$key])) {
                return $data[$key];
            } elseif($data[$key] > 0) {
                return array($data[$key]);
            }
        }
        log_message('error', '(listings_service) Empty listing id');
        return 0;
    }


    /**
     * Buy contact service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return boolean
     */
    public function service_contact_buy($user_service, $user_data, $price = '')
    {
        $contact_count = intval($user_service["service_data_array"]["data_admin_array"]["contact_count"]);

        $service_data["available_data"] = serialize(array("contact_count" => $contact_count));

        if (!empty($user_service['data_admin_array']["service_period"])) {
            $period = time() + $user_service['data_admin_array']["service_period"] * 24 * 60 * 60;
            $service_data["service_end_date"] = date("Y-m-d H:i:s", $period);
        }

        $this->save_service($user_service['id'], $service_data);

        // send notification
        $date_format = $this->ci->pg_date->get_format('date_time_literal', 'st');

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_service["id_user"]);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('service_string_name_contact', 'users_services', $user['lang_id']),
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_package_enabled", $mail_data, '', $user['lang_id']);

        return !empty($user_data['contact']);
    }

    /**
     * Activate contact service
     *
     * @param integer $user_service user service data
     * @param array   $user_data    service data filled by user
     * @param float   $price        payment price
     *
     * @return boolean
     */
    public function service_contact_activate($user_service, $user_data, $price = '')
    {
        $id_user = $this->ci->session->userdata('user_id');

        $this->ci->load->model('Users_model');
        $this->ci->Users_model->add_contact($id_user, $user_data['contact']);

        --$user_service["available_data_array"]["contact_count"];

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["contact_count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);
    }

    /**
     * Validate contact service object
     *
     * @param integer $user_id      user identifier
     * @param array   $user_data    data filled by user
     * @param array   $service_data data from service
     * @param float   $price        service price
     *
     * @return array
     */
    public function service_contact_validate($user_id, $user_data, $service_data = array(), $price = 0)
    {
        return array("errors" => array(), "data" => $data);
    }

    /**
     * Validate contact service
     *
     * @param integer $user_service      user service data
     * @param array   $service_user_data service data filled by user
     * @param float   $price             payment price
     *
     * @return boolean
     */
    public function validate_contact_activate($user_service, $user_data, $price = '')
    {
        $return = array("errors" => array(), "data" => array());

        if (!empty($user_service)) {
            $id_user = $this->ci->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                    $user_service["available_data_array"]["contact_count"] == 0 ||
                    $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'users_services');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'users_services');
        }

        if (isset($user_data['contact'])) {
            $return["data"]["contact"] = intval($user_data['contact']);
            if (empty($return["data"]["contact"])) {
                $return['errors'][] = l('error_contact_empty', 'users_services');
            }
        } else {
            $return['errors'][] = l('error_contact_empty', 'users_services');
        }

        return $return;
    }

    /**
     * Buy post service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return boolean
     */
    public function service_post_buy($user_service, $user_data, $price = '')
    {
        $post_count = intval($user_service["service_data_array"]["data_admin_array"]["post_count"]);
        $post_period = intval($user_service["service_data_array"]["data_admin_array"]["period"]);

        $service_data["available_data"] = serialize(array("post_count" => $post_count, 'post_period' => $post_period));

        return $this->save_service($user_service['id'], $service_data);
    }

    /**
     * Activate post listing service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return void
     */
    public function service_post_activate($user_service, $user_data, $price = '')
    {
        $post_period = $user_service["available_data_array"]["post_period"];
       
        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }
 
        $this->ci->load->model('Listings_model');

        foreach ($listing_id as $id) {
            $listing = $this->ci->Listings_model->get_listing_by_id($id);
            if (!$listing['initial_moderation']) {
                $this->ci->Listings_model->save_listing($listing['id'], array('initial_activity' => $post_period), false);
            } else {
                $date_expired = strtotime($listing['date_expire']);
                if ($date_expired > time()) {
                    $uts = $date_expired;
                } else {
                    $uts = time();
                }
                $date_expired = date('Y-m-d H:i:s', $uts + $post_period * 24 * 60 * 60);
                $this->ci->Listings_model->activate_listing($id, 1, $date_expired);
            }
            $user_service["available_data_array"]["post_count"] --;
        }  

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["post_count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $date_format = $this->ci->pg_date->get_format('date_time_literal', 'st');

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_service["id_user"]);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('service_string_name_post', 'users_services', $user['lang_id']),
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_package_enabled", $mail_data, '', $user['lang_id']);
    }

    /**
     * Validate post service object
     *
     * @param integer $user_id      user identifier
     * @param array   $user_data    data filled by user
     * @param array   $service_data data from service
     * @param float   $price        service price
     *
     * @return array
     */
    public function service_post_validate($user_id, $user_data, $service_data = array(), $price = 0)
    {
        return array("errors" => array(), "data" => $service_data);
    }

    /**
     * Validate post service object
     *
     * @param array $user_service user service
     * @param array $user_data    user data
     *
     * @return array
     */
    public function validate_post_activate($user_service, $user_data)
    {
        $return = array("errors" => array(), "data" => array());

        if (!empty($user_service)) {
            $id_user = $this->ci->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                    $user_service["available_data_array"]["post_count"] == 0 ||
                    $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'users_services');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'users_services');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_listing_empty', 'users_services');
            }
        } else {
            $return['errors'][] = l('error_listing_empty', 'users_services');
        }

        return $return;
    }

    /**
     * Buy combined service
     *
     * @param array $user_service user service data
     * @param array $user_data    user data
     * @param float $price        payment price
     *
     * @return boolean
     */
    public function service_combined_buy($user_service, $user_data, $price = '')
    {
        // send notification
        $date_format = $this->ci->pg_date->get_format('date_time_literal', 'st');

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_service["id_user"]);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('service_string_name_combined', 'users_services', $user['lang_id']),
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_package_enabled", $mail_data, '', $user['lang_id']);

        return true;
    }

    /**
     * Activate combined service
     *
     * @param array $user_service      user service data
     * @param array $service_user_data data filled by user
     * @param float $price             payment price
     *
     * @return void
     */
    public function service_combined_activate($user_service, $service_user_data, $price = '')
    {
        $this->ci->load->model('services/models/Services_users_model');
        $this->Services_users_model->copyService($user_service, array('contact_template', 'listings_post_template'));

        $save_service["status"] = 0;
        $this->save_service($user_service['id'], $save_service);
    }

    /**
     * Validate combined service
     *
     * @param integer $user_id      user identifier
     * @param array   $data         data from user
     * @param array   $service_data data from service
     * @param float   $price        service price
     *
     * @return array
     */
    public function service_combined_validate($user_id, $data, $service_data = array(), $price = "")
    {
        return array("errors" => array(), "data" => $data);
    }

    /**
     * Buy featured service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return boolean
     */
    public function service_featured_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        $service_data["available_data"] = serialize(array("period" => $period));

        $this->save_service($user_service['id'], $service_data);

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_service["id_user"]);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('service_string_name_featured', 'users_services', $user['lang_id']),
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_package_enabled", $mail_data, '', $user['lang_id']);

        return true;
    }

    /**
     * Activate featured service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return void
     */
    public function service_featured_activate($user_service, $user_data, $price = '')
    {
        $user_id = $this->ci->session->userdata('user_id');

        $this->ci->load->model("Users_model");

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_id);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        if (strtotime($user["featured_end_date"]) > 86400) {
            $uts = strtotime($user["featured_end_date"]);
        } else {
            $uts = time();
        }

        $period = $uts + $user_service["available_data_array"]["period"] * 60 * 60 * 24;

        $data = array("is_featured" => 1, "featured_end_date" => date("Y-m-d H:i:s", $period));
        $this->ci->Users_model->save_user($user['id'], $data);

        $save_service["status"] = 0;
        $this->save_service($user_service['id'], $save_service);

        // send notification
        $date_format = $this->ci->pg_date->get_format('date_time_literal', 'st');

        $this->ci->load->helper('date_format');
        $expire_date = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('featured_user_services', 'users_services', $user['lang_id']),
            'date' => $expire_date,
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_service_enabled", $mail_data, '', $user['lang_id']);
    }

    /**
     * Validate featured service
     *
     * @param integer $user_id      user identifier
     * @param array   $data         data from user
     * @param array   $service_data data from service
     * @param float   $price        service price
     * @param array   $user_service user service data
     *
     * @return array
     */
    public function service_featured_validate($user_id, $data, $service_data = array(), $price = "", $user_service = array())
    {
        return array("errors" => array(), "data" => $data);
    }

    /**
     * Validate featured service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     *
     * @return array
     */
    public function validate_featured_activate($user_service, $user_data)
    {
        return array("errors" => array(), "data" => array());
    }

    /**
     * Clear featured services with expired date by cronjob
     *
     * @return void
     */
    public function service_featured_cron()
    {
        $this->ci->load->model("Users_model");
        $clean = $this->ci->Users_model->clear_end_date("featured_end_date", 'featured_user_services');
        echo "Make clean (user featured): " . $clean . "Featured users was removed";
    }

    /**
     * Buy show logo service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return boolean
     */
    public function service_show_logo_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];
        $service_data["available_data"] = serialize(array("period" => $period));
        $this->save_service($user_service['id'], $service_data);

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_service["id_user"]);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('service_string_name_featured', 'users_services', $user['lang_id']),
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_package_enabled", $mail_data, '', $user['lang_id']);

        return true;
    }

    /**
     * Activate show logo service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     * @param float $price        payment price
     *
     * @return void
     */
    public function service_show_logo_activate($user_service, $user_data, $price = '')
    {
        $user_id = $this->ci->session->userdata('user_id');

        $this->ci->load->model("Users_model");

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_id);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        if (strtotime($user["show_logo_end_date"]) > 86400) {
            $uts = strtotime($user["show_logo_end_date"]);
        } else {
            $uts = time();
        }

        $period = $uts + $user_service["available_data_array"]["period"] * 24 * 60 * 60;

        $data = array("is_show_logo" => 1, "show_logo_end_date" => date("Y-m-d H:i:s", $period));
        $this->ci->Users_model->save_user($user['id'], $data);

        // send notification
        $date_format = $this->ci->pg_date->get_format('date_time_literal', 'st');

        $this->ci->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $save_service["status"] = 0;
        $this->save_service($user_service['id'], $save_service);

        $mail_data = array(
            'user' => $user['output_name'],
            'name' => l('show_logo_services', 'users_services', $user['lang_id']),
            'date' => $date_expired,
        );

        $this->ci->load->model("Notifications_model");
        $this->ci->Notifications_model->send_notification($user['email'], "user_service_enabled", $mail_data, '', $user['lang_id']);
    }

    /**
     * Validate show logo service object
     *
     * @param integer $user_id      user identifier
     * @param array   $data         data from user
     * @param array   $service_data data from service
     * @param float   $price        service price
     *
     * @return array
     */
    public function service_show_logo_validate($user_id, $data, $service_data = array(), $price = "", $user_service = array())
    {
        $return = array("errors" => array(), "data" => $data);
        return $return;
    }

    /**
     * Validate show logo service
     *
     * @param array $user_service user service data
     * @param array $user_data    data filled by user
     *
     * @return array
     */
    public function validate_show_logo_activate($user_service, $user_data)
    {
        return array("errors" => array(), "data" => array());
    }

    /**
     * Clear show logo services with expired date by cronjob
     *
     * @return void
     */
    public function service_show_logo_cron()
    {
        $this->ci->load->model("Users_model");
        $clean = $this->ci->Users_model->clear_end_date("show_logo_end_date", 'show_logo_services');
        echo "Make clean(Users show logo): " . $clean . " Users was removed";
    }

    /**
     * Clear contact services with expired date
     *
     * @return void
     */
    public function update_services()
    {
        $data = array(
            "service_end_date" => "0000-00-00 00:00:00",
            "status" => 0,
        );

        $params = array(
            "where" => array(
                "service_end_date <" => date("Y-m-d H:i:s"),
                "UNIX_TIMESTAMP(service_end_date) >" => "86400",
                "status" => "1",
            ),
        );

        $this->ci->load->model("services/models/Services_users_model");
        $clean = $this->ci->Services_users_model->update_service($params, $data);
        echo "User services updating: " . $clean . " services was deactivated";
    }

    /**
     * Check contact service is available
     *
     * @param integer $id_user    user identifier
     * @param integer $id_contact linker identifier
     *
     * @return array
     */
    public function available_contact_action($id_user, $id_contact)
    {
        $is_contact = $this->is_available_contact($id_user, $id_contact);

        $return["available"] = 0;
        $return["content"] = "";
        $return["content_buy_block"] = false;

        if ($is_contact || $id_user == $id_contact) {
            $return["available"] = 1;
        } else {
            $this->ci->load->model("Users_model");
            $user_type = $this->ci->Users_model->get_user_type_by_id($id_user);

            $this->ci->load->model("Services_model");

            $services_params = array();

            $services_params["where_in"]["template_gid"] = array(
                "contact_template",
            );

            $criteria = $this->ci->Services_model->getUserTypeCriteria($user_type);

            $services_params = array_merge_recursive($services_params, $criteria);

            $services_params["where"]["status"] = 1;
            $services_count = $this->ci->Services_model->get_service_count($services_params);

            if ($services_count > 0) {                
                // если сервисы есть, и есть что списать
                // если сервисы есть, но нет с чего списывать
                $return["available"] = 0;
                $return["content_buy_block"] = true;
                
                // if user has membership
                $user_contact_services = $this->getUserContactServices($id_user);
                foreach ($user_contact_services as $user_contact_service) {
                    if (!empty($user_contact_service['id_users_membership'])) {
                        $return["available"] = 1;
                        $return["content_buy_block"] = false;
                        break;
                    }
                }
            } else {
                // если сервисов нет - то бесплатно
                $return["available"] = 1;
            }
        }

        return $return;
    }

    /**
     * Check contact link is available
     *
     * @param integer $id_user    user identifier
     * @param integer $id_contact linker identifier
     * @param array
     */
    public function is_available_contact($id_user, $id_contact)
    {
        if (!$this->ci->pg_module->is_module_active("linker")) {
            return false;
        }

        static $contacts = null;

        if (is_null($contacts)) {
            $this->ci->load->model("Linker_model");
            $results = $this->ci->Linker_model->get_links_simple_arr_count('users_contacts', array($id_user));
            if ($results) {
                $contacts = $results[$id_user];
            }
        }

        return isset($contacts[$id_contact]) && $contacts[$id_contact] > 0;
    }

    /**
     * Check user is featured
     *
     * @param integer $user_id user identifier
     *
     * @return array
     */
    public function is_user_featured($user_id)
    {
        $this->ci->load->model("Users_model");
        $this->ci->Users_model->set_format_settings('use_format', false);
        $user = $this->ci->Users_model->get_user_by_id($user_id);
        $this->ci->Users_model->set_format_settings('use_format', true);
        return array("is_featured" => $user['is_featured'], "featured_end_date" => $user["featured_end_date"]);
    }

    /**
     * Check is show user logo in search listings
     *
     * @param integer $user_id user identifier
     *
     * @return array
     */
    public function is_user_show_logo($user_id)
    {
        $this->ci->load->model("Users_model");
        $this->ci->Users_model->set_format_settings('use_format', false);
        $user = $this->ci->Users_model->get_user_by_id($user_id);
        $this->ci->Users_model->set_format_settings('use_format', true);
        return array('is_show_logo' => $user['is_show_logo'], 'show_logo_end_date' => $user['show_logo_end_date']);
    }

    /**
     * Return data for rewriting seo urls
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function get_seo_settings($method = "", $lang_id = "")
    {
        if (!empty($method)) {
            return $this->_get_seo_settings($method, $lang_id);
        } else {
            $actions = array("index", "my_services");
            $return = array();
            foreach ($actions as $action) {
                $return[$action] = $this->_get_seo_settings($action, $lang_id);
            }

            return $return;
        }
    }

    /**
     * Return data for rewriting seo urls (internal)
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function _get_seo_settings($method, $lang_id = '')
    {
        if ($method == "index") {
            return array(
                "templates" => array(),
                "url_vars" => array(
                    "service_gid" => array("service_gid" => "literal"),
                ),
            );
        } elseif ($method == "my_services") {
            return array(
                "templates" => array(),
                "url_vars" => array(),
            );
        }
    }

    /**
     * Transform seo value from query string to method variables
     *
     * @param string $var_name_from variable from url
     * @param string $var_name_to   method variable
     * @param mixed  $value         variable value
     *
     * @return mixed
     */
    public function request_seo_rewrite($var_name_from, $var_name_to, $value)
    {
        return $value;
    }

    /**
     * Return data of urls for xml site map
     *
     * @return array
     */
    public function get_sitemap_xml_urls()
    {
        return array();
    }

    /**
     * Return data of urls for site map
     *
     * @return array
     */
    public function get_sitemap_urls()
    {
        $this->ci->load->helper("seo");
        $auth = $this->ci->session->userdata("auth_type");
        $user_type = $this->ci->session->userdata("user_type");
        $block = array();

        return $block;
    }

    /**
     * Check contact service is available for specific user type
     *
     * @param string $user_type user type
     *
     * @return boolean
     */
    public function is_active_contact_services($user_type = null)
    {
        $this->ci->load->model("Services_model");

        $services_params = array(
            "where" => array(
                "status" => 1,
            ),
            "where_in" => array(
                "template_gid" => array("contact_template", "combined_template"),
            ),
        );
        if ($user_type) {
            $criteria = $this->ci->Services_model->getUserTypeCriteria($user_type);
            $services_params = array_merge_recursive($services_params, $criteria);
        } else {
            
        }

        $services_count = $this->ci->Services_model->get_service_count($services_params);
        return $services_count > 0;
    }

    /**
     * Check show logo services available for specific user type
     *
     * @param string $user_type user type
     *
     * @return boolean
     */
    public function is_active_show_logo_services($user_type = null)
    {
        $this->ci->load->model("Services_model");

        $services_params = array(
            "where" => array(
                "template_gid" => "show_logo_template",
                "status" => 1,
            ),
        );

        if ($user_type) {
            $criteria = $this->ci->Services_model->getUserTypeCriteria($user_type);
            $services_params = array_merge_recursive($services_params, $criteria);
        }

        $services_count = $this->ci->Services_model->get_service_count($services_params);
        return $services_count > 0;
    }

    /**
     * Check post services available for specific user type
     *
     * @param string $user_type user type
     *
     * @return boolean
     */
    public function is_active_post_services($user_type = null)
    {
        // check pay services exists
        $this->ci->load->model('Services_model');
        $services_params = array(
            'where_in' => array(
                'template_gid' => array('listings_post_template', 'combined_template'),
            ),
            'where' => array(
                'status' => 1,
            ),
        );

        if ($user_type) {
            $criteria = $this->ci->Services_model->getUserTypeCriteria($user_type);
            $services_params = array_merge_recursive($services_params, $criteria);
        }

        $services_count = $this->ci->Services_model->get_service_count($services_params);
        return $services_count > 0;
    }

    /**
     * Return services available by user type
     *
     * @param string $user_type user type
     *
     * @return boolean
     */
    public function get_active_services($user_type)
    {
        $this->ci->load->model("Services_model");

        $services_params = array();

        $services_params["where_in"]["template_gid"] = array(
            "featured_template",
            "show_logo_template",
        );

        $criteria = $this->ci->Services_model->getUserTypeCriteria($user_type);
        $services_params = array_merge_recursive($services_params, $criteria);

        $services_params["where"]["status"] = 1;
        $services = $this->ci->Services_model->get_service_list($services_params);

        return $services;
    }

    /**
     * Return contact services of user as array
     *
     * @param integer $user_id user identifier
     *
     * @return array
     */
    public function getUserContactServices($user_id)
    {
        $params = array(
            'where' => array(
                'id_user' => $user_id,
                'template_gid' => 'contact_template',
                'status' => '1',
            ),
        );

        return $this->get_services_list(null, $params);
    }

    /**
     * Return post services of user as array
     *
     * @param integer $user_id user identifier
     *
     * @return array
     */
    public function getUserPostServices($user_id)
    {
        $params = array(
            'where' => array(
                'id_user' => $user_id,
                'template_gid' => 'listings_post_template',
                'status' => '1',
            ),
        );

        return $this->get_services_list(null, $params);
    }

    /**
     * Render featured users
     * 
     * callback for dynamic blocks
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return void
     */
    public function dynamicBlocksFeaturedUsers($params, $view = 'gallery_middle', $width = 100, $user_type = null)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);

        $this->ci->load->helper('users_services');
        return users_featured($user_type, $params['count'], $size, $view, $width);
    }

    /**
     * Render featured Companies
     * 
     * callback for dynamic blocks
     * 
     * @return void
     */
    public function dynamicBlocksFeaturedCompanies($params, $view = 'gallery_middle', $width = 100)
    {
        return $this->dynamicBlocksFeaturedUsers($params, $view, $width, 'company');
    }

    /**
     * Render featured agents
     * 
     * callback for dynamic blocks
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return void
     */
    public function dynamicBlocksFeaturedAgents($params, $view = 'gallery_middle', $width = 100)
    {
        return $this->dynamicBlocksFeaturedUsers($params, $view, $width, 'agent');
    }

}
