<?php

namespace Pg\Modules\Video_uploads\Models;

/**
 * Video uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Video uploads install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Video uploads
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Video_uploads_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'settings_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'uploads-items' => array(
                            'action' => 'none',
                            'items' => array(
                                'video_menu_item' => array('action' => 'create', 'link' => 'admin/video_uploads', 'status' => 1, 'sorter' => 3),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_video_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - System - Video settings',
            'items' => array(
                'configs_list_item' => array('action' => 'create', 'link' => 'admin/video_uploads', 'status' => 1),
                'system_list_item' => array('action' => 'create', 'link' => 'admin/video_uploads/system_settings', 'status' => 1),
                //'youtube_settings_item' => array('action' => 'create', 'link' => 'admin/video_uploads/youtube_settings', 'status' => 1),
            ),
        ),
    );

    /**
     * Constructor
     *
     * @return Video_uploads_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Install video uploads data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import video uploads languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('video_uploads', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid,
                                              $langs_file);
        }

        return true;
    }

    /**
     * Export video uploads languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall video uploads data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install video uploads data to cronjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        ///// add cronjob ()
        $this->ci->load->model('Cronjob_model');
        $cron_data = array(
            "name" => "Video processing",
            "module" => "video_uploads",
            "model" => "Video_uploads_process_model",
            "method" => "cron_processing_method",
            "cron_tab" => "*/5 * * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);

        $cron_data = array(
            "name" => "Video image cropping",
            "module" => "video_uploads",
            "model" => "Video_uploads_process_model",
            "method" => "cron_images_method",
            "cron_tab" => "*/5 * * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);

        $cron_data = array(
            "name" => "Video check youtube status",
            "module" => "video_uploads",
            "model" => "Video_uploads_process_model",
            "method" => "cron_waiting_method",
            "cron_tab" => "*/5 * * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);
    }

    /**
     * Uninstall video uploads data from cronjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array();
        $cron_data["where"]["module"] = "video_uploads";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install video uploads settings
     *
     * @return void
     */
    public function install_video_uploads()
    {
        //// reculc system settings
        $this->ci->load->model('video_uploads/models/Video_uploads_settings_model');
        $this->ci->Video_uploads_settings_model->reculc_settings();
        $this->ci->Video_uploads_settings_model->write_settings();

        if ((extension_loaded("fileinfo")) && function_exists('finfo_open')) {
            $this->ci->pg_module->set_module_config('video_uploads', 'use_fileinfo', true);
        }
    }

    /**
     * Install data of video uploads module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        
    }

    /**
     * Uninstall data of video uploads module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        
    }

}
