<?php

namespace Pg\Modules\Video_uploads\Models;

/**
 * Video uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Video uploads youtube model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Video uploads
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Video_uploads_youtube_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Settings of video upload
     */
    protected $settings;

    /**
     * Description by default
     *
     * @var array
     */
    protected $defaults = array(
        "name" => 'default name',
        "description" => 'default description',
        "tags" => 'site, files',
    );
    protected $scope = array(
        'UPLOAD' => 'https://www.googleapis.com/auth/youtube.upload',
        'AUTH' => 'https://www.googleapis.com/auth/youtube',
        'PARTNER' => 'https://www.googleapis.com/auth/youtubepartner'
    );

    /**
     * Constructor
     *
     * @return Video_uploads_youtube_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        $this->read_settings();
    }

    /**
     * Return all settings of video uploads module
     *
     * @return void
     */
    public function read_settings()
    {
        $this->settings = $this->ci->pg_module->return_module_all_config('video_uploads');
    }

    /**
     * Return value of video uploads setting or all settings
     *
     * @param string $name setting name
     *
     * @return string
     */
    public function get_settings($name = '')
    {
        return ($name != '') ? $this->settings[$name] : $this->settings;
    }

    /**
     * Set values of all video upload settings
     *
     * @return void
     */
    public function write_settings()
    {
        $this->ci->pg_module->set_module_all_config('video_uploads', $this->settings);
    }

    /**
     * Set value of video upload setting
     *
     * @param string $name  setting name
     * @param string $value setting value
     *
     * @return void
     */
    public function set_settings($name, $value)
    {
        $this->settings[$name] = $value;
    }

    /**
     * Refresh video uploads settings
     *
     * @return array
     */
    public function reculc_permission_settings()
    {
        if (!$this->settings['youtube_converting_source']) {
            $this->settings['youtube_converting_source'] = 'pg core video';
        }
        if (!$this->settings['youtube_converting_login'] || !$this->settings['youtube_converting_password']/* || !$this->settings['youtube_converting_developer_key'] */) {
            $this->settings['use_youtube_converting'] = false;
        }/* else {
          $youtube_auth = $this->youtube_auth(2);

          /*if (!empty($youtube_auth['error']) || empty($youtube_auth['yt'])) {
          $this->settings['use_youtube_converting'] = false;
          } else {
          $this->settings['use_youtube_converting'] = true;
          } */
        /*      } */

        return $this->settings;
    }

    ////// main youtube functions

    /**
     * Authorize to youtube account
     *
     * @return array
     */
    public function youtube_auth($step = 1)
    {
        $return = array('client' => '', 'yt' => '', 'error' => '');
        /*
          $this->settings['youtube_converting_login'] = '606310929562-kd0bb331bpd8ghbool13jcq73pnrengf.apps.googleusercontent.com';
          $this->settings['youtube_converting_password'] = 'BjoadDeMIHCo7GEG06ISidqx';
         */
        $client = new Google_Client;
        $client->setClientId($this->settings['youtube_converting_login']);
        $client->setClientSecret($this->settings['youtube_converting_password']);

        $client->setScopes($this->scope['AUTH']);
        $client->setAccessType('offline');
        $client->setRedirectUri(site_url());
        $client->setApprovalPrompt('force');

        if (isset($_GET['code'])) {
            $client->authenticate($_GET['code'] . '#');
            $this->settings['youtube_access_token'] = $client->getAccessToken();
        }

        if (isset($this->settings['youtube_access_token'])) {
            $client->setAccessToken($this->settings['youtube_access_token']);
        }

        if (!$client->getAccessToken()) {
            $state = '/admin/video_uploads/youtube_settings_check';
            $client->setState($state);

            $authUrl = $client->createAuthUrl();
            redirect($authUrl);
        }

        return $return;
    }

    public function test($file_name, $file_path, $file_data)
    {
        $this->youtube_upload_video($file_name, $file_path, $file_data);
    }

    /**
     * Upload video to youtube account
     *
     * @param string $file_name name of video file
     * @param string $file_path path to video file
     * @param array  $file_data video data
     *
     * @return array
     */
    protected function youtube_upload_video($file_name, $file_path, $file_data)
    {
        $return = array("errors" => array(), "data" => array());

        try {
            $client = new Google_Client;
            $client->setClientId($this->settings['youtube_converting_login']);
            $client->setClientSecret($this->settings['youtube_converting_password']);
            $client->setScopes($this->scope);
            $client->setAccessType('offline');
            $client->setAccessToken($this->settings['youtube_access_token']);

            if ($client->getAccessToken()) {
                if ($client->isAccessTokenExpired()) {
                    $newToken = json_decode($client->getAccessToken());
                    $client->refreshToken($newToken->refresh_token);
                    $this->settings['youtube_access_token'] = $client->getAccessToken();
                }

                $youtube = new Google_Service_YouTube($client);

                // REPLACE this value with the path to the file you are uploading.
                $videoPath = $file_path . $file_name;

                // Create a snippet with title, description, tags and category ID
                // Create an asset resource and set its snippet metadata and type.
                // This example sets the video's title, description, keyword tags, and
                // video category.
                $snippet = new Google_Service_YouTube_VideoSnippet();
                $snippet->setTitle(!empty($file_data["name"]) ? $file_data["name"] : $this->defaults['name']);
                $snippet->setDescription(!empty($file_data["description"]) ? $file_data["description"] : $this->defaults['description']);
                $snippet->setTags(!empty($file_data["tags"]) ? $file_data["tags"] : $this->defaults['tags']);

                // Numeric video category. See
                // https://developers.google.com/youtube/v3/docs/videoCategories/list 
                // Howto & Style
                $snippet->setCategoryId("26");

                // Set the video's status to "public". Valid statuses are "public",
                // "private" and "unlisted".
                $status = new Google_Service_YouTube_VideoStatus();
                $status->privacyStatus = "public";

                // Associate the snippet and status objects with a new video resource.
                $video = new Google_Service_YouTube_Video();
                $video->setSnippet($snippet);
                $video->setStatus($status);

                // Specify the size of each chunk of data, in bytes. Set a higher value for
                // reliable connection as fewer chunks lead to faster uploads. Set a lower
                // value for better recovery on less reliable connections.
                $chunkSizeBytes = 1 * 1024 * 1024;

                // Setting the defer flag to true tells the client to return a request which can be called
                // with ->execute(); instead of making the API call immediately.
                $client->setDefer(true);

                // Create a request for the API's videos.insert method to create and upload the video.
                $insertRequest = $youtube->videos->insert("status,snippet", $video);

                // Create a MediaFileUpload object for resumable uploads.
                $media = new Google_Http_MediaFileUpload(
                        $client, $insertRequest, $file_data["type"], null, true, $chunkSizeBytes
                );

                $media->setFileSize(filesize($videoPath));

                // Read the media file and upload it chunk by chunk.
                $status = false;
                $handle = fopen($videoPath, "rb");
                while (!$status && !feof($handle)) {
                    $chunk = fread($handle, $chunkSizeBytes);
                    $status = $media->nextChunk($chunk);
                }

                fclose($handle);

                if ($status->status['uploadStatus'] == 'uploaded') {
                    $return["data"]["video"] = $status['id'];
                } else {
                    $return["errors"][] = 'upload error';
                }

                // If you want to make other calls after the file upload, set setDefer back to false
                $client->setDefer(true);
            }
        } catch (Google_Service_Exception $e) {
            $return["errors"][] = 'upload error' . $e->getMessage();
        } catch (Google_Exception $e) {
            $return["errors"][] = 'upload error' . $e->getMessage();
        }

        return $return;
    }

    /**
     * Return video information from yuotube
     *
     * @param integer $video_id video identifier
     *
     * @return array
     */
    public function youtube_get_info($video_id)
    {
        $return = array("errors" => array(), "data" => array());

        try {
            $client = new Google_Client;
            $client->setClientId($this->settings['youtube_converting_login']);
            $client->setClientSecret($this->settings['youtube_converting_password']);
            $client->setScopes($this->scope);
            $client->setAccessType('offline');
            $client->setAccessToken($this->settings['youtube_access_token']);
            $youtube = new Google_Service_YouTube($client);

            if ($client->getAccessToken()) {
                if ($client->isAccessTokenExpired()) {
                    $newToken = json_decode($client->getAccessToken());
                    $client->getRefreshToken($newToken->refresh_token);
                    $this->settings['youtube_access_token'] = $client->getAccessToken();
                    $client->setAccessToken($this->settings['youtube_access_token']);
                }

                // получить видео по ID
                $getSnippet = $youtube->videos->listVideos("snippet, player, statistics, contentDetails, recordingDetails, statistics",
                                                           array('id' => $video_id));
            }
            if (isset($getSnippet['modelData']['items'][0]['snippet']['title'])) {
                $return['data']['title'] = $getSnippet['modelData']['items'][0]['snippet']['title'];
            }
            if (isset($getSnippet['modelData']['items'][0]['snippet']['description'])) {
                $return['data']['description'] = $getSnippet['modelData']['items'][0]['snippet']['description'];
            }
            if (isset($getSnippet['modelData']['items'][0]['snippet']['categoryId'])) {
                $return['data']['category'] = $getSnippet['modelData']['items'][0]['snippet']['categoryId'];
            }
            if (isset($getSnippet['modelData']['items'][0]['player']['embedHtml'])) {
                $return['data']['flash_player_url'] = $getSnippet['modelData']['items'][0]['player']['embedHtml'];
            }
            if (isset($getSnippet['modelData']['items'][0]['contentDetails']['duration'])) {
                $return['data']['duration'] = $getSnippet['modelData']['items'][0]['contentDetails']['duration'];
            }
            if (isset($getSnippet['modelData']['items'][0]['statistics']['viewCount'])) {
                $return['data']['view_count'] = $getSnippet['modelData']['items'][0]['statistics']['viewCount'];
            }
            if (isset($getSnippet['modelData']['items'][0]['statistics']['likeCount'])) {
                $return['data']['rating'] = $getSnippet['modelData']['items'][0]['statistics']['likeCount'];
            }
            if (isset($getSnippet['modelData']['items'][0]['recordingDetails']['location']['latitude'])) {
                $return['data']['geo']['lat'] = $getSnippet['modelData']['items'][0]['recordingDetails']['location']['latitude'];
            }
            if (isset($getSnippet['modelData']['items'][0]['recordingDetails']['location']['longitude'])) {
                $return['data']['geo']['lon'] = $getSnippet['modelData']['items'][0]['recordingDetails']['location']['longitude'];
            }
            if (isset($getSnippet['modelData']['items'][0]['snippet']['thumbnails']['medium']['url'])) {
                $return['data']['thumbs']['medium']['url'] = $getSnippet['modelData']['items'][0]['snippet']['thumbnails']['medium']['url'];
            }
            if (isset($getSnippet['modelData']['items'][0]['snippet']['thumbnails']['medium']['width'])) {
                $return['data']['thumbs']['medium']['width'] = $getSnippet['modelData']['items'][0]['snippet']['thumbnails']['medium']['width'];
            }
            if (isset($getSnippet['modelData']['items'][0]['snippet']['thumbnails']['medium']['height'])) {
                $return['data']['thumbs']['medium']['height'] = $getSnippet['modelData']['items'][0]['snippet']['thumbnails']['medium']['height'];
            }
        } catch (Google_Service_Exception $e) {
            $return["errors"][] = 'upload error' . $e->getMessage();
        } catch (Google_Exception $e) {
            $return["errors"][] = 'upload error' . $e->getMessage();
        }

        return $return;
    }

    ///// methods for proccess model

    /**
     * Upload video to youtube account
     *
     * @param string $file_name name of video file
     * @param string $file_path path to video file
     * @param array  $file_data video data
     * @param array  $config    configuration gid
     *
     * @return array
     */
    public function processing_method($file_name, $file_path, $file_data, $config)
    {
        return $this->youtube_upload_video($file_name, $file_path, $file_data);
    }

    /**
     * Create thumbnails for video object
     *
     * @param string $file_name name of video file
     * @param string $file_path path to video file
     *
     * @return array
     */
    public function images_method($file_name, $file_path)
    {
        return $this->waiting_method($file_name, $file_path);
    }

    /**
     * Process thumbnails of video object
     *
     * @param string $file_name name of video file
     * @param string $file_path path to video file
     *
     * @return array
     */
    public function waiting_method($file_name, $file_path)
    {
        $return = array("errors" => array(), "data" => array());

        $youtube_data = $this->youtube_get_info($file_name);
        if (!empty($youtube_data["errors"])) {
            $return['errors'] = $youtube_data["errors"];

            return $return;
        }

        if (!empty($youtube_data["data"]["thumbs"][0]) && $youtube_data["data"]["thumbs"][0]["time"] != '00:00:00') {
            $return["data"]['image'] = $this->save_ext_image($file_path, $youtube_data["data"]["thumbs"][0]["url"]);
        }

        return $return;
    }

    /**
     * Remove video object by identifier
     *
     * @param integer $video_id video identifier
     *
     * @return array
     */
    public function delete_method($video_id)
    {
        $return = array("errors" => array(), "data" => array());

        try {
            $client = new Google_Client;
            $client->setClientId($this->settings['youtube_converting_login']);
            $client->setClientSecret($this->settings['youtube_converting_password']);
            $client->setScopes($this->scope);
            $client->setAccessType('offline');
            $client->setAccessToken($this->settings['youtube_access_token']);

            if ($client->getAccessToken()) {
                if ($client->isAccessTokenExpired()) {
                    $newToken = json_decode($client->getAccessToken());
                    $client->refreshToken($newToken->refresh_token);
                    $this->settings['youtube_access_token'] = $client->getAccessToken();
                }

                $youtube = new Google_Service_YouTube($client);
                $youtube->videos->delete($video_id);
            }
        } catch (Google_Service_Exception $e) {
            $return["errors"][] = 'delete error';
        } catch (Google_Exception $e) {
            $return["errors"][] = 'delete error';
        }

        return $return;
    }

    /**
     * Save image from url to local
     *
     * @param string $file_path path for saving
     * @param string $url       url of image
     *
     * @return string
     */
    protected function save_ext_image($file_path, $url)
    {
        $path_parts = pathinfo($url);
        $file_name = $path_parts["basename"];
        $content = file_get_contents($url);
        $f = fopen($file_path . $file_name, 'w');
        if ($f) {
            fwrite($f, $content);
            fclose($f);
        }

        return $file_name;
    }

    /**
     * Return embed code of video object by default
     *
     * @param integer $id     video identifier
     * @param integer $width  video width
     * @param integer $height video height
     *
     * @return string
     */
    public function get_default_embed($id, $width = 480, $height = 360)
    {
        return '<iframe width="' . $width . '" height="' . $height . '" src="http://www.youtube.com/embed/' . $id . '" frameborder="0" allowfullscreen></iframe>';
    }

}
